package com.esd.ucm.handlers;

/**
 * @author SLiu
 *
 */
import java.util.Properties;
import com.esd.ucm.util.UCMTextUtil;

import toolbox.db.*;
import java.io.*;

public class UCMOutputFilterHandler implements IDataHandler
{
	final String DELIMITER = "delimiter";
	final String STRING_QUALIFIER = "string-qualifier";
	final String INDEX = "index";
	Properties properties = null;
	SectionSpec sectionSpec = null;
	FieldSpec[] fieldSpecs = null;
	private java.io.PrintWriter writer = null;
	private char delimiter = ',';
	private String qualifier = null;
	private UCMTextUtil textUtil = null;
	private int[] indices = null;
	
	
	/**
	 * Constructor for UCMOutputFilterHandler.
	 */
	public UCMOutputFilterHandler() {
		super();
		properties = new Properties();
	}

	/**
	 * @see toolbox.db.IDataHandler#addAllProperties(java.util.Properties)
	 */
	public void addAllProperties(Properties arg0) {
		properties.putAll(arg0);
	}

	/**
	 * @see toolbox.db.IDataHandler#close()
	 */
	public void close() {
		try{			
			writer.close();
		}catch(Exception ex){
			throw new RuntimeException(ex);
		}
	}


	/**
	 * @see toolbox.db.IDataHandler#handleHeader(DataRow)
	 */
	public void handleHeader(DataRow row) 
	{
		handleContent(row); 
	}

	/**
	 * @see toolbox.db.IDataHandler#handleContent(DataRow)
	 */
	public void handleContent(DataRow row) 
	{
		
		try{
			String[] dataElements = row.getDataElements();
			if(dataElements==null){
				writer.println(row.getRawData());
			}else{			
				writer.println(textUtil.join(row.getDataElements(), this.delimiter, this.fieldSpecs, this.indices));
			}
			
			writer.flush();
		}catch(Exception ex){
			throw new RuntimeException(ex);
		}
	}

	/**
	 * @see toolbox.db.IDataHandler#handleTrailer(DataRow)
	 */
	public void handleTrailer(DataRow row) 
	{
		handleContent(row); 
	}


	/**
	 * @see toolbox.db.IDataHandler#handleException(java.lang.Throwable)
	 */
	public void handleException(Throwable arg0) {
	}

	
	/**
	 * @see toolbox.db.IDataHandler#handleWarning(java.lang.String, java.lang.Throwable)
	 */
	public void handleWarning(String arg0, Throwable arg1) {
	}

	/**
	 * @see toolbox.db.IDataHandler#init()
	 */
	public void init() 
	{		
		String indexStr = properties.getProperty(INDEX);		
		try{
			String[] indexStrs = indexStr.split(",");
			indices = new int[indexStrs.length];			
			for(int i=0; i<indexStrs.length; i++)
			{				
				indices[i] = Integer.valueOf(indexStrs[i]).intValue();
			}			
		}catch(Exception ex){
			throw new RuntimeException(ex);
		}
		indexStr = properties.getProperty(DELIMITER);
		if(indexStr!=null)	delimiter = indexStr.charAt(0);
		
		indexStr = properties.getProperty(STRING_QUALIFIER);
		if(indexStr!=null)	qualifier = indexStr;	
		
		if (textUtil==null)
		{
			textUtil = new UCMTextUtil();
		}
		textUtil.setStringQuaqlifier(qualifier);
		
		if (writer==null)
		try{
			writer = new PrintWriter(new FileWriter(properties.getProperty("out")));
		}catch(Exception ex){
			throw new RuntimeException(ex);
		}	
	}

	/**
	 * @see toolbox.db.IDataHandler#setSectionSpec(toolbox.db.SectionSpec)
	 */
	public void setSectionSpec(SectionSpec arg0) 
	{
		this.sectionSpec = arg0;
		fieldSpecs = sectionSpec.getFieldSpecs();
	}

}
