package com.esd.ucm.util;

import java.io.FileInputStream;
import java.util.Properties;
import java.util.Map;
import java.util.logging.*;

import org.apache.poi.hssf.model.*;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.hssf.*;

import com.gonetix.properties.RegExProperties;


/**
 * @author SLiu
 *
 */
public class Cusip2IsinMap extends RegExProperties 
{

	private Logger logger = Logger.getLogger("Cusip2Isin");
	
	public void loadFromXls(String systemId)
	{
		try{
			POIFSFileSystem fs  = new POIFSFileSystem(new FileInputStream(systemId));
    		HSSFWorkbook wb = new HSSFWorkbook(fs);
    		HSSFSheet sheet = wb.getSheetAt(0);
			int start = sheet.getFirstRowNum();
			int end = sheet.getLastRowNum();
			for(int i=start; i<=end; i++)
			{				
				HSSFRow row = sheet.getRow(i);
    			put(row);
			}			
			
		}catch(Exception ex){
			throw new RuntimeException(ex);	
		}
	}
	
	
	private void put(HSSFRow row)
	{
		try{			
    		String cusip = getCellValueAsString(row.getCell((short)0));
    		String isin = getCellValueAsString(row.getCell((short)1)); 
    		if (cusip!=null &&  isin!=null){
    			if (isin.equalsIgnoreCase("NULL")) isin = cusip;
    			this.setProperty(cusip, isin);
    		}    		
		}catch(Exception ex){
			logger.log(Level.WARNING, "Error at row number [" + row.getRowNum() + "]");			
		}
	}
	
	private String getCellValueAsString(HSSFCell cell)
	{
		String ret = null;
		switch (cell.getCellType())
		{
			case HSSFCell.CELL_TYPE_NUMERIC:
				ret = String.valueOf(cell.getNumericCellValue());
			break;
			case HSSFCell.CELL_TYPE_STRING:			
				ret = cell.getStringCellValue();
			break;			
		}		
		return ret;		
	}


}
