package com.esd.util;

import java.io.*;
import java.util.zip.*;
import java.util.regex.*;

import com.gonetix.util.CmdLineOptions;


/**
 * @author SLiu
 *
 */
public final class FileUtil
{

	public static boolean append(File source, File target) throws IOException
	{
		boolean su = false;
		FileInputStream fin = null;
		FileOutputStream fout = null;
		try{
			byte[] buffer = new byte[1024];
			fin = new FileInputStream(source);
			fout = new FileOutputStream(target, true);
			int len = 0;
			while ((len=fin.read(buffer))>0)
			{
				fout.write(buffer, 0, len);
			}
			su = true;
		}finally{
				if (fin!=null) fin.close();
				if (fout!=null) fout.close();
		}
		return su;
	}

	public static boolean copy(File source, File target, boolean overwriteTarget)  throws IOException
	{
		boolean su = false;
		if (!overwriteTarget && target.exists()) return su;
		FileInputStream fin = null;
		FileOutputStream fout = null;
		try{
			byte[] buffer = new byte[1024];
			fin = new FileInputStream(source);
			fout = new FileOutputStream(target);
			int len = 0;
			while ((len=fin.read(buffer))>0)
			{
				fout.write(buffer, 0, len);
				fout.flush();
			}
			su = true;
		}finally{
				if (fin!=null) fin.close();
				if (fout!=null) fout.close();
		}
		return su;
	}


	public static void addToZip(String zipFile, String filename) throws IOException
	{
		File zf = new File(zipFile);
		File nzf = new File("." + zipFile + ".temp");
		nzf.deleteOnExit();
		move(zf, nzf, true);
		ZipFile zfile = new ZipFile(nzf);
		ZipOutputStream zipOut = null;
		zipOut = new ZipOutputStream(new FileOutputStream(zipFile));
		java.util.Enumeration enum = zfile.entries();
		byte[] buffer = new byte[512];
		while (enum.hasMoreElements())
		{
			ZipEntry zentry = (ZipEntry)enum.nextElement();
			InputStream zein = zfile.getInputStream(zentry);
			zipOut.putNextEntry(zentry);
			int len=0;
		    while((len=zein.read(buffer)) != -1)
		    {
		       zipOut.write(buffer,0,len);
		    }
		    zein.close();
		    zipOut.closeEntry();
		}
		zfile.close();
		archive(zipOut, new File[]{new File(filename)}, null);
		zipOut.close();
	}


	public static String[] listFromDirectory(File directory, String mask)
	{
		return directory.list(new FileNameMaskFilter(mask));
	}

	public static File[] listFilesFromDirectory(File directory, String mask)
	{
		return directory.listFiles(new FileNameMaskFilter(mask));
	}

	public static void archive(String zipfile, String filename) throws IOException
	{
		archive(zipfile, new File(filename));
	}

	public static void archive(String zipfile, File file) throws IOException
	{
		archive(zipfile, new File[]{file} , null);
	}

	public static void archive(String zipfile, File[] files, String parent) throws IOException
	{
		ZipOutputStream zipOut = null;
		try{
			zipOut = new ZipOutputStream(new FileOutputStream(zipfile));
			archive(zipOut, files, parent);
		}finally{
			if (zipOut != null) zipOut.close();
		}
	}

	protected static void archive(ZipOutputStream zipOut, File[] files, String parent)  throws IOException
	{
		parent = ( parent==null)?"":(parent+File.separator);
		FileInputStream fin = null;
		byte[] buffer = new byte[512];
		for(int i=0; i<files.length; i++)
		{
			if (files[i].exists()==false) continue;
			String entryName = parent + files[i].getName();
			if (files[i].isFile())
			{
				try{
					fin = new FileInputStream(files[i]);
					ZipEntry zentry = new ZipEntry(entryName);
					zipOut.putNextEntry(zentry);
					int len=0;
				    while((len=fin.read(buffer)) != -1)
				    {
				       zipOut.write(buffer,0,len);
				    }
				    zipOut.closeEntry();
				}finally{
					if (fin != null) fin.close();
				}
			}else{
				File[] fs = files[i].listFiles(new FileNameMaskFilter("*"));
				archive(zipOut, fs, entryName);
			}
		}
	}

	public static boolean copy(File source, File target) throws IOException
	{
		return copy(source, target, true);
	}

	public static boolean move(File source, File target, boolean overwriteTarget) throws IOException
	{
		boolean su = copy(source, target, overwriteTarget);
		if (su)
		{
			source.delete();
		}
		return su;
	}

	public static boolean move(File source, File target) throws IOException
	{
		return move(source, target, true);
	}

        /**
         * Deletes a filename.
         * @param filename
         */
        public static void delete(String filename) {
            File f = new File(filename);
            f.delete();
        }

        /**
         * Returns the extension of a filename. If the filename has no '.',
         * returns null.
         */
        public static String getExtention(String fileName) {
            int n = fileName.lastIndexOf('.');
            if (n == -1) {
                return (null);
            } else {
                return (fileName.substring(n+1));
            }
        }

        /**
         * Returns a filename with the extension changed.
         */
        public static String changeExtension(String fileName, String extension) {
            int n = fileName.lastIndexOf('.');
            if (n == -1) {
                return (fileName + "." + extension);
            } else {
                return (fileName.substring(0, n) + "." + extension);
            }
        }

	public static void main(String[] args) throws Exception
	{
		CmdLineOptions cmdOptions = CmdLineOptions.getCmdLineOptions(args);
		if (cmdOptions.getOption("z")!=null || cmdOptions.getOption("Z")!=null)
		{
			archive(args[1], args[2]);
		}else if (cmdOptions.getOption("za")!=null || cmdOptions.getOption("Za")!=null)
		{
			addToZip(args[1], args[2]);
		}
	}


}

final class FileNameMaskFilter implements java.io.FilenameFilter
{
    private String mask = null;
    /** Creates a new instance of FileMaskFilter */
    public FileNameMaskFilter(String mask1)
    {

        StringBuffer buffer = new StringBuffer(128);
        for(int i=0; i<mask1.length(); i++){
            char c = mask1.charAt(i);
            if ( c=='*' ) buffer.append(".*");
            else if ( c=='?') buffer.append(".");
            else if ( c=='.') buffer.append("\\.");
            else buffer.append(c);
        }
        this.mask = buffer.toString();
    }

    public boolean accept(java.io.File file, String str)
    {
        return Pattern.matches(mask, str);
    }

    public String toString()
    {
        return mask;
    }

}
