package com.esdi.sql;


import java.util.ArrayList;
import java.sql.*;

import tudor.sql.util.TuSQLMetaData;


/**

 Tudor Abstract Class for dynamic SQL statements. Copied from tudor.sql.util.TuSQL
 to add a "default" attribute to the set operations to indicate columns that have
 default (non null) values that should not be changed via an undate statement.

 @version 1.1 12/06/1999
 @author Randall Richard
 @author Gene Dubetskiy
 @author Bill Kelemen

 </p>
 Stores and dynamicly generates SQL statements
 */

public abstract class ESDSQL extends Object {
    protected ArrayList m_setColumns;
    protected TuSQLMetaData m_tusqlmd;
    protected String[] pKey;

    public ESDSQL(TuSQLMetaData md) {
        this.m_tusqlmd = md;
        this.m_setColumns = new ArrayList();
    }

    /**
     Clears all elements in the Column list
     */
    public void clear() {
        this.m_setColumns.clear();
    }

    /**
     Set Column as a primary key for WHERE clause
     setSQLKey("order_id") would create " WHERE  order_id = [value] "
     */
    public void setSQLKey(String[] pKey) throws Exception {
        this.m_tusqlmd.SetPrivateField(pKey);
        this.pKey = pKey;
    }

    public void setSQLKey(String pKey) throws Exception {
        setSQLKey(new String[] {pKey});
    }

    /**
     Replaces/Creates the element at the specified column with the specified element
     Throws exception if column does not belong to the table
     */

    // set's with only two parametes (isDefault = false)

    public void set(String column, Integer value) throws Exception {
        set(column, (Object) value, false);
    }

    public void set(String column, Double value) throws Exception {
        set(column, (Object) value, false);
    }

    public void set(String column, String value) throws Exception {
        set(column, (Object) value, false);
    }

    public void set(String column, Timestamp value) throws Exception {
        set(column, (Object) value, false);
    }

    // set's with third parameter isDefault

    public void set(String column, Integer value, boolean isDefault) throws Exception {
        set(column, (Object) value, isDefault);
    }

    public void set(String column, Double value, boolean isDefault) throws Exception {
        set(column, (Object) value, isDefault);
    }

    public void set(String column, String value, boolean isDefault) throws Exception {
        set(column, (Object) value, isDefault);
    }

    public void set(String column, Timestamp value, boolean isDefault) throws Exception {
        set(column, (Object) value, isDefault);
    }

    // generic set method

    protected void set(String column, Object value, boolean isDefault) throws Exception {
        ESDSQLColumn SQLColumn = null;

        String className = value.getClass().getName();
        //    String columnType = this.m_tusqlmd.getColumnType(column);
        Class columnType = this.m_tusqlmd.getColumnType(column);

        if (className == null) {
            throw new Exception(className + " - unsupported...");
        }

        //    if(className.compareTo(columnType)!=0)  {
        if (columnType.isInstance(value) == false) {
            throw new Exception(className + " - type missmatch for column " + column +
                "; expected " + columnType);
        }

        int fOffset = this.m_tusqlmd.GetFieldOffset(column);

        if (fOffset < 0) {
            throw new Exception(column + " - unknown column.  sp_help might help");
        }
        for (int i = 0; i < this.m_setColumns.size(); i++) {
            SQLColumn = (ESDSQLColumn) this.m_setColumns.get(i);
            if (SQLColumn.getName().compareTo(column) == 0) {
                break;
            }
            SQLColumn = null;
        }
        if (SQLColumn == null) {
            SQLColumn = new ESDSQLColumn(column, value);
            SQLColumn.setColumnIndex(fOffset);
            this.m_setColumns.add(SQLColumn);
        } else {
            SQLColumn.setValue(value);
        }
        SQLColumn.isDefault = isDefault;
    }

    /**
     Returns element for specified column.
     Returns null if column was not set.
     */
    public Object get(String column) {
        ESDSQLColumn c = this.getColumnByName(column);

        if (c != null) {
            return c.getValue();
        }
        return null;
    }

    protected ESDSQLColumn getColumnByName(String column) {
        int n = this.m_setColumns.size();
        ESDSQLColumn c;

        for (int i = 0; i < n; i++) {
            c = (ESDSQLColumn) this.m_setColumns.get(i);
            if (c.getName().compareTo(column) == 0) {
                return c;
            }
        }
        return null;
    }

    public void set(String column, int v) throws Exception {
        set(column, new Integer(v));
    }

    public void set(String column, double v) throws Exception {
        set(column, new Double(v));
    }

    public void set(String column, int v, boolean isDefault) throws Exception {
        set(column, new Integer(v), isDefault);
    }

    public void set(String column, double v, boolean isDefault) throws Exception {
        set(column, new Double(v), isDefault);
    }

    protected void setPlus(String column, Object v) throws Exception {
        ESDSQLColumn c = this.getColumnByName(column);

        if (c != null) {
            c.setValue(v);
            c.isPlus = true;
            return;
        }
        throw new Exception(column + " - unknown column.  sp_help might help");
    }

    public void setPlus(String column, Timestamp v) throws Exception {
        setPlus(column, (Object) v);
    }

    public void setPlus(String column, int v) throws Exception {
        setPlus(column, new Integer(v));
    }

    public void setPlus(String column, double v) throws Exception {
        setPlus(column, new Double(v));
    }

    public void setExpression(String column, String value) throws Exception {
        ESDSQLColumn c = this.getColumnByName(column);

        if (c != null) {
            c.setValue(value);
            c.isExpression = true;
            return;
        } else {
            //set(column, (Object) "");
            c = new ESDSQLColumn(column, value);
            int fOffset = this.m_tusqlmd.GetFieldOffset(column);

            if (fOffset < 0) {
                throw new Exception(column + " - unknown column.  sp_help might help");
            }
            c.setColumnIndex(fOffset);
            this.m_setColumns.add(c);
            c.isExpression = true;
            return;
        }
    }

    abstract public String getSQL() throws Exception;
}
