package com.esdi.sql;


import tudor.sql.util.*;

import java.util.Hashtable;

import java.sql.Connection;


/**
 * Extends TuSQLFactory to provide a TuSQL enhanced object for insert and update
 * operations.
 */
public class ESDSQLFactory {

    static Hashtable metaDataHash = new Hashtable();

    static TuSQLMetaData getMetaData(Connection c, String db, String table) {
        String key = db + "," + table;
        TuSQLMetaData tsmd;

        if ((tsmd = (TuSQLMetaData) metaDataHash.get(key)) == null) {
            tsmd = new TuSQLMetaData(c, db, table);
            metaDataHash.put(key, tsmd);
        }
        return (tsmd);
    }

    /**
     * Creates an Insert and Update object. The default getSQL() method returns
     * an Update and Insert statement in addition to separate insert and update
     * SQL statement methods.
     */
    static public ESDSQLInsertUpdate createInsertUpdate(Connection c, String db, String table, String[] key) throws Exception {
        TuSQLMetaData tsmd = getMetaData(c, db, table);
        ESDSQLInsertUpdate SQLobj = new ESDSQLInsertUpdate(tsmd);

        SQLobj.setSQLKey(key);
        return SQLobj;
    }

    static public ESDSQLInsertUpdate createInsertUpdate(Connection c, String db, String table, String key) throws Exception {
        return createInsertUpdate(c, db, table, new String[] {key}
            );
    }

    /**
     Creates Insert object
     */
    static public TuSQL createInsert(Connection c, String db, String table) {
        TuSQLMetaData tsmd = getMetaData(c, db, table);

        return (new TuSQLInsert(tsmd));
    }

    /**
     Creates Update object
     */
    static public TuSQL createUpdate(Connection c, String db, String table, String[] key) throws Exception {
        TuSQLMetaData tsmd = getMetaData(c, db, table);
        TuSQL SQLobj = new TuSQLUpdate(tsmd);

        SQLobj.setSQLKey(key);
        return SQLobj;
    }

    static public TuSQL createUpdate(Connection c, String db, String table, String key) throws Exception {
        return createUpdate(c, db, table, new String[] {key}
            );
    }

    /**
     Creates Update/Insert object
     */
    static public TuSQL createUpdateInsert(Connection c, String db, String table, String[] key) throws Exception {
        TuSQLMetaData tsmd = getMetaData(c, db, table);
        TuSQL SQLobj = new TuSQLUpdateInsert(tsmd);

        SQLobj.setSQLKey(key);
        return SQLobj;
    }

    static public TuSQL createUpdateInsert(Connection c, String db, String table, String key) throws Exception {
        return createUpdateInsert(c, db, table, new String[] {key}
            );
    }

    /**
     Creates Delete object
     */
    static public TuSQL createDelete(Connection c, String db, String table, String[] key) throws Exception {
        TuSQLMetaData tsmd = getMetaData(c, db, table);
        TuSQL SQLobj = new TuSQLDelete(tsmd);

        SQLobj.setSQLKey(key);
        return SQLobj;
    }

    static public TuSQL createDelete(Connection c, String db, String table, String key) throws Exception {
        return createDelete(c, db, table, new String[] {key}
            );
    }

    static public void setTableKey(String db, String table) {};

}


class TuSQLInsert extends TuSQL {
    TuSQLInsert(TuSQLMetaData md) {
        super(md);
    }

    public String getSQL() throws Exception {
        return this.m_tusqlmd.GetInsertStatement(this.m_setColumns);
    }
}


class TuSQLDelete extends TuSQL {
    TuSQLDelete(TuSQLMetaData md) {
        super(md);
    }

    public String getSQL() throws Exception {
        return this.m_tusqlmd.GetDeleteStatement(this.m_setColumns);
    }
}


class TuSQLUpdate extends TuSQL {
    TuSQLUpdate(TuSQLMetaData md) {
        super(md);
    }

    public String getSQL() throws Exception {
        return this.m_tusqlmd.GetUpdateStatement(this.m_setColumns);
    }
}


class TuSQLUpdateInsert extends TuSQL {
    TuSQLUpdateInsert(TuSQLMetaData md) {
        super(md);
    }

    public String getSQL() throws Exception {
        return this.m_tusqlmd.GetUpdateStatement(this.m_setColumns) +
            " if @@rowcount=0 " + this.m_tusqlmd.GetInsertStatement(this.m_setColumns);
    }
}

