package com.esdi.sql;


import java.util.ArrayList;
import java.util.Iterator;

import tudor.sql.util.TuSQLMetaData;


/**
 * Extends Tudor TuSQL abstract class to provide an Insert and Update
 * object. This provides a getSQL() method compatible with the ESDSQLUpdateInsert
 * defined in ESDSQLFactory (that performs an update and if no rows insert)
 * as well as individual insert and update SQL operations.
 */
public class ESDSQLInsertUpdate extends ESDSQL {

    public ESDSQLInsertUpdate(TuSQLMetaData md) {
        super(md);
    }

    /**
     * Makes a combined SQL statement with an UPDATE followed by an INSERT if
     * no rows were updated in the first part. Only fields with isDefault = true
     * are included in the update statement, although all of them are included
     * in the insert statement. This prevents the process from overwriting
     * user-changed values with default (non-null) values.
     * @return The SQL statement
     * @throws Exception
     */
    public String getSQL() throws Exception {
        String updateStmt = getSQLUpdate();
        if (updateStmt != null) {
            return (getSQLUpdate()
                    + " if @@rowcount=0 "
                    + getSQLInsert());
        } else {
            return (getSQLInsert());
        }
    }

    /**
     * Makes a SQL INSERT statement with all the fields, default and non-default.
     * @return The SQL statement
     * @throws Exception
     */
    public String getSQLInsert() throws Exception {
        return (this.m_tusqlmd.GetInsertStatement(this.m_setColumns));
    }

    /**
     * Makes a SQL UPDATE statement for this TuSQL table. Only fields with
     * isDefault = true are included in the update statement. This prevents
     * the process from overwriting user-changed values with default (non-null)
     * values.
     * @return The SQL statement
     * @throws Exception
     */
    public String getSQLUpdate() throws Exception {
        // make list of non-default columns
        ArrayList nonDefaultColumns = new ArrayList();
        boolean allKeys = true;

        for (int i = 0; i < this.m_setColumns.size(); i++) {
            ESDSQLColumn column = (ESDSQLColumn) this.m_setColumns.get(i);

            if (!column.isDefault) {
                nonDefaultColumns.add(column);
                allKeys &= (isPrimaryKey(column.getName()));
            }
        }

        if (nonDefaultColumns.size() == 0 || allKeys) {
            return (null);
        } else {
            // make update statement with non-default columns
            return (this.m_tusqlmd.GetUpdateStatement(nonDefaultColumns));
        }
    }

    /**
     * Returns true if col is a primary key for this ESDSQL object.
     * @param col
     * @return
     */
    private boolean isPrimaryKey(String col) {
        for (int i=0; i<pKey.length; i++) {
            if (pKey[i].equals(col)) {
                return (true);
            }
        }
        return (false);
    }

}
