package com.esdi.test;

import junit.framework.*;

import java.util.*;
import java.lang.reflect.*;
import java.io.*;

/**
 * Just like a TestSuite but only tests leaf level testMethods, not inherited
 * ones.  Blame it on the private modifier
 */
public class LeafTestSuite
extends TestSuite {

    /**
     * Constructs an empty LeafTestSuite.
     */
    public LeafTestSuite() {
    }

    /**
     * Constructs a TestSuite from the given class. Adds all the methods
     * starting with "test" as test cases to the suite.
     * Parts of this method was written at 2337 meters in the Hffihtte,
     * Kanton Uri
     */

    public LeafTestSuite(final Class theClass) {
        setName(theClass.getName());
        Constructor constructor = null;
        try {
            constructor = getConstructor(theClass);
        } catch (NoSuchMethodException e) {
            addTest(warning("Class " + theClass.getName() + " has no public constructor TestCase(String name)"));
            return;
        }

        if (!Modifier.isPublic(theClass.getModifiers())) {
            addTest(warning("Class " + theClass.getName() + " is not public"));
            return;
        }

        Class superClass = theClass;
        Vector names = new Vector();
        Method[] methods = superClass.getDeclaredMethods();
        for (int i = 0; i < methods.length; i++) {
            addTestMethod(methods[i], names, constructor);
        }
        if (testCount() == 0)
            addTest(warning("No tests found in " + theClass.getName()));
    }

    /**
     * Constructs an empty TestSuite.
     */
    public LeafTestSuite(String name) {
        setName(name);
    }

    protected void addTestMethod(Method m, Vector names, Constructor constructor) {
        String name = m.getName();
        if (names.contains(name))
            return;
        if (isPublicTestMethod(m)) {
            names.addElement(name);

            Object[] args = new Object[]{name};
            try {
                addTest((Test) constructor.newInstance(args));
            } catch (InstantiationException e) {
                addTest(warning("Cannot instantiate test case: " + name + " (" + exceptionToString(e) + ")"));
            } catch (InvocationTargetException e) {
                addTest(warning("Exception in constructor: " + name + " (" + exceptionToString(e.getTargetException()) + ")"));
            } catch (IllegalAccessException e) {
                addTest(warning("Cannot access test case: " + name + " (" + exceptionToString(e) + ")"));
            }

        } else { // almost a test method
            if (isTestMethod(m))
                addTest(warning("Test method isn't public: " + m.getName()));
        }
    }

    /**
     * Converts the stack trace into a string
     */
    protected String exceptionToString(Throwable t) {
        StringWriter stringWriter = new StringWriter();
        PrintWriter writer = new PrintWriter(stringWriter);
        t.printStackTrace(writer);
        return stringWriter.toString();

    }

    /**
     * Gets a constructor which takes a single String as
     * its argument.
     */
    protected Constructor getConstructor(Class theClass) throws NoSuchMethodException {
        Class[] args = {String.class};
        return theClass.getConstructor(args);
    }

    /**
     */
    protected boolean isPublicTestMethod(Method m) {
        return isTestMethod(m) && Modifier.isPublic(m.getModifiers());
    }

    /**
     */
    protected boolean isTestMethod(Method m) {
        String name = m.getName();
        Class[] parameters = m.getParameterTypes();
        Class returnType = m.getReturnType();
        return parameters.length == 0 && name.startsWith("test") && returnType.equals(Void.TYPE);
    }


    /**
     * Returns a test which will fail and log a warning message.
     */
    protected Test warning(final String message) {
        return new TestCase("warning") {
            protected void runTest() {
                fail(message);
            }
        };
    }
}
