package com.esdi.util;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author unascribed
 * @version 1.0
 */

public class ErrorTracker {

    private static ErrorTracker errorTracker;
    private String loadName;

    private int sqlErrorLimit;
    private int parseErrorLimit;
    private int recordErrorLimit;
    private int modifierErrorLimit;

    private int insertUpdateCount = 0;
    private int sqlErrorCount = 0;
    private int parseErrorCount = 0;
    private int recordErrorCount = 0;
    private int modifierErrorCount = 0;

    private int recordsProcessed = 0;
    private int recordsSkipped = 0;

    // making this a singleton
    public static ErrorTracker getInstance() {
        if (errorTracker == null) {
            String message = "Error Tracker is not initialized";
            System.out.println(message);
            throw new RuntimeException(message);
        }
        return errorTracker;
    }

    public static void initialize(String loadName) {
        if (errorTracker != null) {
            String message = "Error Tracker is already initialized";
            System.out.println(message);
            throw new RuntimeException(message);
        }
        errorTracker = new ErrorTracker(loadName);
    }

    private ErrorTracker(String loadName) {
        this.loadName = loadName;

        // reading system properties here
        int defaultSqlErrorLimit = Integer.getInteger("DEFAULT.sqlErrorLimit").intValue();
        sqlErrorLimit = Integer.getInteger(loadName+ "." + "sqlErrorLimit", defaultSqlErrorLimit).intValue();
        int defaultParseErrorLimit = Integer.getInteger("DEFAULT.parseErrorLimit").intValue();
        parseErrorLimit = Integer.getInteger(loadName+ "." + "parseErrorLimit", defaultParseErrorLimit).intValue();
        int defaultRecordErrorLimit = Integer.getInteger("DEFAULT.recordErrorLimit").intValue();
        recordErrorLimit = Integer.getInteger(loadName+ "." + "recordErrorLimit", defaultRecordErrorLimit).intValue();
        int defaultModifierErrorLimit = Integer.getInteger("DEFAULT.modifierErrorLimit").intValue();
        modifierErrorLimit = Integer.getInteger(loadName+ "." + "modifierErrorLimit", defaultModifierErrorLimit).intValue();

        /*
        System.out.println("SQL Error Limit: " + sqlErrorLimit);
        System.out.println("Parse Error Limit: " + parseErrorLimit);
        System.out.println("Record Error Limit: " + recordErrorLimit);
        System.out.println("Modifier Error Limit: " + modifierErrorLimit);
        */
    }

    public void incrementSqlError() {
        sqlErrorCount++;
    }

    public void incrementParseError() {
        parseErrorCount++;
    }

    public void incrementRecordError() {
        recordErrorCount++;
    }

    public void incrementModifierError() {
        modifierErrorCount++;
    }

    public void incrementRecordsProcessed() {
        recordsProcessed++;
    }

    public void incrementRecordsSkipped() {
        recordsSkipped++;
    }

    /**
     * @return 0 or 1 for now
     * 0 - success
     * 1 - some limit exceeded
     */
    public int getExitCode() {
        int exitCode = 0;
        if (sqlErrorCount >= sqlErrorLimit) {
            exitCode = 1;
            System.out.println("SQL Error Limit Exceeded");
        }
        if (parseErrorCount >= parseErrorLimit) {
            exitCode = 1;
            System.out.println("Parse Error Limit Exceeded");
        }
        if (recordErrorCount >= recordErrorLimit) {
            exitCode = 1;
            System.out.println("Record Error Limit Exceeded");
        }
        if (modifierErrorCount >= modifierErrorLimit) {
            exitCode = 1;
            System.out.println("Modifier Error Limit Exceeded");
        }
        return exitCode;
    }

    public String getExitMessage() {
        String message = "Processed " + recordsProcessed + ", ";
        message += "Skipped " + recordsSkipped + ", ";
        message += "SQL Errors " + sqlErrorCount + ", ";
        message += "Parse Errors " + parseErrorCount + ", ";
        message += "Record Errors " + recordErrorCount + ", ";
        message += "Modifier Errors " + modifierErrorCount;
        return message;
    }

    public int getRecordsProcessed() {
        return (recordsProcessed);
    }

    public int getRecordsSkipped() {
        return (recordsSkipped);
    }

    public int getSqlErrorCount() {
        return (sqlErrorCount);
    }

    public int getParseErrorCount() {
        return (parseErrorCount);
    }

    public int getRecordErrorCount() {
        return (recordErrorCount);
    }

    public int getModifierErrorCount() {
        return (modifierErrorCount);
    }

    public int getInsertUpdateCount() {
        return (insertUpdateCount);
    }

    public void setInsertUpdateCount(int insertUpdateCount) {
        this.insertUpdateCount = insertUpdateCount;
    }

}