package com.esdi.util;

/**
 * <p>Description: IDC International Corporate Actions are identifed by different things according
 * to the type of record:
 * <ul>
 * <li>Reqmts: Record type (DA) contains all cash, dividend options and stock dividends.  Record type (DB) is additional information,
 * associated to the corporate action stated in the preceding (DA) record  The link between the (DA) and (DB) is the same
 * security identification number, (position 01-12), and the sequence number, (position 18).  Not all (DA) records will have
 * corresponding (DB) records.</li>
 * <li>Reqmts: If KA record designates a corporate action type (such as a rights issue or misc. record) the corresponding
 * KF, KG, and KH need to be incomplete MISC. corporate actions.</li>
 * <li>MB: Others appear linked by sedol and first letter of type, eg T1, T2.</li>
 *
 * <p>Copyright: Copyright (c) Emphasis Software Development 2002</p>
 * @author Martin Buis
 * @version 1.0
 */

public class IDCInternationalCAKeyScheme
implements CAParserKeyScheme {

    public IDCInternationalCAKeyScheme() {
    }

    /**
     * Return true if lineOrKey is the first line of a record
     */
    public boolean isNewRecord(String priorKey, String lineOrKey) {
        return ! (buildKey(lineOrKey).equals(priorKey));
    }

    /**
     * Return the key according to the type of line
     */
    public String getKey(String line) {
        return buildKey(line);
    }

    /////////////////////////////////////////////////////////////////////////////////
    // IMPLEMENTATION
    /////////////////////////////////////////////////////////////////////////////////

    private String buildKey(String line) {
        String id = line.substring(0,14);
//        String id = line.substring(0,13);   // get the S-sedol, L=local flag also
        String type = line.substring(13,14);
        if (type.equals("D")) {
            String sequence = line.substring(17,18);
            return id + "Dx" + sequence;
        } else {
            // not sure if Xs, Ts, Ks etc should be considered separate records or part of the same record
            // this implementation considers them part of the same record
            return id;
        }
    }


    /////////////////////////////////////////////////////////////////////////////////
    // MAIN
    /////////////////////////////////////////////////////////////////////////////////

    public static void main(String[] args) {
        String[] lines = {
            "CH000101516*A08110200106252101                         1                      9N   JYN N N  NN6.75    420050701          USD",
            "CH000101516*BEnron Corp",
            "CH000101516*C6 3/4 % Debs /1993-1.7.05 Sr Subord -In Default-                                                                       US293561AT33",
            "CH000101516*DEnron                                                            921231                                 US 102",
            "CH000101516*E6.75 Enron 05SSDebs      0  26           USD",
            "CH000101516*F                                                                                                                           US 11 Y",
            "CH000101516*H19930701",
            "CH000101516*I                     01012XY",
            "CH000101516*A08111199307222101                                                                         20050701          USD200000000",
            "CH000101516*BEnron Corp",
            "CH000101516*C6 3/4 % Debs /1993-1.7.05 Sr Subord -In Default-",
            "CH000101516*DEnron                                                                                                   US",
            "CH000101516*E6.75 Enron 05SSDebs      0  26",
            "CH000101516*F1000        1000                                                                                                                  1000",
            "CH000101516*GLehman Brothers            Nationsbank Tx             10",
            "CH000101516*H19930701                                   99.497  4                                                                                    ",
            "CH000105544*A08130200106252101                                                                         20930715          USD",
            "CH000107234*A08130200106252101                                                                         20930715          USD" };

        IDCInternationalCAKeyScheme s = new IDCInternationalCAKeyScheme();
        for (int i = 0; i < lines.length; i++) {
            String line = lines[i];
            String key = s.getKey(line);
            System.out.println(key + ", " + line);
        }
    }

}