package com.esdi.util;

/**
 * <p>Title: ParserKeySchemeLoader</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c)2002 goNetix</p>
 * <p>Company: goNetix </p>
 * @author Joe LeVasseur Joe@LeVasseur.com
 * @version 1.0
 */
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.lang.reflect.Member;
import java.io.File;
import com.gn.util.*;

public class ParserKeySchemeLoader {

    private String          m_VendorFormat = null;
    private String          m_ParserInfoDir = null;
    private String          m_SimpleParseInfoEntry = null;
    private String          m_ParserKeyClass = null;
    private Hashtable       m_ParsersInfoHash = null;
    private boolean         m_IsSimpleParseScheme = false;
    private int[]           m_SimpleParseInfo = null;
    private final String    RELATIVEPARSERINFODIR = "parserinfo";
    private ICustomParserKeyScheme m_CustomParseKeySch = null;

    private boolean hasSeparator = false;
    private String separator = "";

    //--------------------------------------------------------------------------
    public ParserKeySchemeLoader(String VendorFormat)
    {
        m_VendorFormat = VendorFormat;
        init();
    }
    //--------------------------------------------------------------------------
    public String getKey(String Rec)
    {
        String Ret = null;
        if (Rec == null || Rec.equals("")==true) return Ret;
        try
        {
            if(m_IsSimpleParseScheme)
            {
                Ret = Rec.substring(m_SimpleParseInfo[0],m_SimpleParseInfo[0] +
                    m_SimpleParseInfo[1]);
            }
            else
            {
                Ret = m_CustomParseKeySch.getKey(Rec);
            }
        }
        catch(Exception ex)
        {
            System.out.println("Error in ParserSchemeLoader: " + ex.toString());
            System.out.println("Rec=" + Rec);
            ex.printStackTrace();
        }
        return Ret;
    }

    public boolean hasSeparator() {
        return hasSeparator;
    }

    public String getSeparator() {
        return separator;
    }

    public ICustomParserKeyScheme getCustomScheme() {
        return m_CustomParseKeySch;
    }

    //--------------------------------------------------------------------------
    private void init() //throws Exception
    {
        try
        {
            loadParsersInfoHash();
            m_SimpleParseInfoEntry = getParserInfoEntry();
            m_IsSimpleParseScheme = isSimpleParseScheme();
            if(m_IsSimpleParseScheme)
            {
                m_SimpleParseInfo = getParseInfo();
            }
            else
            {
                m_CustomParseKeySch =(ICustomParserKeyScheme)Class.forName(m_ParserKeyClass).newInstance();
            }
        }
        /*
        catch(ClassNotFoundException cnfe)
        {
            System.out.println("Error in ParserSchemeLoader: " + cnfe.toString());
            cnfe.printStackTrace();
            throw cnfe;
        }
        */
        catch(Exception ex)
        {
            System.out.println("Error in ParserKeySchemeLoader.init: " + ex.toString());
            ex.printStackTrace();
            //throw ex;
        }
    }
    //--------------------------------------------------------------------------
    private int[] getParseInfo()
    {
        int retInfo[] = new int[2];
        RecordParserUtils rpu = new RecordParserUtils();
        try
        {
            String rets[] = StringUtils.split(m_SimpleParseInfoEntry,",");
            if(rets[0].equalsIgnoreCase("Simple"))
            {
                String StartPos = rets[1];
                String NumChars = rets[2];
                retInfo[0] = (Integer.valueOf(StartPos).intValue() -1);
                retInfo[1] = (Integer.valueOf(NumChars).intValue());
            } else if (rets[0].equalsIgnoreCase("Separator")) {
                String StartPos = rets[1];
                String NumChars = rets[2];
                if (rets.length > 3) {
                    separator = rets[3];
                } else {
                    // defaulting separator to comma
                    // the line in the properties file should be
                    // Separator,1,2 not  Separator,1,2,,
                    separator = ",";
                }
                hasSeparator = true;
                retInfo[0] = (Integer.valueOf(StartPos).intValue() -1);
                retInfo[1] = (Integer.valueOf(NumChars).intValue());
            }
        }
        catch(Exception ex)
        {
            System.out.println("Error in ParserKeySchemeLoader.verifyIsSimpleParse: "
                + ex.toString());
            ex.printStackTrace();
        }
        return retInfo;
    }
    //--------------------------------------------------------------------------
    private void loadParsersInfoHash()
    {
        RecordParserUtils rpu = new RecordParserUtils();
        m_ParserInfoDir = rpu.getConfigDir("ESDLoader") + File.separator + RELATIVEPARSERINFODIR;
        m_ParsersInfoHash = rpu.loadProperties(m_ParserInfoDir + File.separator + "parsers.properties");
    }

    //--------------------------------------------------------------------------
    private String getParserInfoEntry()
    {
        return (String)m_ParsersInfoHash.get(m_VendorFormat);
    }
    //--------------------------------------------------------------------------
    private boolean isSimpleParseScheme()
    {
        return verifyIsSimpleParse();
    }
    //--------------------------------------------------------------------------
    // returns true for either Simple or Separator
    private boolean verifyIsSimpleParse()
    {
        boolean retVal = false;
        RecordParserUtils rpu = new RecordParserUtils();
        try
        {
            String Rets[] = StringUtils.split(m_SimpleParseInfoEntry,",");
            if (Rets.length >= 1)
            {
                if(Rets[0].equalsIgnoreCase("Simple") || Rets[0].equalsIgnoreCase("Separator")) {
                    retVal = true;
                }
                else if(Rets[0].equalsIgnoreCase("SeparatorNoKey")) {
                    hasSeparator=true;
                    if(Rets.length > 3) {
                        separator=Rets[3];
                    }
                    else {
                        separator=",";
                    }
                    hasSeparator=true;
                    m_ParserKeyClass = Rets[1];
                }
                else
                    m_ParserKeyClass = Rets[0];
            }
        }
        catch(Exception ex)
        {
            System.out.println("Error in ParserKeySchemeLoader.verifyIsSimpleParse: "
                + ex.toString());
            ex.printStackTrace();
        }
        return retVal;
    }
    //--------------------------------------------------------------------------




    //--------------------------------------------------------------------------
    public static void main(String[] args)
    {
        try
        {
            ParserKeySchemeLoader pksl = new ParserKeySchemeLoader("JoeTest1");
            /*
            String key = pksl.getKey("D1668R9ANEDCX AN3711DAIMLERCHRYSLER AG          ECALL"+
                " JAN 0070000  1050800000050000550000055000005500000550B8000119200200000000" +
                "508010041110000002230000002076904120104110104120115A0020770D1668R12000112180" +
                "000200000000000000000000200200112182001202020200000002001200010904230- " +
                "                                        00203P");
            */
            String key = pksl.getKey("P0AA.N                20010913  -999940100-9999401000000" +
            "-9999401000000-9999401000000-9999301000000-9999401000000-9999401000000-" +
            "9999401000000-9999403000000-9999401000000");
            System.out.println("Key=" + key);
        }
        catch(Exception ex)
        {
            System.out.println("Error in Main: " + ex.toString());
        }
        System.exit(0);
    }
    //--------------------------------------------------------------------------
}