package com.esdi.util;

public class SecurityNameGenerator {

  static java.text.DecimalFormat df4 = new java.text.DecimalFormat("0.0000");
  static java.text.DecimalFormat df3 = new java.text.DecimalFormat("0.000");
  static java.text.SimpleDateFormat option_name_date_formatter = new java.text.SimpleDateFormat("yyyy-MM-dd");
  static java.text.SimpleDateFormat bond_name_date_formatter = new java.text.SimpleDateFormat("MM/dd/yy");
  static java.text.SimpleDateFormat bond_local_code_date_formatter = new java.text.SimpleDateFormat("yyyyMMdd");

  /**
  * returns system generated Option name
  * @param char put_call_flag      common..base_option.put_call_flag
  * @param String underlying_code    common..instrument.lcoal_code of underlying tid
  * @param String currency_code    common..instrument.currency_code
  * @param String counterparty_code    common..organization.abbrev
  * @param String option_type    common..option_type.abbrev
  * @param String to_ccy_code    common..currency_exchange.to_ccy_code(needed for FX Options)
  * @param String from_ccy_code    common..currency_exchange.from_ccy_code(needed for FX Options)
  * @param String exchange_abbrev    common..exchange.abbrev
  * @param Date expiry_date    common..base_option.expiration_date
  * @param double strike_price    common..base_option.strike_price
  * @param String comments
  */
  public static String getOptionName(char put_call_flag, String underlying_code,
                                String currency_code, String counterparty_code,
                                String option_type_abbrev, String to_curr_code,
                                String from_curr_code, String exchange_abbrev,
                                java.util.Date expiry_date, double strike_price,
                                String comments)
  {
      StringBuffer option_name = new StringBuffer();
        if(from_curr_code==null)
                from_curr_code = new String();
        if(to_curr_code==null)
                to_curr_code = new String();
      if (from_curr_code.length() > 0){
        option_name.append(from_curr_code);
        option_name.append(to_curr_code);
      } else {
        option_name.append(underlying_code);
      }
      // Add expiration date
      option_name.append(" ");
      option_name.append(option_name_date_formatter.format(expiry_date));
      // Add strike price
      if (strike_price >= 10){
        option_name.append(" ");
        option_name.append(df3.format(strike_price));
      } else {
        option_name.append(" ");
        option_name.append(df4.format(strike_price));
      }
      // Add put/call and option type
      option_name.append(" ");
      option_name.append(put_call_flag);
      option_name.append(" ");
      option_name.append(option_type_abbrev);
      // For currency options, add the currency in which the instrument settles
      if (from_curr_code.length() > 0 ) {
        option_name.append(" [");
        option_name.append(currency_code);
        option_name.append("]");
      }
      // Add either exchange or counterparty
      if ((exchange_abbrev.length() > 0)&&(!exchange_abbrev.equals("OTC"))) {
        option_name.append(" ");
        option_name.append(exchange_abbrev);
      } else {
        option_name.append(" ");
        option_name.append(counterparty_code);
      }
      //comments
      option_name.append(" ");
      if(comments != null)
        option_name.append(comments);
      return option_name.toString().trim();
  }
  /**
  * returns system generated bond name
  * @param String bond_class_abbrev      common..bond_class.abbrev
  * @param String series_code    common..bond.series_code
  * @param double coupon    common..bond.coupon
  * @param Date maturity_date    common..bond.maturity_date
  */
  public static String getBondName(String bond_class_abbrev, String bond_series_code,
    double coupon, java.util.Date maturity_date,
    boolean isFloater, String nextCallYear, String couponType, String debtType)
  {
      if(bond_class_abbrev == null)
        bond_class_abbrev = new String();

      StringBuffer bond_name = new StringBuffer();
      bond_name.append(bond_class_abbrev);

      // add coupon amount
      bond_name.append(" ");
      bond_name.append(df4.format(coupon));
      bond_name.append("% ");
      // add maturity_date
      bond_name.append(bond_name_date_formatter.format(maturity_date));

      if(isFloater) {
          bond_name.append(" ");
          bond_name.append("FRN");
      }

      if(nextCallYear != null) {
          bond_name.append(" '");
          bond_name.append(nextCallYear);
      }

      if(couponType.equals("STRP")) {
          bond_name.append(" ");
          bond_name.append(debtType);
          bond_name.append(" STRP");
      }

      if(bond_series_code != null) {
          bond_name.append(" ");
          bond_name.append(bond_series_code);
      }

      return bond_name.toString().trim();
  }

  /**
  * returns system generated bond symbol
  * @param String bond_class_abbrev      common..bond_class.abbrev
  * @param double coupon    common..bond.coupon
  */
  public static String getBondSymbol(String bond_class_abbrev, double coupon,
    java.util.Date maturity_date)
  {
      if(bond_class_abbrev == null)
        bond_class_abbrev = new String();
      StringBuffer bondSymbol = new StringBuffer();
      bondSymbol.append(bond_class_abbrev);
      // add coupon amount
      bondSymbol.append("_");
      bondSymbol.append(df4.format(coupon));
      bondSymbol.append("_");
      if (bondSymbol.length() <= 12) {
          bondSymbol.append(bond_local_code_date_formatter.format(maturity_date));
      }
      return bondSymbol.toString().trim();
  }
}