package com.esdi.util;

import java.io.*;
import java.util.*;

/**
 * General purpose string counter class.
 * @author Bill Kelemen
 */
public class StringCounter {

    private HashMap strings = new HashMap();
    private boolean overflow = false;
    private int maxStrings = 10000;

    ////////////////////////////////////////////////////////////////////////////
    // constructors
    ////////////////////////////////////////////////////////////////////////////

    /**
     * Creates a StringCounter for a maximum of 10000 Strings.
     */
    public StringCounter() {
    }

    /**
     * Creates a StringCounter for an indicated maximum number of Strings.
     */
    public StringCounter(int maxStrings) {
        this.maxStrings = maxStrings;
    }

    ////////////////////////////////////////////////////////////////////////////
    // operations
    ////////////////////////////////////////////////////////////////////////////

    /**
     * Counts a new occurance of a string. If it is an old occurance, then the count is
     * incremented. <br><br>
     * @param s
     */
    public void count(String s) {
        if (s == null) {
            s = "";
        }
        StringCount string = (StringCount) strings.get(s);
        if (string == null) {
            if (strings.size() > maxStrings) {
                overflow = true;
            } else {
                strings.put(s, new StringCount(s));
            }
        } else {
            string.incCount();
        }
    }

    /**
     * Returns the HashMap of errors tracked. The hashmap has pairs of
     * (String, Integer) objects.
     */
    public HashMap getCounts() {
        HashMap map = new HashMap();
        for (Iterator iter = strings.keySet().iterator(); iter.hasNext(); ) {
            StringCount string = (StringCount) strings.get(iter.next());
            map.put(string.getString(), new Integer(string.getCount()));
        }
        return (map);
    }

    /**
     * Returns true if the HashMap has overflowed meaning that some
     * strings groups were skipped.
     * @return
     */
    public boolean overflowed() {
        return (overflow);
    }

    /**
     * Internal class to keep track of strings and counters.
     */
    private class StringCount {

        private String s;
        private int count;

        public StringCount(String s) {
            this.s = s;
            this.count = 1;
        }

        public void incCount() {
            count++;
        }

        public String getString() {
            return (s);
        }

        public int getCount() {
            return (count);
        }

    }

}
