package com.gn.util;

import java.io.*;

import org.apache.oro.io.GlobFilenameFilter;
import org.apache.oro.text.GlobCompiler;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) goNetix 2002</p>
 * <p>Company: </p>
 * @author Joe LeVasseur Joe@LeVasseur.com
 * @version 1.0
 */
import java.io.IOException;

public class FileUtils {

    public FileUtils()
    {
    }

    /**
     * Verifies that a file exists on disk.
     * @param fileName Filename to check
     * @return true is file exists, false otherwise
     */
    public static boolean fileExists(String fileName) {
        File file = new File(fileName);
        return (file.exists() && file.isFile());
    }

    /**
     * Verifies that a directory exists on disk.
     * @param directory Directory to check
     * @return true is directory exists, false otherwise
     */
    public static boolean dirExists(String directory) {
        File file = new File(directory);
        return (file.exists() && file.isDirectory());
    }

    /**
     * Returns an array of filenames that match the pattern. Rules for pattern
     * are similar to the UNIX ls or DOS dir command. This method uses the
     * apache.oro.text.GlobCompiler for pattern mattching.
     * @see org.apache.oro.text.GlobCompiler
     * @param pattern Path and filename pattern to match (ex. "/tmp/*.java")
     * @param incPath If true, the resurn values will contain the complete absolute
     *        path as well as the filename. If false, only filenames are returned.
     * @return array of string filenames
     */
    public static String[] ls(String pattern, boolean incPath) {
        File f = new File(pattern);

        // separate into path and filename (allows glob wildcards) and make sure
        // path ends with a File.separator
        String path;
        String filename;
        if (f.isDirectory()) {
            path = pattern;
            filename = "*.*";
        } else {
            filename = f.getName();
            path = pattern.substring(0, pattern.length()-filename.length());
        }
        path = new File(path).getAbsolutePath() + File.separator;

        // get directory listing...
        f = new File(path);
        FilenameFilter filter =
            new GlobFilenameFilter(filename, GlobCompiler.CASE_INSENSITIVE_MASK);

        // return array of filenames
        String[] filenames = f.list(filter);
        if (incPath) {
            for (int i=0; i<filenames.length; i++) {
                filenames[i] = path + filenames[i];
            }
        }
        return (filenames);
    }

    public static void splitFiles(String fname, int maxLinesPerFile) throws IOException {
        int BUFFER_SIZE = 1024*1024;
        int p = fname.lastIndexOf(".");
        String name = fname.substring(0, p);
        String ext = fname.substring(p, fname.length());
        BufferedReader in = new BufferedReader(new FileReader(fname), BUFFER_SIZE);
        int fileNbr = 1;
        BufferedWriter out = new BufferedWriter(new FileWriter(name + "-" + fileNbr + ext), BUFFER_SIZE);
        System.out.println("Writing " + name + "-" + fileNbr + ext);
        int n = 0;
        String line;
        while ((line = in.readLine()) != null) {
            n++;
            if (n % maxLinesPerFile == 0) {
                out.flush();
                out.close();
                fileNbr++;
                out = new BufferedWriter(new FileWriter(name + "-" + fileNbr + ext), BUFFER_SIZE);
                System.out.println("\nWriting " + name + "-" + fileNbr + ext);
            }
            if (n % 50000 == 0) {
                System.out.print(".");
            }
            out.write(line);
            out.write("\n");
        }
        out.flush();
        out.close();
        in.close();
    }

    /**
     * Returns an array of filenames that match the pattern. Rules for pattern
     * are similar to the UNIX ls or DOS dir command. This method uses the
     * apache.oro.text.GlobCompiler for pattern mattching.
     * @see org.apache.oro.text.GlobCompiler
     * @param pattern Path and filename pattern to match (ex. "/tmp/*.java")
     * @return array of string filenames without paths
     */
    public static String[] ls(String pattern) {
        return (ls(pattern, false));
    }

    public static boolean rename(String from, String to) {
        File fileFrom = new File(from);
        File fileTo = new File(to);
        return (fileFrom.renameTo(fileTo));
    }

    public static void main(String[] args) throws Exception {
        //FileUtils fileUtils1 = new FileUtils();
        /*
        String fileName = "D:\\ExceptionReports\\ExceptionReport-00000681.xml";
        if (args != null && args.length > 0) {
            fileName = args[0];
        }
        FileUtils.splitFiles(fileName, 500000);
        */

        // test rename method
        if (args.length >= 2) {
            System.out.println("Rename "+args[0]+" to "+args[1]+" returned "+
                               rename(args[0], args[1]));
        }
    }
}