package com.gn.util;
/**
 * <p>Title: </p>
 * <p>Description: Generic file parser/loader </p>
 * <p>Copyright: Copyright (c) goNetix 2002</p>
 * <p>Company: goNetix LLC</p>
 * @author Joe LeVasseur Joe@LeVasseur.com
 * @version 1.0
 */
import com.gn.util.*;
import java.util.Hashtable;
import java.util.HashMap;
import java.io.RandomAccessFile;


public class GenericFileLoader
{
    private static boolean m_IsCSVInput=false;
    private static boolean m_Output2XML=false;
    private static boolean m_Output2File=false;
    private static boolean m_Output2CSV=false;
    private static String m_OutputFile=null;
    private static String m_InputFile=null;
    private static String m_KeyScheme=null;
    private static String m_FixedKey=null;

    private static boolean hasSeparator = false;
    private static String separator = "";

    public GenericFileLoader()
    {
    }

    public static void main(String args[])
    {
        try
        {
            parseParameters(args);
            if(m_Output2XML)
            {
                parse2XML();
            }
            else if(m_Output2File)
            {
                parse2File();
            }
            else
            {
                parse2CSV();
            }

        }
        catch(Exception e)
        {
            System.err.println(e);
            e.printStackTrace();
        }

        System.exit(0);
    }

    //--------------------------------------------------------------------------
    private static void printOptions()
    {
        System.err.println("Generic file parser (c) 2002 goNetix LLC");
        System.err.println("Usage: java GenericFileLoader <DataFile> <OutFile> <KeyScheme> <-x -c>");
        System.err.println("-x Output to XML \n-c Output to CSV");
        System.err.println("Example:\njava GenericFileLoader C:\\TestGFL.txt c:\\Out.xml GFLTest -x");
        System.exit(1);
    }
    //--------------------------------------------------------------------------
    private static boolean isOptionPresent(String opt, String args[])
    {
        for(int i = 0; i < args.length; i++)
        {
            if(opt.equalsIgnoreCase(args[i]))
                return true;
        }
        return false;
    }
    //--------------------------------------------------------------------------
    private static void parseParameters(String[] args)
    {
        if( args.length < 3 )
        {
            printOptions();
        }
        m_Output2XML=isOptionPresent("-x",args);
        m_Output2File=isOptionPresent("-f",args);
        m_Output2CSV=isOptionPresent("-c",args);
        m_IsCSVInput=isOptionPresent("-icsv",args);
        m_InputFile=args[0];
        m_OutputFile=args[1];
        m_KeyScheme=args[2];
        if(!FileUtils.fileExists(m_InputFile))
        {
            System.err.println("\nCan't find " + m_InputFile + '\n');
            printOptions();
        }
    }
    //--------------------------------------------------------------------------
    private static void parse2XML()
    {
        String currLine = "";
        String Result = "";
        int skippedRecs = 0;
        Hashtable propertiesHashtable = new Hashtable();
        String sep = java.io.File.separator;
        try
        {

            RecordParserUtils rpu = new RecordParserUtils();
            int[] offsets=getSimpleParseInfo();
            RecordFeeder rf = new RecordFeeder(m_InputFile);
            RecordParser rp = new RecordParser();
            RandomAccessFile rafOut = new RandomAccessFile(m_OutputFile,"rw");
            propertiesHashtable=rpu.getPropsFiles(rpu.getConfigDir("ESDLoader") +
                sep + "props" + sep + m_KeyScheme);
            rafOut.setLength(0);
            rafOut.writeBytes("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
            rafOut.writeBytes("<data>\n");
            int numrecs = 0;
            while((currLine=rf.getRecord()) != null && currLine.length()> offsets[1])
            {
                numrecs++;
                String key;
                if(m_FixedKey != null)
                    key = m_FixedKey;
                else
                    key = currLine.substring(offsets[0],offsets[0]+ offsets[1]);
                Hashtable currHT = (Hashtable)propertiesHashtable.get(key);
                if (currHT != null)
                {
                  if(!m_IsCSVInput)
                    if (hasSeparator) {
                        Result = rp.parseRecordToXMLRec(currLine,currHT, separator);
                    } else {
                        Result = rp.parseRecordToXMLRec(currLine,currHT);
                    }
                    rafOut.writeBytes(Result);
                }
                else
                    skippedRecs++;
            }
            rafOut.writeBytes("</data>\n");
            System.out.println("Skipped " + skippedRecs + " of " + numrecs + " records");
        }
        catch(Exception ex)
        {
            System.out.println("Error in parse2XML: " + ex.toString());
            ex.printStackTrace();
        }
    }
    //--------------------------------------------------------------------------
    private static void parse2CSV()
    {
        String currLine = "";
        String Result = "";
        int skippedRecs = 0;
        Hashtable propertiesHashtable = new Hashtable();
        String sep = java.io.File.separator;
        try
        {

            RecordParserUtils rpu = new RecordParserUtils();
            int[] offsets=getSimpleParseInfo();
            RecordFeeder rf = new RecordFeeder(m_InputFile);
            RecordParser rp = new RecordParser();
            RandomAccessFile rafOut = new RandomAccessFile(m_OutputFile,"rw");
            propertiesHashtable=rpu.getPropsFiles(rpu.getConfigDir("ESDLoader") +
                sep + "props" + sep + m_KeyScheme);
            rafOut.setLength(0);
            int numrecs = 0;
            while((currLine=rf.getRecord()) != null && currLine.length()> offsets[1])
            {   // Record should be at LEAST as long as the key
                numrecs++;
                String key;
                if(m_FixedKey != null)
                    key = m_FixedKey;
                else
                    key = currLine.substring(offsets[0],offsets[0]+ offsets[1]);
                Hashtable currHT = (Hashtable)propertiesHashtable.get(key);
                if (numrecs == 1)
                {
                  Hashtable firstHT =
                    (Hashtable)propertiesHashtable.get(propertiesHashtable.keys().nextElement());
                  Result = rp.getCSVHeader(firstHT);
                  Result += '\n';
                  rafOut.writeBytes(Result);
                }
                if (currHT != null)
                {
                  if (hasSeparator) {
                      Result = rp.parseRecordToCSV(currLine,currHT, separator);
                  } else {
                      Result = rp.parseRecordToCSV(currLine,currHT);
                  }
                  Result += '\n';
                  rafOut.writeBytes(Result);
                }
                else
                    skippedRecs++;
            }
            System.out.println("Skipped " + skippedRecs + " of " + numrecs + " records");
        }
        catch(Exception ex)
        {
            System.out.println("Error in Parse2CSV: " + ex.toString());
            ex.printStackTrace();
        }
    }
    //--------------------------------------------------------------------------
    private static void parse2File()
    {
        String currLine = "";
        String Result = "";
        int skippedRecs = 0;
        Hashtable propertiesHashtable = new Hashtable();
        String sep = java.io.File.separator;
        try
        {

            RecordParserUtils rpu = new RecordParserUtils();
            int[] offsets=getSimpleParseInfo();
            RecordFeeder rf = new RecordFeeder(m_InputFile);
            RecordParser rp = new RecordParser();
            RandomAccessFile rafOut = new RandomAccessFile(m_OutputFile,"rw");
            propertiesHashtable=rpu.getPropsFiles(rpu.getConfigDir("ESDLoader") +
                sep + "props" + sep + m_KeyScheme);
            rafOut.setLength(0);
            int numrecs = 0;
            while((currLine=rf.getRecord()) != null && currLine.length()> offsets[1])
            {   // Record should be at LEAST as long as the key
                numrecs++;
                String key;
                if(m_FixedKey != null)
                    key = m_FixedKey;
                else
                    key = currLine.substring(offsets[0],offsets[0]+ offsets[1]);
                Hashtable currHT = (Hashtable)propertiesHashtable.get(key);

                if (currHT != null)
                {
                  rafOut.writeBytes(currLine + '\n');
/*
                  Result = rp.parseRecordToCSV(currLine,currHT);
                  Result += '\n';
                  rafOut.writeBytes(Result);
*/
                }
                else
                    skippedRecs++;
            }
            System.out.println("Skipped " + skippedRecs + " of " + numrecs + " records");
        }
        catch(Exception ex)
        {
            System.out.println("Error in Parse2CSV: " + ex.toString());
            ex.printStackTrace();
        }
    }
    //--------------------------------------------------------------------------
    private static int[] getSimpleParseInfo()
    {
        int retInfo[] = new int[2];
        RecordParserUtils rpu = new RecordParserUtils();
        String sep = java.io.File.separator;
        try
        {
            String ParserInfoDir = rpu.getConfigDir("ESDLoader") + sep + "parserinfo";
            Hashtable ParsersInfoHash = rpu.loadProperties(ParserInfoDir +
                sep + "parsers.properties");

            String KeyScheme=(String)ParsersInfoHash.get(m_KeyScheme/*.replaceAll(" ", "_")*/);
            String Rets[] = StringUtils.split(KeyScheme,",");
            if(Rets[0].equalsIgnoreCase("Simple"))
            {
                String StartPos = Rets[1];
                String NumChars = Rets[2];
                retInfo[0] = (Integer.valueOf(StartPos).intValue() -1);
                retInfo[1] = (Integer.valueOf(NumChars).intValue());
            }
            else if(Rets[0].equalsIgnoreCase("Separator"))
            {
                String StartPos = Rets[1];
                String NumChars = Rets[2];
                retInfo[0] = (Integer.valueOf(StartPos).intValue() -1);
                retInfo[1] = (Integer.valueOf(NumChars).intValue());
                if (Rets.length > 3) {
                    separator = Rets[3];
                } else {
                    // defaulting separator to comma
                    // the line in the properties file should be
                    // Separator,1,2 not  Separator,1,2,,
                    separator = ",";
                }
                hasSeparator = true;
            }
            else if(Rets[0].equalsIgnoreCase("NoKey"))
            {
                String fixedKey = Rets[1];
                if(fixedKey != null && fixedKey.equals("")==false)
                {
                    m_FixedKey = fixedKey;
                }
            }
        }
        catch(Exception ex)
        {
            System.out.println("Error in getSimpleParseInfo: "
                + ex.toString());
            ex.printStackTrace();
        }
        return retInfo;
    }
    //--------------------------------------------------------------------------


}//;


/* Contents of TestGL.txt
Key1Joe LeVasseur  06096015010102
Key211002David Lattimore099010102
Key1Donald Duck    01002063010102
Key1Mickey Mouse   06096155010102
Key211002Spider Man     056010102
Key1Sunny Liu      11002059010102
Key4Bad Boy        11002021010102
*/

/* Key1.properties
Field1=5,15,the_name,String
Field2=20,5,the_zip,String
Field3=25,3,the_age,Long
Field4=28,6,the_date,Date,MMddyy
*/

/* Key2.properties
Field1=5,5,the_zip,String
Field2=10,15,the_name,String
Field3=25,3,the_age,Long
Field4=28,6,the_date,Date,MMddyy
*/

/* parser.properties
GFLTest=Simple,1,4
*/