package com.gn.util;

/**
 * <p>Title: ObjectCaster</p>
 * <p>Description: Class to take a string description of an object
 * ("String", "Long", etc) and a string value and return an instance
 * of the desired class with the value of the string inside. </p>
 * <p>Copyright: Copyright (c)2002  goNetix </p>
 * <p>Company: goNetix </p>
 * @author Joe LeVasseur <Joe@LeVasseur.com>
 * @version 1.0
 */
import java.util.Date;
import java.util.Calendar;
import java.text.SimpleDateFormat;
import java.text.ParseException;
import java.math.BigDecimal;

public class ObjectCaster
{

    private int m_2DigitYearOffset = (-30);
    //--------------------------------------------------------------------------
    public ObjectCaster()
    {
    }
    //--------------------------------------------------------------------------
    public Object castValueOf(String ObjectType, String StringValue) throws Exception
    {
        return getObject(ObjectType,StringValue,"", null);
    }
    //--------------------------------------------------------------------------
    public Object castValueOf(String ObjectType, String StringValue,
        String Modifier) throws Exception
    {
        return getObject(ObjectType,StringValue,Modifier, null);
    }
    //--------------------------------------------------------------------------
    public Object castValueOf(String ObjectType, String StringValue,
        String Modifier, String Formatter) throws Exception
    {
        return getObject(ObjectType,StringValue,Modifier,Formatter);
    }
    //--------------------------------------------------------------------------

    private Object getObject(String ObjectType, String StringValue,
        String Modifier, String Formatter)
        throws Exception
    {
        Object tmpObj = null;
        String tmpVal = null;
        String tmpModifier = null;
        if(StringValue != null)
            tmpVal=StringValue.trim();
        if(Modifier != null)
            tmpModifier = Modifier.trim();
        else
            tmpModifier = "";
        try
        {
            if (ObjectType == null) {
                ObjectType = "STRING";
            }
            if(tmpVal != null)
            {
                String szVal= ObjectType.toUpperCase();
                if(szVal.equals("STRING"))
                    tmpObj = new String(tmpVal + tmpModifier);
                else if(szVal.equals("LONG"))
                {
                    Double d = new Double(tmpVal + tmpModifier);
                    Long l = new Long(d.longValue());
                    if(l.equals(new Long(Long.MAX_VALUE)) ||
                        l.equals(new Long(Long.MIN_VALUE)))
                    {
                        throw new Exception("NumberFormatException:");
                    }
                    tmpObj = l;
                }
                else if(szVal.equals("SHORT"))
                {
                    Float f = new Float(tmpVal + tmpModifier);
                    Short s = new Short(f.shortValue());
                    if(s.equals(new Short(Short.MAX_VALUE)) ||
                        s.equals(new Short(Short.MIN_VALUE)))
                    {
                        throw new Exception("NumberFormatException:");
                    }
                    tmpObj = s;
                }
                else if(szVal.equals("DATE"))
                    tmpObj = parseDate(tmpVal + tmpModifier, Formatter);
                else if(szVal.equals("DOUBLE") || szVal.equals("FLOAT"))
                    tmpObj = new Double(tmpVal + tmpModifier);
                else if(szVal.equals("INTEGER"))
                {
                    Double d = new Double(tmpVal + tmpModifier);
                    Integer i = new Integer(d.intValue());
                    if(i.equals(new Integer(Integer.MAX_VALUE)) ||
                        i.equals(new Integer(Integer.MIN_VALUE)))
                    {
                        throw new Exception("NumberFormatException:");
                    }
                    tmpObj = i;
                }
                else if(szVal.equals("BOOLEAN"))
                    tmpObj = parseBoolean(tmpVal + tmpModifier);
                else if(szVal.equals("CHAR"))
                {
                    tmpVal = tmpVal + tmpModifier;
                    char ch = 0;
                    if (tmpVal != null && !tmpVal.equals("")) {
                        ch = tmpVal.charAt(0);
                    }
                    tmpObj = new Character(ch);
                }
                else
                {
                    tmpObj = new String(tmpVal + tmpModifier);
                    System.out.println("Warning: no DataType found for value, returning string :" + tmpVal);
                }
            }
        }
        catch(Exception ex)
        {
            throw new Exception("Error in ObjectCaster.getObject: " + ex.toString() +
            " ObjectType=" + ObjectType + " StringValue=" + StringValue +
            " Modifier=" + Modifier);
        }
        return tmpObj;
    }
    //--------------------------------------------------------------------------
    private Date parseDate(String s, String DateFormat) throws ParseException
    {
        Date d = null;
        SimpleDateFormat date_fmt;
        if( s == null || s.trim().equals("")==true) return null;
        date_fmt = new SimpleDateFormat("yyyy");
        Integer windowYear = new Integer((Calendar.getInstance().get(Calendar.YEAR) + m_2DigitYearOffset));
        String szYear = windowYear.toString();
        date_fmt.set2DigitYearStart(date_fmt.parse(szYear));
        date_fmt.applyLocalizedPattern(DateFormat);
        date_fmt.setLenient(false);
        d = date_fmt.parse(s);
        if(Math.abs(DateUtils.yearDiff(d,new Date())) > 100)
        {
            System.err.println("Warning: Unreasonable Date:" + d.toString());
        }
        return d;
    }
    //--------------------------------------------------------------------------
    private Boolean parseBoolean(String s)
    {
        String szVal = null;
        boolean bRet = false;
        if(s != null && s.equals("") == false)
        {
            szVal = s.toUpperCase();
            if(szVal.equals("TRUE") || szVal.equals("1") || szVal.equals("YES"))
                bRet = true;
        }
        return new Boolean(bRet);
    }
    //--------------------------------------------------------------------------
    public void set2DigitYearOffset(int offset)
    {
        m_2DigitYearOffset=offset; // %THISYEAR% + offset will be used, default set
    }                              // at top of this module
    //--------------------------------------------------------------------------
    public static void main(String[] args)
    {
        try
        {
            ObjectCaster objectCaster1 = new ObjectCaster();
            System.out.println(objectCaster1.castValueOf("String","67GF"));
            System.out.println(objectCaster1.castValueOf("String","8888").getClass());
            //-----------These two throw exceptions ----------------------------
            //System.out.println(objectCaster1.castValueOf("Double","GGGHHH"));
            //System.out.println(objectCaster1.castValueOf("Long","1234.55"));
            //------------------------------------------------------------------
            System.out.println(objectCaster1.castValueOf("Long","123455"));
            System.out.println(objectCaster1.castValueOf("Long","123455").getClass());
        }
        catch(Exception ex)
        {ex.printStackTrace();}
    }
    //--------------------------------------------------------------------------
}