package com.gn.util;

/**
 * <p>Title: ParserDataField</p>
 * <p>Description: Container for parsed results, represents one column</p>
 * <p>Copyright: Copyright (c) goNetix 2002</p>
 * <p>Company: goNetix</p>
 * @author Joe LeVasseur
 * @version 1.0
 */
import java.text.ParseException;

public class ParserDataField {

    private String m_FieldName = null;
    private String m_FieldContents = null;
    private String m_FriendlyName = null;
    private String m_DataType = null;
    private String m_Modifier = null;
    private String m_Formatter = null;
    private boolean m_ReturnNull = false;

    private ParserDataField() {}

    public ParserDataField(String FieldName,String FieldContents)
    {
        m_FieldName=FieldName;
        m_FieldContents=FieldContents;
        m_FriendlyName=FieldName;
        m_DataType=null;
        m_Modifier = null;
    }

    public ParserDataField(String FieldName,String FieldContents,
        String FriendlyName)
    {
        m_FieldName=FieldName;
        m_FieldContents=FieldContents;
        m_FriendlyName=FriendlyName;
        m_DataType=null;
        m_Modifier = null;
    }

    public ParserDataField(String FieldName,String FieldContents,
        String FriendlyName,String DataType)
    {
        m_FieldName=FieldName;
        m_FieldContents=FieldContents;
        m_FriendlyName=FriendlyName;
        m_DataType = DataType;
        if (m_DataType != null) {
            m_DataType = m_DataType.toUpperCase();
        }
        m_Modifier = null;
    }

    public ParserDataField(String FieldName,String FieldContents,
        String FriendlyName,String DataType,String Modifier)
    {
        m_FieldName=FieldName;
        m_FieldContents=FieldContents;
        m_FriendlyName=FriendlyName;
        m_DataType = DataType;
        if (m_DataType != null) {
            m_DataType = m_DataType.toUpperCase();
        }
        m_Modifier = Modifier;
    }

    public ParserDataField(String FieldName,String FieldContents,
        String FriendlyName,String DataType,String Modifier, String Formatter)
    {
        m_FieldName=FieldName;
        m_FieldContents=FieldContents;
        m_FriendlyName=FriendlyName;
        m_DataType = DataType;
        if (m_DataType != null) {
            m_DataType = m_DataType.toUpperCase();
        }
        m_Modifier = Modifier;
        m_Formatter = Formatter;
    }


    public String getFieldName()
    {
        return m_FieldName;
    }

    public String getFieldContents()
    {
        return m_FieldContents;
    }

    public void setFieldContents(String FieldContents)
    {
        m_FieldContents = FieldContents;
    }

    public String getFriendlyName()
    {
        return m_FriendlyName;
    }

    public String getDataType()
    {
        return m_DataType;
    }

    public String getModifier()
    {
        return m_Modifier;
    }

    public String getFormatter()
    {
        return m_Formatter;
    }

    /**
     *
     * @param value - if this module level variable is set to true then
     * getCastDataValue will return a null. We need a way to bypass the
     * conversion in the case of specific logic.
     */
    public void setNulled(boolean value)
    {
        m_ReturnNull = value;
    }

    public boolean isNulled()
    {
        return m_ReturnNull;
    }

    public Object getCastDataValue() throws ParseException
    {
        Object obj = null;
        try
        {
            if(m_ReturnNull)
            {
                return null;
            }
            ObjectCaster oc = new ObjectCaster();
            obj = oc.castValueOf(m_DataType,m_FieldContents,m_Modifier,m_Formatter);
        }
        catch(Exception ex)
        {
            throw new ParseException(ex.toString() + " FieldName=" + m_FieldName + " DataType=" + m_DataType + " FieldContents=" + m_FieldContents +
                " Modifier=" + m_Modifier, -1);
        }
        return obj;
    }

    public String toString()
    {
       String output = "FieldName=" + m_FieldName + " FieldContents=" + m_FieldContents +
                " FriendlyName=" + m_FriendlyName + " DataType=" + m_DataType +
                " Modifier=" + m_Modifier + " Formatter=" + m_Formatter;
        return output;
    }

    public void applyModifier(String modifier)
    {
        m_Modifier = modifier;
    }
}
