package com.gn.util;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileNotFoundException;
import java.io.IOException;

/**
 * <p>Title: Generic LineFeeder Class</p>
 * <p>Description: Generic LineFeeder Class</p>
 * <p>Copyright: Copyright (c) 2002 goNetix</p>
 * <p>Company: goNetix</p>
 * @author J.LeVasseur
 * @version 0.80
 */
public class RecordFeeder {

  private String m_CurrentRecord = "";
  private String m_NextRecord = "";
  private String m_FileName = "";
  private BufferedReader in = null;

  //-------------------------------------------------------------------

  /**
   * Constructor for a RecordFeeder using a default buffer size of 16K.
   * @param FileName
   * @throws FileNotFoundException
   */
  public RecordFeeder(String FileName) throws FileNotFoundException
  {
    this(FileName, 16);
  }

  public RecordFeeder(String FileName, int bufferSize) throws FileNotFoundException
  {
    OpenTheFile(FileName, bufferSize);
    m_FileName=FileName;
  }

  //-------------------------------------------------------------------
  public boolean hasRecord()
  {
    return (m_NextRecord != null);
  }
  //-------------------------------------------------------------------
  public String getRecord()
  {
    getNextRecord();
    return m_CurrentRecord;
  }
  //-------------------------------------------------------------------
  private void getNextRecord()
  {
    try
    {
      m_CurrentRecord = m_NextRecord;
      //if (m_CurrentRecord == null) System.out.println("Hey");
      m_NextRecord = in.readLine();
    }
    catch(Exception ex)
    {
      System.out.println("Error in RecordFeeder.getNextRecord:" + ex.toString());
      ex.printStackTrace();
      m_NextRecord = null;
    }
  }
  //-------------------------------------------------------------------
  private void OpenTheFile(String FileName, int BufferSize)
  {
    try
    {
      in = new BufferedReader(new FileReader(FileName), BufferSize);
      m_NextRecord = in.readLine();
    }
    catch(Exception ex)
    {
      System.out.println("Error in RecordFeeder.OpenFile: " + ex.toString());
      ex.printStackTrace();
      m_NextRecord = null;
    }
  }
  //-------------------------------------------------------------------

  /**
   * Closes the file if open.
   */
  public void close() {
    try {
        if (in != null) {
            in.close();
            in = null;
        }
    } catch (Exception ex) {
      System.out.println("Error in RecordFeeder.close: " + ex.toString());
      ex.printStackTrace();
      in = null;
    }
  }

}

