package com.gn.util;

/**
 * <p>Title: Generic String Record Parser (RecordParser)</p>
 * <p>Description: Generic String Record Parser</p>
 * <p>Copyright: Copyright (c) 2002 goNetix LLC</p>
 * <p>Company: goNetix</p>
 * @author J.LeVasseur (Joe@LeVasseur.com)
 * @version 1.00
 */

import java.io.FileInputStream;
import java.util.Hashtable;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.util.HashMap;
import java.text.ParseException;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.util.Iterator;

public class RecordParser
{

  private Hashtable m_FieldSpecs;
  private boolean m_DoCharEscaping = true;   // Default to true
  private boolean m_ReturnCastValue = true;  // Default to true
  private String m_XMLCSVDateFmt = "MM-dd-yyyy";
  private boolean m_NeedSubstitution=false;
  private String m_CSVOutputDelimiter = ",";
  private boolean m_UseTablenameInHeader = true;

  //-------------------Constructors------------------------------------
  public RecordParser()
  {
  }
  public RecordParser(String FileName)
  {
    m_FieldSpecs=loadProperties(FileName);
  }
  //-------------------------------------------------------------------
  public HashMap parseRecord(String Rec)
  {
    return parseRecord(Rec,m_FieldSpecs);
  }
  //-------------------------------------------------------------------
  /**
   *
   * @param Rec - The record itself
   * @param FieldSpecs - A HashTable containing the fields: Field1=0,10[,FieldName][,DataType]
   *
   *
   * <p>FieldSpecs should look like below ...</p>
   * <p>Field1=0,10,CUSIP,String</p>
   * <p>Field2=10,10,SomeNumber,Double</p>
   * <p>Field3=20,10</p>
   * <br>
   * @return A HashMap of ParserDataField objects
   *
   */
  public HashMap parseRecord(String Rec, Hashtable FieldSpecs)
  {
    HashMap hm = new HashMap();
    String FieldName = null;
    String FieldContents = null;
    String FriendlyName = null;
    String DataType = null;
    String Modifier = null;
    String Formatter = null;
    try
    {
      m_FieldSpecs=FieldSpecs;
      if (m_FieldSpecs == null)
        throw new ParseException("No fieldspecs provided",-1);
      for(Iterator i = m_FieldSpecs.keySet().iterator(); i.hasNext();)
      {
        String key = i.next().toString();
        String szValue = (String)m_FieldSpecs.get(key);
        if (szValue != null && !szValue.equals(""))
        {
          String szVals[] = StringUtils.split(szValue,",");
          FieldName = key;
          if (szVals.length >= 2)
          {
            String StartPos = szVals[0];
            String NumChars = szVals[1];
            DataType = null;
            Modifier = null;
            Formatter = null;
            // Properties file is "1" based, Java strings are "0" based ...
            int StartPosition = (Integer.valueOf(StartPos).intValue() -1);
            int NumberofChars = Integer.valueOf(NumChars).intValue();
            if (szVals.length >= 3) // Return actual friendly name as key
            {
              key = szVals[2];
            }
            if (szVals.length >= 4)
            {
                DataType=szVals[3];
            }
            if (szVals.length >= 5)
            {
                Modifier=szVals[4];
            }
            if (szVals.length >= 6)
            {
                Formatter = szVals[5];
            }
            FriendlyName = key;
            FieldContents = Rec.substring(StartPosition,StartPosition + NumberofChars);
            hm.put(FieldName,new ParserDataField(FieldName,FieldContents,
                FriendlyName,DataType,Modifier,Formatter));
          }
        }
      }
      hm = doSubstitutions(hm);
    }
    catch(Exception ex)
    {
      System.err.println("Error in RecordParser.parseRecord:" + ex.toString());
      System.err.println("Record:" + Rec);
      ex.printStackTrace();
    }
    return hm;
  }
  //-------------------------------------------------------------------
  /**
   *
   * @param Rec The DELIMITED Record to be parsed.
   * @param FieldSpecs A Hashtable containing the fields: Field1=0,10{,FieldName]
   * @param Delimiter (",", "|", "\t", whatever
   * @return A HashMap of ParserDataField with the Fields/Columns
   */
  public HashMap parseRecord(String Rec,Hashtable FieldSpecs,String Delimiter)
  {
    HashMap hm = new HashMap();
    String FieldName = null;
    String FieldContents = null;
    String FriendlyName = null;
    String DataType = null;
    String Modifier = null;
    String Formatter = null;
    String TmpRec = null;
    String TmpChar = null;
    String[] SplitResults;
    try
    {
      m_FieldSpecs=FieldSpecs;
      if (m_FieldSpecs == null)
        throw new ParseException("No fieldspecs provided",-1);

      //---------Start Changes JTL 06/18/02----------------------
      // Fix for a "near bug". If there are empty delimiters on
      // the end of the input string, the array is returned without
      // these elements/strings. I'm adding a char on the end, parsing,
      // and then removing the char. JTL
      TmpChar = getTemporaryChar(Delimiter);
      TmpRec = Rec + TmpChar;
      SplitResults = StringUtils.split(TmpRec,Delimiter);
      removeTemporaryChar(SplitResults,TmpChar);
      //---------------------------------------------------------

      //SplitResults = StringUtils.split(Rec,Delimiter);

      for(Iterator i = m_FieldSpecs.keySet().iterator(); i.hasNext();)
      {
        DataType = null;
        Modifier = null;
        Formatter = null;
        String key = i.next().toString();
        String szValue = (String)m_FieldSpecs.get(key);
        if (szValue != null && !szValue.equals(""))
        {
          String szVals[] = StringUtils.split(szValue,",");
          FieldName = key;
          if (szVals.length >= 2)
          {
            String StartPos = szVals[0];
            String NumChars = szVals[1];
            int StartPosition = (Integer.valueOf(StartPos).intValue() -1);
            if (szVals.length >= 3) // Return actual friendly name as key
            {
              key = szVals[2];
            }
            if (szVals.length >= 4)
            {
                DataType=szVals[3];
            }
            if (szVals.length >= 5)
            {
                Modifier=szVals[4];
            }
            if (szVals.length >= 6)
            {
                Formatter = szVals[5];
            }
            FriendlyName = key;
            // If "0" or less is assigned to StartPosition field we put an
            // empty string in the PDF's FieldContents
            if(StartPosition >= 0) {
                FieldContents = SplitResults[StartPosition];
            } else {
                FieldContents = "";
            }
            hm.put(FieldName,new ParserDataField(FieldName,FieldContents,
                FriendlyName,DataType,Modifier,Formatter));
          }
        }
      }
      hm = doSubstitutions(hm);
    }
    catch(Exception ex)
    {
      System.out.println("Error in RecordParser.parseRecord:" + ex.toString());
      ex.printStackTrace();
    }
    return hm;
  }
  //-------------------------------------------------------------------
  /**
   *
   * @param Rec The record
   * @return Comma Separated Values string
   */
  public String parseRecordToCSV(String Rec)
  {
    return parseRecordToCSV(Rec, m_FieldSpecs);
  }
  //-------------------------------------------------------------------
  /**
   *
   * @param Rec The record
   * @param FieldSpecs A hashtable containing the fields: Field1=0,10,[FieldName]
   * @return Comma Separated Values string
   * <p>There is no third overloaded version of this because you don't need to</p>
   * <p>parse a delimited to delimited. Hopefully....</p>
   */
  public String parseRecordToCSV(String Rec, Hashtable FieldSpecs)
  {
    StringBuffer CSVString = new StringBuffer();
    String tmpStr = "";
    try
    {

      if (FieldSpecs == null)
        throw new ParseException("No fieldspecs provided",-1);
      HashMap results = parseRecord(Rec,FieldSpecs);
      for (int i=1; i <= results.size(); i++)
      {
        String key = "Field" + i;
        String testKey = null;
        for(Iterator it = FieldSpecs.keySet().iterator(); it.hasNext();) {
            testKey = it.next().toString();
            if(testKey.endsWith(key)) {
                key = testKey;
                break;
            }
        }
        ParserDataField Fields = (ParserDataField)results.get(key);
        if(Fields != null)
        {
            if (m_ReturnCastValue)
                if (Fields.getDataType().equalsIgnoreCase("DATE"))
                {
                    Date date = (Date)Fields.getCastDataValue();
                     SimpleDateFormat formatter
                        = new SimpleDateFormat(m_XMLCSVDateFmt);
                    tmpStr = formatter.format(date);
                }
                else
                    tmpStr = Fields.getCastDataValue().toString();
            else
                tmpStr = Fields.getFieldContents();
            if (m_DoCharEscaping) tmpStr=escapeCSV(tmpStr);
            CSVString.append(tmpStr);
            if (i != results.size()) CSVString.append(m_CSVOutputDelimiter);
        }
      }
    }
    catch(Exception ex)
    {
      System.out.println("Error in RecordParser.parseRecordToCSV: " + ex.toString());
      ex.printStackTrace();
    }
    return CSVString.toString();
  }
  //-------------------------------------------------------------------
  public String parseRecordToCSV(String Rec, Hashtable FieldSpecs,String OutputDelimiter)
  {
    m_CSVOutputDelimiter=OutputDelimiter;
    return parseRecordToCSV(Rec,FieldSpecs);
  }
  //-------------------------------------------------------------------
  public String getCSVHeader(Hashtable FieldSpecs,boolean UseTablename)
  {
    m_UseTablenameInHeader=UseTablename;
    return getCSVHeader(FieldSpecs);
  }

  /**
   *  Use this to generate the first row of CSV file- if needed
   * @param FieldSpecs Hashtable of Fields specs
   * @return Comma delimited string of field names
   */
  public String getCSVHeader(Hashtable FieldSpecs)
  {
    StringBuffer CSVString = new StringBuffer();
    String tmpStr = null;
    try
    {
      m_FieldSpecs=FieldSpecs;
      if (m_FieldSpecs == null)
        throw new ParseException("No fieldspecs provided",-1);
      for(int i = 1; i <= FieldSpecs.size(); i++)
      {
        String key = "Field" + i;
        String testKey = null;
        for(Iterator it = FieldSpecs.keySet().iterator(); it.hasNext();) {
            testKey = it.next().toString();
            if(testKey.endsWith(key)) {
                key = testKey;
                break;
            }
        }
        String szValue = (String)FieldSpecs.get(key);
        if (szValue != null && !szValue.equals(""))
        {
          String szVals[] = StringUtils.split(szValue,",");
          switch (szVals.length)
          {
            case 2:
              CSVString.append(key);
              break;
            case 3:
            case 4:
            case 5:
            case 6:
            case 7:
              String tmpKey = "";
              int posOfDot = key.indexOf(".");
              if(posOfDot != -1 && m_UseTablenameInHeader) {
                tmpKey = key.substring(0,posOfDot + 1);
              }
              tmpKey += szVals[2];
              CSVString.append(tmpKey);
              break;
            default:
              // Well ... ???
          }
          if (i < m_FieldSpecs.size()) CSVString.append(m_CSVOutputDelimiter);
        }
      }
      tmpStr = CSVString.toString();
      if(tmpStr.endsWith(",")) {
        tmpStr = tmpStr.substring(0,tmpStr.length()-1);
      }
    }
    catch(Exception ex)
    {
      System.out.println("Error in RecordParser.getCSVHeader: " + ex.toString());
      ex.printStackTrace();
    }
    return tmpStr;
  }
  //-------------------------------------------------------------------
  /**
  *  Use this to generate the first row of CSV file- if needed
  * @return Comma delimited string of field names
  */
  public String getCSVHeader()
  {
    return getCSVHeader(m_FieldSpecs);
  }
  //-------------------------------------------------------------------
  /**
   *
   * @param Rec The record
   * @return XML string
   */
  public String parseRecordToXMLRec(String Rec)
  {
    return parseRecordToXMLRec(Rec, m_FieldSpecs);
  }
  //-------------------------------------------------------------------
  /**
   *
   * @param Rec The record
   * @param FieldSpecs A hashtable containing the fields: Field1=0,10,[FieldName]
   * @return XML string
   */
  public String parseRecordToXMLRec(String Rec, Hashtable FieldSpecs)
  {
    StringBuffer XMLString = new StringBuffer();
    XMLString.append("<record>\n");
    String key = "";
    try
    {

      if (FieldSpecs == null)
        throw new ParseException("No fieldspecs provided",-1);
      HashMap results = parseRecord(Rec,FieldSpecs);
      for (int i=1; i <= results.size(); i++)
      {
        // Only fields that begin with "Field" get in formatted output.
        ParserDataField Fields = (ParserDataField)results.get("Field" + i);
        if(Fields != null)
        {
            key = Fields.getFriendlyName();
            XMLString.append("\t<" + key + ">");
            String szField;
            if (m_ReturnCastValue)
            {
                if (Fields.getDataType().equalsIgnoreCase("DATE"))
                {
                    try
                    {
                        Date date = (Date)Fields.getCastDataValue();
                         SimpleDateFormat formatter
                            = new SimpleDateFormat(m_XMLCSVDateFmt);
                        szField = formatter.format(date);
                    }
                    catch(Exception ignore){szField="";} // What else????
                }
                else
                    szField = Fields.getCastDataValue().toString();
            }
            else
                szField = Fields.getFieldContents();
            if (m_DoCharEscaping) szField = escapeXML(szField);
            XMLString.append(szField);
            XMLString.append("</" + key + ">\n");
          }
      }
      XMLString.append("</record>\n");
    }
    catch(Exception ex)
    {
      System.out.println("Error in RecordParser.parseRecordToXMLRec: " + ex.toString());
      ex.printStackTrace();
      XMLString.setLength(0); XMLString.append("<record>\n</record>\n");
    }
    return XMLString.toString();
  }
  //-------------------------------------------------------------------
  /**
   * <p>This version is for DELIMITED records only, the properties</p>
   * <p>file should be something like this: Field1=1,0[,FriendlyColumnName]</p>
   * <p>(First number is the position of the field in the delimited string- ie:</p>
   * <p>First, Second, etc. Leave second number at zero.)</p>
   * @param Rec The DELIMITED record to parse
   * @param FieldSpecs A hashtable containing the fields: Field1=1,,name,String
   * @param Delimiter (",", "|", "\t", whatever
   * @return XML string
   */
  public String parseRecordToXMLRec(String Rec,Hashtable FieldSpecs,String Delimiter)
  {
    StringBuffer XMLString = new StringBuffer();
    XMLString.append("<record>\n");
    String key = "";
    //String DebugInfo = "";
    try
    {
      if (FieldSpecs == null)
        throw new ParseException("No fieldspecs provided",-1);
      HashMap results = parseRecord(Rec,FieldSpecs,Delimiter);

      for (int i=1; i <= results.size(); i++)
      {
        // Only fields that begin with "Field" get in formatted output.
        ParserDataField Fields = (ParserDataField)results.get("Field" + i);
        if(Fields != null)
        {
            key = Fields.getFriendlyName();
            //DebugInfo += " " + key;
            XMLString.append("\t<" + key + ">");
            String szField;
            if (m_ReturnCastValue)
            {
                String szDataType = Fields.getDataType();
                //DebugInfo += " " + szDataType;
                if (szDataType != null && szDataType.equalsIgnoreCase("DATE"))
                {
                    Date date = (Date)Fields.getCastDataValue();
                     SimpleDateFormat formatter
                        = new SimpleDateFormat(m_XMLCSVDateFmt);
                    szField = formatter.format(date);
                    //DebugInfo += " " + szField;
                }
                else
                    szField = Fields.getCastDataValue().toString();
                    //DebugInfo += " " + szField;
            }
            else
            {
                szField = Fields.getFieldContents();
                //DebugInfo += " " + szField;
            }
            if (m_DoCharEscaping) szField = escapeXML(szField);
            XMLString.append(szField);
            XMLString.append("</" + key + ">\n");
        }
      }
      XMLString.append("</record>\n");
    }
    catch(Exception ex)
    {
      System.err.println("Error in RecordParser.parseRecordToXMLRec: " + ex.toString());
      //System.out.println(DebugInfo);
      ex.printStackTrace();
      XMLString.setLength(0); XMLString.append("<record>\n</record>\n");
    }
    return XMLString.toString();
  }
  //-------------------------------------------------------------------
  /**
   *
   * @param Rec The record to parse
   * @return Array of strings
   */
  public Object[] parseRecordToArray(String Rec)
  {
    return parseRecordToArray(Rec, m_FieldSpecs);
  }
  //-------------------------------------------------------------------
  /**
   *
   * @param Rec The record to parse
   * @param FieldSpecs A hashtable of field specs
   * @return Array of Objects
   */
  public Object[] parseRecordToArray(String Rec, Hashtable FieldSpecs)
  {
    Object[] arrRet = null;
    Object tmpObj = null;
    try
    {
      if (FieldSpecs == null)
        throw new ParseException("No fieldspecs provided",-1);
      HashMap results = parseRecord(Rec,FieldSpecs);
      arrRet = new Object[results.size()];
      for (int i=0; i < results.size(); i++)
      {
        ParserDataField Fields = (ParserDataField)results.get("Field" + i);
        tmpObj = Fields.getCastDataValue();
        arrRet[i]=tmpObj;
      }
    }
    catch(Exception ex)
    {
      System.out.println("Error in RecordParser.parseRecordToArray: " + ex.toString());
      ex.printStackTrace();
    }
    return arrRet;
  }
  //-------------------------------------------------------------------
  /**
   *  Function that translates a properties file to a SQL
   *  statement to create a corresponding table
   * @param TableName Name of Table to create
   * @return String contain table creation SQL
   */
  public String getSQLForTableCreation(String TableName)
  {
    return getSQLForTableCreation(TableName,m_FieldSpecs);
  }
  //-------------------------------------------------------------------
  /**
   *  Function that translates a properties file to a SQL
   *  statement to create a corresponding table
   * @param TableName Name of Table to create
   * @param FieldSpecs HashTable of Fields specs
   * @return String contain table creation SQL
   */
  public String getSQLForTableCreation(String TableName, Hashtable FieldSpecs)
  {
    StringBuffer SQLString = new StringBuffer();
    SQLString.append("CREATE TABLE " + TableName + "(");
    try
    {
      m_FieldSpecs=FieldSpecs;
      if (m_FieldSpecs == null)
        throw new ParseException("No fieldspecs provided",-1);
      for(int i = 1; i <= FieldSpecs.size(); i++)
      {
        String key = "Field" + i;
        String szValue = (String)FieldSpecs.get(key);
        if (szValue != null && !szValue.equals(""))
        {
          String szVals[] = StringUtils.split(szValue,",");
          switch (szVals.length)
          {
            case 2:
              SQLString.append(key);
              break;
            case 3:
              SQLString.append(szVals[2]);
              break;
            default:
              // Well ... ???
          }
          SQLString.append(" VARCHAR(" + szVals[1] + ")");
          if (i < m_FieldSpecs.size()) SQLString.append(", ");
          if (i % 3 == 0) SQLString.append('\n');
        }
      }
      SQLString.append(");");
    }
    catch(Exception ex)
    {
      System.out.println("Error in RecordParser.getSQLForTableCreation: " + ex.toString());
      ex.printStackTrace();
    }
    return SQLString.toString();
  }
  //-------------------------------------------------------------------
  /**
   * <p>Escapes illegal XML characters- "&amp;","&lt;","&gt;","&apos;"
   * @param InputStr Input string
   * @return An escaped string
   */
  protected String escapeXML(String InputStr)
  {
    String SearchChars[] = {"&","<",">","'"};
    String ReplaceChars[] = {"&amp;","&lt;","&gt;","&apos;"};
    String tmpStr = InputStr;
    int iPos = 0;
    try
    {
      for(int i=0; i < SearchChars.length; i++)
      {
        iPos = tmpStr.indexOf(SearchChars[i]);
        if (iPos != -1)
        {
          tmpStr = tmpStr.substring(0,iPos) + ReplaceChars[i]
            + tmpStr.substring(iPos + 1);
        }
      }
    }
    catch(Exception ex)
    {
      System.out.println("Error in RecordParser.escapeXML: " + ex.toString());
      ex.printStackTrace();
    }
    return tmpStr;
  }
  //-------------------------------------------------------------------

  /**
   * <p>Escapes illegal CSV characters- ","
   * @param InputStr Input string
   * @return An escaped string
   */
  protected String escapeCSV(String InputStr)
  {
    return (escapeCSV(InputStr, new String[] {","}));
  }

  /**
   * <p>Escapes illegal CSV characters.
   * @param InputStr Input string
   * @param SearchChars array of search strings that should be escaped
   * @return An escaped string
   */
  protected String escapeCSV(String InputStr, String[] SearchChars)
  {
    //String SearchChars[] = {","};
    String tmpStr = InputStr;
    int iPos = 0;
    try
    {
      for(int i=0; i < SearchChars.length; i++)
      {
        iPos = tmpStr.indexOf(SearchChars[i]);
        if (iPos != -1)
        {
          tmpStr = "\"" + tmpStr + "\"";
        }
      }
    }
    catch(Exception ex)
    {
      System.out.println("Error in RecordParser.escapeCSV: " + ex.toString());
      ex.printStackTrace();
    }
    return tmpStr;
  }



  //---------------------------------------------------------------------------
  /**
   *
   * @param FileName - a Properties Key/Value file
   * @return Hashtable of Keys/Values
   */
  protected Hashtable loadProperties(String FileName)
  {
    Properties prop = new Properties();
    try
    {
      prop.load(new FileInputStream(FileName));
    }
    catch(Exception ex)
    {
      System.out.println("Error in RecordParser.loadProperties: " + ex.toString());
      ex.printStackTrace();
    }
    return prop;
  }
  //----------------------------------------------------------------------------
  protected HashMap doSubstitutions(HashMap hm)
  {

    boolean needsAnotherPass;
    do
    {
        needsAnotherPass = false;               // assume this pass will do it
        boolean performedSubstitutions = false; // did we perform any subs in this pass?

        String szField = null;
        String szValue = null;
        String szModifier = null;
        StringSubst ss = new StringSubst();

        // make substitution structure...
        for(Iterator i = hm.keySet().iterator(); i.hasNext();)
        {
            Object currKey = i.next();
            ParserDataField pdf = (ParserDataField)hm.get(currKey);
            szField = pdf.getFieldName();
            szValue = pdf.getFieldContents();
            szModifier = pdf.getModifier();
            if (szModifier != null) {
                if (szModifier.indexOf("$") != -1) {
                    szValue = null;     // value is not known yet
                } else {
                    szValue = szValue.trim() + szModifier.trim();
                }
            }
            ss.put(szField, szValue);
        }

        // perform substitutions...
        for(Iterator i = hm.keySet().iterator(); i.hasNext();)
        {
            Object currKey = i.next();
            ParserDataField pdf = (ParserDataField)hm.get(currKey);
            szModifier = pdf.getModifier();
            if(szModifier != null && szModifier.equals("") == false)
            {
                boolean bNeedsSubst = (szModifier.indexOf("$") != -1);
                if(bNeedsSubst)
                {
                    if(!ss.containsKey(getBareFieldName(szModifier)))
                    {
                        throw new RuntimeException("Error in Recordparser.doSubstitutions" +
                            " field does not exist: " + getBareFieldName(szModifier));
                    }
                    else
                    {
                        String szSubstVal = ss.subst(szModifier);
                        if (szSubstVal != null) {
                            pdf.applyModifier(szSubstVal);
                            performedSubstitutions = true;
                        } else {
                            needsAnotherPass = true;   // we need another substitution pass
                        }
                    }
                }
            }
        }

        // check for cycles...
        if (!performedSubstitutions && needsAnotherPass) {
            throw new RuntimeException(
                "Error in Recordparser.doSubstitutions: cycle detected");
        }

    }
    while (needsAnotherPass);

    return hm;
  }
  //-------------------------------------------------------------------
  /**
   * @Purpose: Change or assign FieldSpec HashTable via properties
   * @param FileName Name of properties file
   */
  public void setPropertiesFile(String FileName)
  {
    m_FieldSpecs=loadProperties(FileName);
  }
  //-------------------------------------------------------------------
  /**
   *
   * @param DoIt True/False whether to escape "&amp;","&lt;","&gt;"
   */
  public void setDoCharEscaping(boolean DoIt)
  {
    m_DoCharEscaping=DoIt;
  }
  //-------------------------------------------------------------------
  public void setReturnCastValue(boolean DoIt)
  {
    m_ReturnCastValue=DoIt;
  }
  //-------------------------------------------------------------------
  public void setXMLCSVDateFormat(String fmt)
  {
    m_XMLCSVDateFmt=fmt;
  }
  //-------------------------------------------------------------------
  public void setCSVOutputDelimiter(String delimiter)
  {
    m_CSVOutputDelimiter=delimiter;
  }
  //-------------------------------------------------------------------
  private String getBareFieldName(String in)
  {
        String tmp = in;
        int firstToken = in.indexOf("$");
        if(firstToken != -1)
        {
            tmp = tmp.substring(firstToken+1);
            firstToken = tmp.indexOf("$");
            if (firstToken != -1) {
                tmp = tmp.substring(0, firstToken);
            }
        }
        return tmp;
  }
  //------------------------------------------------------------------
  private String getTemporaryChar(String Delimiter)
  {
    String CharToAdd = " ";
    if(Delimiter.equals(" ")) {
        CharToAdd = ":";
    }
    return CharToAdd;
  }
  //------------------------------------------------------------------
  private void removeTemporaryChar(String[] arr , String ch)
  {
    String tmpStr = null;
    int len = arr.length;
    int Pos = 0;
    if(len > 0) {
        tmpStr = arr[len -1];
        Pos = tmpStr.lastIndexOf(ch);
        if(Pos > -1) {
            tmpStr=tmpStr.substring(0,Pos);
            arr[len -1] = tmpStr;
        }
    }
  }
} // End of class
