package com.gn.util;

/**
 * <p>Title: RecordParserUtils</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) goNetix 2002</p>
 * <p>Company: </p>
 * @author Joe LeVasseur Joe@LeVasseur.com
 * @version 1.0
 */
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Properties;
import java.io.FileReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.RandomAccessFile;
import java.util.StringTokenizer;
import java.util.Iterator;

public class RecordParserUtils {

    private final boolean DEBUGMODE = true;

    public RecordParserUtils() {
    }

    //--------------------------------------------------------------------------
    public Hashtable loadProperties(String FileName)
    {
        Properties prop = new Properties();
        try
        {
          prop.load(new FileInputStream(FileName));
        }
        catch(Exception ex)
        {
          System.out.println("Error in RecordParserUtils.loadProperties: " + ex.toString());
          ex.printStackTrace();
        }
        return prop;
    }

    //--------------------------------------------------------------------------
    public Hashtable loadRecordParserProperties(String FileName)
    {
        RecordParserProperties prop = new RecordParserProperties();
        try
        {
          prop.load(new FileInputStream(FileName));
        }
        catch(Exception ex)
        {
          System.out.println("Error in RecordParserUtils.loadRecordParserProperties: "
            + ex.toString());
          ex.printStackTrace();
        }
        return prop;
    }

    //--------------------------------------------------------------------------

    public String getCWD()
    {
        // This returns where the source file is ...
        String Ret = null;
        try
        {
            Ret = new File(new File(".").getAbsolutePath()).getParent();
        }
        catch(Exception ex)
        {
            System.out.println("Error in RecordParserUtils.getCWD: " + ex.toString());
            ex.printStackTrace();
        }
        return Ret;
    }

    public String getConfigDir(String projectName) {
        if (projectName == null) {
            return getCWD() + java.io.File.separator + "conf";
        } else {
            return getCWD() + java.io.File.separator + "conf" +
                java.io.File.separator + projectName ;
        }
    }
    //--------------------------------------------------------------------------
    public Hashtable getPropsFiles(String directory)
    {
      Hashtable ht = new Hashtable();
      Hashtable tmpHt = null;
      String sep = java.io.File.separator;
      try
      {
        File propsDir = new File(directory);
        if (DEBUGMODE) System.out.println("Properties files location: " + propsDir);
        String[] lst = null;
        if (propsDir.isDirectory() && propsDir.exists()) {
            lst = propsDir.list();
            for(int i = 0; i < lst.length; i++)
            {
              String filename = lst[i];
              int iPos = filename.lastIndexOf(".");
              boolean IsPropsFile = filename.endsWith("properties");
              if (iPos != -1 && IsPropsFile == true)
              {
                  String key = filename.substring(0,iPos).toUpperCase();
                  tmpHt = loadProperties(propsDir.getAbsolutePath() + sep +  filename);
                  ht.put(key,tmpHt);
              }
            }
        } else {
           System.out.println("Error in RecordParserUtils.getPropsFiles:\n"
                            + "  Invalid directory: "+directory);
        }
      }
      catch(Exception ex)
      {
        System.out.println("Error in RecordParserUtils.getPropsFiles: " + ex.toString());
        ex.printStackTrace();
      }
      return ht;
    }
    //--------------------------------------------------------------------------
    public ParserDataField getPDFWithFriendlyName(HashMap hm, String friendlyName)
    {
        ParserDataField pdf = null;
        try
        {
            ParserDataField[] rets = getPDFsWithFriendlyName(hm,friendlyName);
            if(rets == null)
            {
                return null;
            }
            switch(rets.length)
            {
                case 0:
                    break; //null
                case 1:
                    pdf = rets[0];
                    break;
                default:
                    throw new RecordParserUtilsException("getPDFWithFriendlyName: Ambiguous "
                    + "request for friendlyName: " + friendlyName);

            }
        }
        catch(ClassCastException ce)
        {
            System.out.println("Warning: Error in getPDFWithFriendlyName: " +
                ce.toString());  // Well, maybe they put a plain string in there...???
        }
        return pdf;
    }
    //--------------------------------------------------------------------------
    public ParserDataField getPDFWithFriendlyName(HashMap hm, String friendlyName,
        String fieldNamePrefix)
    {
        ParserDataField pdf = null;
        try
        {
            for(Iterator i = hm.keySet().iterator(); i.hasNext();)
            {
                Object currKey = i.next();
                String fieldName = (String)currKey;
                pdf = (ParserDataField)hm.get(currKey);
                //if(pdf.getFriendlyName().equalsIgnoreCase(friendlyName)
                if(pdf.getFriendlyName().equals(friendlyName)
                    && pdf.getFieldName().startsWith(fieldNamePrefix))
                {
                    return pdf;
                }
            }
        }
        catch(ClassCastException ce)
        {
            System.out.println("Warning: Error in getPDFWithFriendlyName: " +
                ce.toString());  // Well, maybe they put a plain string in there...???
        }
        return null;
    }
    //--------------------------------------------------------------------------
    public ParserDataField[] getPDFsWithFriendlyName(HashMap hm, String friendlyName)
    {
        ParserDataField pdf = null;
        ArrayList arrList = new ArrayList();
        ParserDataField[] rets = null;
        try
        {
            for(Iterator i = hm.keySet().iterator(); i.hasNext();)
            {
                Object currKey = i.next();
                pdf = (ParserDataField)hm.get(currKey);
                //if(pdf.getFriendlyName().equalsIgnoreCase(friendlyName))
                if(pdf.getFriendlyName().equals(friendlyName))
                {
                    arrList.add(pdf);
                }
            }
            if(arrList.size() > 0)
            {
                rets = (ParserDataField[])arrList.toArray(new ParserDataField[arrList.size()]);
            }
        }
        catch(ClassCastException ce)
        {
            System.out.println("Warning: Error in getPDFsWithFriendlyName: " +
                ce.toString());  // Well, maybe they put a plain string in there...???
        }
        return rets;
    }
    //--------------------------------------------------------------------------

    public ParserDataField getPDFWithFieldName(HashMap hm, String fieldName)
    {
        ParserDataField pdf = null;
        try
        {
            for(Iterator i = hm.keySet().iterator(); i.hasNext();)
            {
                Object currKey = i.next();
                pdf = (ParserDataField)hm.get(currKey);
                //if(pdf.getFieldName().equalsIgnoreCase(fieldName))
                if(pdf.getFieldName().equals(fieldName))
                {
                    return pdf;
                }
            }
        }
        catch(ClassCastException ce)
        {
            System.out.println("Warning: Error in getPDFWithFieldName: " +
                ce.toString()); // Well, maybe they put a plain string in there...???
        }
        return null;
    }
    //--------------------------------------------------------------------------
    public String getFileNameFromFileAndPath(String inFile)
    {
        int posSeparator = inFile.lastIndexOf(File.separator);
        if(posSeparator > -1) {
            return inFile.substring(posSeparator + 1);
        }
        return inFile;
    }

}
