package com.gn.util;

/**
 * <p>Title: StringSubst</p>
 * <p>Description: Simple class to do string substitution. </p>
 * <p>Extends HashMap. It replaces any string it finds in its
 * internal HashMap that are enclosed in "$". </p>
 * <p>StringSubst stringSubst1 = new StringSubst(); </p>
 * <p>stringSubst1.put("Joe","Ugly"); </p>
 * <p>stringSubst1.put("Tom","Dick"); </p>
 * <p>String Ret  = stringSubst1.Subst("The quick brown fox jumped over $Joe$ today"); </p>
 * <p>System.out.println(Ret); </p>
 * <p>Copyright: Copyright(c)2002 goNetix</p>
 * <p>Company: goNetix </p>
 * @author Joe LeVasseur <Joe@LeVasseur.com>
 * @version 1.0
 */

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Enumeration;
import java.util.StringTokenizer;

public class StringSubst extends HashMap
{
    private String m_Token = "$";  // Default token
    //--------------------Constructors------------------------------------------
    public StringSubst()
    {
        super();
    }
    public StringSubst(Map m)
    {
        super(m);
    }

    public StringSubst(Properties prop)
    {
        super();
        Enumeration enum = prop.keys();
        while(enum.hasMoreElements())
        {
            String key = (String)enum.nextElement();
            put(key,prop.getProperty(key));
        }
    }
    //-------------------Methods-Accessors--------------------------------------
    /**
     * Performs substitutions on a given input string.
     * @param substStr Input string with possible keys (ex. $KEY$) to substitute.
     * @return Returns the substituted string if all keys are knowm, or null if
     * one or more are unknown at this time (because they depend on non-substituded
     * values).
     */
    public String subst(String substStr)
    {
        StringTokenizer tokenizer = new StringTokenizer(substStr,m_Token);
        StringBuffer buf = new StringBuffer();
        try
        {
            while(tokenizer.hasMoreTokens())
            {
                String s = tokenizer.nextToken();
                if(containsKey(s))
                {
                    s=(String)get(s);
                    if (s == null) {
                        // the value of this key is still not known
                        return (null);
                    }
                }
                buf.append(s);
            }
        }
        catch(Exception ex)
        {
            System.out.println("Error in StringSubst.Subst: " + ex.toString());
            ex.printStackTrace();
        }
        return buf.toString();
    }
    //--------------------------------------------------------------------------
    public void setToken(String token)
    {
        m_Token = token;
    }
    //--------------------------------------------------------------------------
    public String getToken()
    {
        return m_Token;
    }
    //--------------------------------------------------------------------------




    //--------------------------------------------------------------------------
    public static void main(String[] args)
    {
        try
        {
            Properties props = new Properties();
            props.load(new java.io.FileInputStream("C:\\Joe.properties"));
            StringSubst stringSubst1 = new StringSubst(props);
            String Ret  = stringSubst1.subst("$Field1$$Field2$$Field3$");
            System.out.println(Ret);
        }
        catch(Exception ex)
        {}
    }
    //--------------------------------------------------------------------------

}
