package com.gonetix.automation;


import java.util.*;

/**
 * @author SLiu
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public final class Task 
{
	private Project container = null;
	private String id = null;
	private Vector dependents = null;	
	private Properties localProperties = null;
	private Vector actionList = null;	
	
	/**
	 * Constructor for Task.
	 */
	public Task(Project container) 
	{
		super();
		dependents = new Vector();
		actionList = new Vector();
		this.container = container;		
	}

	/**
	 * this is main entry point for each task
	 */
	public void perform() throws ActionExecutionException
	{
		try{
			int size = actionList.size();
			for(int i=0; i<size; i++){
				ActionWrapper wrapper = (ActionWrapper)actionList.elementAt(i);
				Action action = container.getActionRegistry().getActionInstance(wrapper.getActionName());
				action.setTask(this);
				action.setActionText(wrapper.getText());
				action.setActionProperties(wrapper.getActionProperties());
				if ( action instanceof ActionContainer && wrapper.hasSubAction()){
					runSubaction((ActionContainer)action, wrapper);
				}
				action.execute();				
			}			
		}catch(Exception ex){
			throw new ActionExecutionException("Task perform exception", ex);
		}
	}
	
	
	public void runSubaction(ActionContainer parent, ActionWrapper wrapper) throws Exception
	{
		ActionWrapper[] aws = wrapper.children();
		for(int i=0; i<aws.length; i++){
			ActionWrapper aw = aws[i];
			Action action = container.getActionRegistry().getActionInstance(aw.getActionName());
			if ( action instanceof Subaction)
			{
				Subaction child = (Subaction)action;
				child.setParent(parent);
				child.setTask(this);
				child.setActionText(aw.getText());
				child.setActionProperties(aw.getActionProperties());
				if ( child instanceof ActionContainer && aw.hasSubAction())
				{
					runSubaction((ActionContainer)child, aw);
				}
				child.execute();
			}else{
				throw new ActionExecutionException("action " + aw.getActionName() + " is not allowed in " + parent.getActionName());
			}			
		}
		
	}
	
	public ActionRegistry getActionRegistry() {
		return container.getActionRegistry();
	}
	
	public ActionWrapper newAction(String actionName)
	{
		ActionWrapper wrapper = new ActionWrapper(actionName);
		actionList.add(wrapper);
		return wrapper;
	}	
	
	public void addDependent(String list)
	{
		if (list==null || list.length()==0) return;
		String[] dps = list.trim().split("\\s*,\\s*");
		for(int i=0; i<dps.length; i++)
		{
			dependents.addElement(dps[i]);	
		}
	}

	/**
	 * Returns the id.
	 * @return String
	 */
	public String getId() {
		return id;
	}

	/**
	 * Sets the id.
	 * @param id The id to set
	 */
	public void setId(String id) {
		this.id = id;
	}
	
	public String[] getDependents()
	{
		String[] ret = new String[dependents.size()];
		dependents.copyInto(ret);
		return ret;
	}
	
	public String getProperty(String key)
	{
		return this.getProperty(key, null);	
	}
	
	public String getCmdlineProperty(String key)
	{
		return this.getCmdlineProperty(key, null);
	}
	
	public String getCmdlineProperty(String key, String value)
	{
		String retValue = null;
		Properties prop = this.container.getCmdLineProperties();
		if (prop!=null){
			retValue = prop.getProperty(key);
			if ( retValue!=null) return retValue;
		}
		return value;
	}
	
	public String getProperty(String key, String value)
	{
		String retValue = null;		
		if ( this.localProperties!=null){
			retValue = localProperties.getProperty(key);
			if ( retValue!=null) return retValue;
		}
		Properties prop = this.container.getGlobalProperties();
		if ( prop!=null){
			retValue = prop.getProperty(key, value);
			if ( retValue!=null) return retValue;					
		}		
		return value;
	}
	
	
	/**
	 * Sets the localProperties.
	 * @param localProperties The localProperties to set
	 */
	public void setLocalProperties(Properties localProperties) {
		this.localProperties = localProperties;
	}

	/**
	 * Returns the container.
	 * @return Project
	 */
	public Project getContainer() {
		return container;
	}

	/**
	 * Sets the container.
	 * @param container The container to set
	 */
	public void setContainer(Project container) {
		this.container = container;
	}

}
