/*
 * SQLStatement.java
 *
 * Created on August 22, 2002, 12:59 PM
 */

package com.gonetix.common;

/**
 *
 * @author  SLiu
 */
public final class SQLStatement {
    
    public static final int REGULAR = 1;
    public static final int PREPARED = 2;
    public static final int CALLABLE = 3;
    
    protected static final String REGULARSTR = "REGULAR";
    protected static final String PREPAREDSTR = "PREPARED";
    protected static final String CALLABLESTR = "CALLABLE";
    
    private String id = null;
    private int type = REGULAR;
    private String statement = null;    
    
    /** Creates a new instance of SQLStatement */
    public SQLStatement(String id, String typeStr, String statement) throws Exception
    {
        if ( id == null ) throw new Exception("FATAL: id attribute is required for statement element.");
        this.id = id;
        this.setType(typeStr);
        this.statement = statement;
    }
    
    public SQLStatement(String id, String typeStr)  throws Exception {
        this(id, typeStr, null);        
    }
    
    public SQLStatement(String id, int type, String statement) throws Exception
    {
        if ( id == null ) throw new Exception("FATAL: id attribute is required for statement element.");
        this.id = id;
        this.type = type;
        this.statement = statement;
    }
    
    public SQLStatement(String id, int type)  throws Exception {
        this(id, type, null);        
    }
    
    public SQLStatement(String id)  throws Exception {
        this(id, REGULAR, null);           
    }
    
    public String getId()
    {
        return this.id;
    }
    
    public int getType()
    {
       return this.type;
    }
    
    public void setType(int value) throws Exception
    {
        if (value<REGULAR || value>CALLABLE) 
            throw new Exception("FATAL: unknown SQL statement type (1, 2, 3)."); 
        this.type = value;
    }
    
    public void setType(String typeStr) throws Exception
    {
        if ( typeStr == null ) typeStr = REGULARSTR;
        if ( typeStr.equalsIgnoreCase(REGULARSTR)){
             this.type =REGULAR;
        }else if (typeStr.equalsIgnoreCase(PREPAREDSTR)) {
            this.type = PREPARED;
        }else if (typeStr.equalsIgnoreCase(CALLABLESTR)) {
            this.type = CALLABLE;
        }else{
            throw new Exception("FATAL: unknown SQL statement type (REGULAR, PREPARED, CALLABLE).");
        }
    }
    
    public String getStatement()
    {
        return this.statement;
    }
    
    public void setStatement(String value)
    {
        this.statement = value;
    }
    
}
