package com.gonetix.diagnostic;


import java.util.*;

/**
 * @author SLiu
 * 
 * 
 */
public final class Trace 
{

	private static final String HEADER = "[TRACE] ";
	private static boolean enable = true;
	private static final Vector listeners = new Vector();
		
	static
	{
		ConsoleTraceListener consoleTraceListner = new ConsoleTraceListener();
		listeners.add(consoleTraceListner);		
	}
	
	/**
	 * Constructor for Trace.
	 */
	protected Trace() {
		super();
	}
	
	public static void Assert(boolean expression, String msg)
	{
		if (!enable) return;
		if ( expression == false ) writeLine(msg);
	}
	
	public static void println(String msg)
	{
		if (!enable) return;
		writeLine(msg);
	}
	
	public static void printlnIf(boolean expression, String msg)
	{
		if (!enable) return;
		if ( expression ) writeLine(msg);
	}
	
	public static void print(String msg)
	{
		if (!enable) return;
		write(msg);
	}
	
	public static void printIf(boolean expression, String msg)
	{
		if (!enable) return;
		if ( expression ) write(msg);
	}
	
	protected static void write(String msg)
	{
		int size = listeners.size();
		for(int i=0; i<size; i++)
		{
			ITraceListener traceListener = (ITraceListener)listeners.elementAt(i);
			traceListener.print(HEADER);
			traceListener.print(msg);
			traceListener.flush();
		}
	}
	
	protected static void writeLine(String msg)
	{
		write(msg + "\n");
	}

	/**
	 * Returns the enable.
	 * @return boolean
	 */
	public static boolean isEnable() {
		return enable;
	}

	/**
	 * Sets the enable.
	 * @param enable The enable to set
	 */
	public static void setEnable(boolean enable) 
	{
		Trace.enable = enable;
	}
	
	public static void addTraceListner(ITraceListener traceListener)
	{
		listeners.addElement(traceListener);
	}
	
	public static void removeTraceListner(ITraceListener traceListener)
	{
		listeners.remove(traceListener);
	}

}
