/*
 * QueryEngine.java
 *
 * Created on August 8, 2002, 2:13 PM
 */

package com.gonetix.framework.dbengine;

import java.sql.*;
import java.util.*;
import com.gonetix.framework.xmlEngine.*;

/**
 *
 * @author  SLiu
 */
public class QueryEngine {
    
    private static final String dbUrlKey = "url";
    private static final String dbDvrKey = "driver";
    private static final String dbUserKey = "user";
    private static final String dbPwdKey = "password";
    private static final String defaultKey = "DEFAULT";    
    
    
    private Hashtable praparedCache;
    private Hashtable stmtHolder;
    private Properties dbProps;
    private Connection dbSvrConn;
    
    /** Creates a new instance of QueryEngine */
    public QueryEngine() {
        stmtHolder = new Hashtable(20);
        praparedCache = new Hashtable(20);
    }
    
     public QueryEngine(Properties dbProps) 
     {
         this();   
         this.dbProps = dbProps;
     }
     
     public void connect() throws SQLException
     {
         try{
             Class.forName(dbProps.getProperty(dbDvrKey));
             dbSvrConn = DriverManager.getConnection(dbProps.getProperty(dbUrlKey),
                                                     dbProps.getProperty(dbUserKey), 
                                                     dbProps.getProperty(dbPwdKey));             
         }catch(Exception ex){
             ex.printStackTrace();
             throw new SQLException("Fatal: Failed connect to database server.");
         }
     }
     
     public Connection createConnection(Properties prop) throws SQLException
	  {
	      if (prop==null) prop = this.dbProps;
	      Connection con = null;
          try{
              Class.forName(prop.getProperty(dbDvrKey)).newInstance();
          }catch(Throwable e){
              throw new SQLException("FATAL: SQLExecutor loading driver excepton.\n" + e.getMessage());
          }
          con = DriverManager.getConnection(prop.getProperty(dbUrlKey), prop.getProperty(dbUserKey), prop.getProperty(dbPwdKey));
	      
	      return con;
	  }
     
     public ResultSet query(String sql) throws SQLException
     {
         if ( dbSvrConn==null ) throw new SQLException("Fatal: SQL Connection has not been established yet.");
         if ( dbSvrConn.isClosed() ) throw new SQLException("Fatal: SQL Connection has been closed.");
         Statement stmt = dbSvrConn.createStatement();
          if ( stmt==null ) throw new SQLException("Fatal: can not create sql Statement.");
         ResultSet rs = stmt.executeQuery(sql);
         stmtHolder.put(rs, stmt);         
         return rs;
     }
     
     public Object[] queryRow(String sql) throws SQLException
     {
          ResultSet rs =  query(sql);
          ResultSetMetaData rsmd = rs.getMetaData();
          int count = rsmd.getColumnCount();
          Object[] objs = new Object[count];
          if (rs!=null && rs.next()){
              for(int i=0; i<count; i++){
                  objs[i] = rs.getObject(i+1);
              }              
          }
          closeResultSet(rs);
          return objs;
     }
     
     public Object queryScalar(String sql) throws SQLException
     {
          ResultSet rs =  query(sql);
          ResultSetMetaData rsmd = rs.getMetaData();          
          Object obj = null;
          if (rs!=null && rs.next()){              
                  obj = rs.getObject(1);                        
          }
          closeResultSet(rs);
          return obj;
     }
     
     
     public String[] queryStringRow(String sql) throws SQLException
     {
          ResultSet rs =  query(sql);
          ResultSetMetaData rsmd = rs.getMetaData();
          int count = rsmd.getColumnCount();
          String[] objs = new String[count];
          if (rs!=null && rs.next()){
              for(int i=0; i<count; i++){
                  objs[i] = rs.getString(i+1);
              }              
          }         
          closeResultSet(rs);
          return objs;
     }
     
     public String queryStringScalar(String sql) throws SQLException
     {
          ResultSet rs =  query(sql);
          ResultSetMetaData rsmd = rs.getMetaData();          
          String obj = null;
          if (rs!=null && rs.next()){              
                  obj = rs.getString(1);                        
          }
          closeResultSet(rs);
          return obj;
     }
     
     
     public ResultSet queryCache(String key, Object[] params, int[] sqltypes) throws SQLException
     {
         PreparedStatement stmt = (PreparedStatement)praparedCache.get(key);
         if ( stmt==null ) throw new SQLException("Fatal: SQL Statement has not been prepared yet.");
         for (int i=0; i<params.length; i++){
             if ( params[i] == null){
                stmt.setNull(i+1, sqltypes[i]);
             }else{
                stmt.setObject(i+1, params[i], sqltypes[i]);
             }
         }
         ResultSet rs = stmt.executeQuery(); 
         stmtHolder.put(rs, stmt);
         return rs;         
     }
     
     public ResultSet queryCache(Parameter[] params) throws SQLException
     {              
         return queryCache(defaultKey,  params);         
     }
     
     public ResultSet queryCache(String key, Parameter[] params) throws SQLException
     {
         PreparedStatement stmt = (PreparedStatement)praparedCache.get(key);
         if ( stmt==null ) throw new SQLException("Fatal: SQL Statement has not been prepared yet.");
         for (int i=0; i<params.length; i++){
             if ( params[i] == null){
                throw new SQLException("FATAL: Invalid paramter.");
             }else{
                 stmt.setObject(i+1, params[i].getObject(), params[i].getType());
             }
         }
         ResultSet rs = stmt.executeQuery();
         stmtHolder.put(rs, stmt);
         return rs;         
     }
     
     public int executeCache(Parameter[] params) throws SQLException
     {              
         return executeCache(defaultKey,  params);         
     }
     
     public int execute(String sql) throws SQLException
     {
         if ( dbSvrConn==null ) throw new SQLException("FATAL: SQL Connection has not been established yet.");
         if ( dbSvrConn.isClosed() ) throw new SQLException("FATAL: SQL Connection has been closed.");
         Statement stmt = dbSvrConn.createStatement();
          if ( stmt==null ) throw new SQLException("FATAL: can not create sql Statement.");
         int rs = stmt.executeUpdate(sql);
         stmt.close();        
         return rs;
     }
     
     
     public int executeCache(String key, Parameter[] params) throws SQLException
     {
         PreparedStatement stmt = (PreparedStatement)praparedCache.get(key);
         if ( stmt==null ) throw new SQLException("Fatal: SQL Statement has not been prepared yet.");
         for (int i=0; i<params.length; i++){
             if ( params[i] == null){
                throw new SQLException("FATAL: Invalid paramter.");
             }else{
                 stmt.setObject(i+1, params[i].getObject(), params[i].getType());
             }
         }
         int rs = stmt.executeUpdate();         
         return rs;         
     }
      
      
     public int executeCache(String key, Object[] params, int[] sqltypes) throws SQLException
     {
         PreparedStatement stmt = (PreparedStatement)praparedCache.get(key);
         if ( stmt==null ) throw new SQLException("Fatal: SQL Statement has not been prepared yet. -- " + key);
         for (int i=0; i<params.length; i++){
             if ( params[i] == null){
                stmt.setNull(i+1, sqltypes[i]);
             }else{
                stmt.setObject(i+1, params[i], sqltypes[i]);
             }
         }
         int rs = stmt.executeUpdate();             
         return rs;
     }
     
     public Object queryCacheScalar(String key, Object[] params, int[] sqltypes) throws SQLException
     {
          ResultSet rs =  queryCache(key, params, sqltypes);                   
          Object obj = null;
          if (rs!=null && rs.next()){              
                  obj = rs.getObject(1);                        
          }
          if ( rs!=null ) rs.close();
          return obj;
     }
     
     public String[] queryCacheStringRow(String key, Object[] params, int[] sqltypes) throws SQLException
     {
          ResultSet rs = queryCache(key, params, sqltypes); 
          ResultSetMetaData rsmd = rs.getMetaData();
          int count = rsmd.getColumnCount();
          String[] objs = new String[count];
          if (rs!=null && rs.next()){
              for(int i=0; i<count; i++){
                  objs[i] = rs.getString(i+1);
              }              
          }
          if ( rs!=null ) rs.close();          
          return objs;
     }
     
     public Object[] queryCacheRow(String key, Object[] params, int[] sqltypes) throws SQLException
     {
          ResultSet rs = queryCache(key, params, sqltypes);
          ResultSetMetaData rsmd = rs.getMetaData();
          int count = rsmd.getColumnCount();
          Object[] objs = new Object[count];
          if (rs!=null && rs.next()){
              for(int i=0; i<count; i++){
                  objs[i] = rs.getObject(i+1);
              }              
          }
          if ( rs!=null ) rs.close();
          return objs;         
     }
     
     public String queryCacheStringScalar(String key, Object[] params, int[] sqltypes) throws SQLException
     {
          ResultSet rs =  queryCache(key, params, sqltypes);                   
          String obj = null;
          if (rs!=null && rs.next()){              
                  obj = rs.getString(1);                        
          }
          if ( rs!=null ) rs.close();
          return obj;
     }
     
     public void prepare(String key, String sql) throws SQLException
     {
         if ( dbSvrConn==null ) throw new SQLException("Fatal: SQL Connection has not been established yet.");
         if ( dbSvrConn.isClosed() ) throw new SQLException("Fatal: SQL Connection has been closed.");
         PreparedStatement stmt = dbSvrConn.prepareStatement(sql);
         if (stmt!=null) praparedCache.put(key, stmt);         
     }
     
     public void prepare(String sql) throws SQLException
     {
           prepare(defaultKey, sql);                  
     }
     
     public PreparedStatement prepareSatement(PreparedStatement pstmt, Parameter[] params) throws SQLException
	  {
	  		for(int i=0; i<params.length; i++){
	            if ( params[i] == null){
	                throw new SQLException("FATAL: Invalid paramter.");
	             }else{
	                 pstmt.setObject(i+1, params[i].getObject(), params[i].getType());
	             }
	      }  	
	  	  return pstmt;
	  }
	  
	  public PreparedStatement prepareSatement(String sqlStr, Parameter[] params) throws SQLException
	  {
	  	PreparedStatement pstmt = this.dbSvrConn.prepareStatement(sqlStr);  		
	  	return prepareSatement(pstmt, params);
	  }
     
     public void closeResultSet(ResultSet rs)
     {
         if ( rs == null ) return;
         Statement stmt = (Statement)stmtHolder.remove(rs);
         try{
             if ( stmt != null ) stmt.close();
             rs.close();
         }catch(SQLException sqlex){
             System.err.println("Warning: " + sqlex.getMessage());
         }
         
     }
     
     public void close()
     {
         try{
            if(dbSvrConn!=null) dbSvrConn.close();
         }catch(SQLException sqlex){
             System.err.println("Warning: " + sqlex.getMessage());
         }
     }
    
    
    
}
