/*
 * Parameter.java
 *
 * Created on August 12, 2002, 12:15 PM
 */

package com.gonetix.framework.dbengine;

import java.sql.*;

/**
 *
 * @author  SLiu
 */
public final  class SQLParameter {
    
    private int sqlType;
    private Object value = null;
    private boolean isFromCMD = false;
    private int cmdIndex = 0;
    private String formatString = null;
    
    /** Creates a new instance of Parameter */
    public SQLParameter(String typeString, String valueString, String formatString) throws SQLException {
        this.formatString = formatString;
        sqlType =  getSqlType(typeString);
        value = getValue(valueString, formatString);        
    }
    
    public SQLParameter(String typeString, String valueString) throws SQLException {
        sqlType =  getSqlType(typeString);
        value = getValue(valueString, null);        
    }
    
    private int getSqlType(String typeString) throws SQLException
    {  
        int type = Types.VARCHAR;
        try{
            typeString = typeString.toUpperCase();
            if (typeString.equals("VARCHAR")){
                type = Types.VARCHAR;
            }else if (typeString.equals("CHAR")){                
                type = Types.CHAR;
            }else if (typeString.equals("LONGVARCHAR")){
                 type = Types.LONGVARCHAR;  
            }else if (typeString.equals("INT") || typeString.equals("INTEGER")){
                 type = Types.INTEGER;
            }else if (typeString.equals("BIGINT") || typeString.equals("LONG")){
                 type = Types.BIGINT;
            }else if (typeString.equals("BIT") || typeString.equals("BOOLEAN")){
                 type = Types.BIT;
            }else if (typeString.equals("DATE") || typeString.equals("DATETIME")){
                 type = Types.DATE;
            }else if (typeString.equals("DECIMAL") || typeString.equals("NUMBER")){
                 type = Types.DECIMAL;
            }else if (typeString.equals("DOUBLE")){
                 type = Types.DOUBLE;
            }else if (typeString.equals("FLOAT")){
                 type = Types.FLOAT;
            }else if (typeString.equals("NUMERIC")){
                 type = Types.NUMERIC;
            }else if (typeString.equals("REAL")){
                 type = Types.REAL;
            }else if (typeString.equals("SMALLINT")){
                 type = Types.SMALLINT;
            }else if (typeString.equals("TIME")){
                 type = Types.TIME;
            }else if (typeString.equals("TIMESTAMP")){
                 type = Types.TIMESTAMP;
            }else if (typeString.equals("TINYINT")){
                 type = Types.TINYINT;
            }else{
                type = Types.OTHER;
            }            
        }catch(Exception e){
            throw new  SQLException("FATAL: Invalid SQL Type String");  
        }
        return type;
    }
    
    private Object getValue(String valueString) throws SQLException
    {
        return getValue(valueString, null);
    }
    
    public void setCmdIndex(String intStr){ 
        this.isFromCMD = true;
        this.cmdIndex = Integer.valueOf(intStr).intValue();
    }
    
    public int getCmdIndex(){
        return this.cmdIndex;
    }
    
    public void setCmdValue(String valueString) throws SQLException
    {
        value = getValue(valueString, formatString);
    }
    
    public boolean isFromCMDLine()
    {
        return this.isFromCMD;
    }
    
    private Object getValue(String valueString, String formatString) throws SQLException
    {
        if (valueString==null || valueString.equalsIgnoreCase("NULL")) return null;
        if ( valueString.equals("CMDLINE")){
            isFromCMD = true;
            return null;
        }        
        java.text.SimpleDateFormat sdf;
        Object obj = null;
        try{
        switch(sqlType){
            case Types.VARCHAR:
            case Types.CHAR:
            case Types.LONGVARCHAR:
               obj =  valueString;
               break;
            case Types.INTEGER:
               obj = Integer.valueOf(valueString);
               break;
           case Types.SMALLINT:
               obj = Short.valueOf(valueString);
               break;
           case Types.TINYINT:
               obj = Byte.valueOf(valueString);
               break;
           case Types.BIT:
               if (valueString.equals("1")) valueString = "true";
               if (valueString.equals("0")) valueString = "false";
               obj = Boolean.valueOf(valueString);
               break;
          case Types.DOUBLE:
               obj = Double.valueOf(valueString);
               break;
           case Types.FLOAT:
           case Types.REAL:
               obj =  Float.valueOf(valueString);
               break;
           case Types.DECIMAL:
           case Types.NUMERIC:
               obj =  new java.math.BigDecimal(valueString);
               break;
           case Types.DATE:
               sdf = new java.text.SimpleDateFormat(formatString);
               obj =  new java.sql.Date((sdf.parse(valueString)).getTime());
               break;
           case Types.TIME:
               sdf = new java.text.SimpleDateFormat(formatString);
               obj =  new java.sql.Time((sdf.parse(valueString)).getTime());
               break;  
           case Types.TIMESTAMP:
               sdf = new java.text.SimpleDateFormat(formatString);
               obj =  new java.sql.Timestamp((sdf.parse(valueString)).getTime());
               break;    
        }
        
        }catch(Exception e){
            throw new SQLException("FATAL: Invalid Parameter value.\n" + e.toString() );
        }
        return obj; 
    }
    
    
    public Object getObject()
    {
        return this.value;
    }
    
    public int getType()
    {
        return this.sqlType;
    }
    
}
