package com.gonetix.framework.dbengine;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * @author SLiu
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public final class SQLStatement {

	public final static int REGULAR = 1;
	public final static int PREPARED = 2;
	public final static int CALLABLE = 3;
	public final static int UNKNOWN = 4;
	
	protected final String REGULAR_STR = "REGULAR";
	protected final String PREPARED_STR = "PREPARED";
	protected final String CALLABLE_STR = "CALLABLE";

	private String id = null;
	private String sqlText = null;
	private String href = null;
	private int type = REGULAR;
	private String src = null;
	private String xmlDir = null;
	
	

	public SQLStatement() 
	{		
	}
	
	public SQLStatement(String id, String type, String statement) {
        if ( id!=null) this.id = id;
        initStatementType(type);
        this.sqlText = statement;
    }
	
	public SQLStatement(String id) {
		this.id = id;
	}

	/**
	 * Returns the href.
	 * @return String
	 */
	public String getHref() {
		return href;
	}

	/**
	 * Returns the id.
	 * @return String
	 */
	public String getId() {
		return id;
	}

	/**
	 * Returns the sqlText.
	 * @return String
	 */
	public String getSqlText() {
		if (sqlText == null) {
			if (src != null)
				sqlText = readFile(src);
			else if (href != null)
				sqlText = readWebFile(href);

		}

		return sqlText;
	}

	protected String readFile(String path) {
		StringWriter buffer = new StringWriter(2048);
		PrintWriter pw = new PrintWriter(buffer);
		BufferedReader br = null;
		try {
			br = new BufferedReader(new FileReader(path));
			String line = null;
			while ((line = br.readLine()) != null) {
				pw.println(line);
				pw.flush();
			}

		} catch (IOException ioex) {
			throw new SQLStatementSourceNotFoundException("Could not read from " + src, ioex);
		} finally {
			if (pw != null)
				try {
					pw.close();
				} catch (Exception ex) {
					;
				};
			if (br != null)
				try {
					br.close();
				} catch (Exception ex) {
					;
				}
		}
		return buffer.toString();
	}

	protected String readWebFile(String path) {
		StringWriter buffer = new StringWriter(2048);
		PrintWriter pw = new PrintWriter(buffer);
		BufferedReader br = null;
		try {
			URL url = new URL(path);
			URLConnection urlConn = url.openConnection();
			urlConn.setDoInput(true);
			urlConn.setDoOutput(false);
			urlConn.setUseCaches(false);

			br =
				new BufferedReader(
					new InputStreamReader(urlConn.getInputStream()));
			String line = null;
			while ((line = br.readLine()) != null) {
				pw.println(line);
				pw.flush();
			}
		} catch (IOException ioex) {
			throw new SQLStatementSourceNotFoundException("Could not read from " + path, ioex);
		} finally {
			if (pw != null)
				try {
					pw.close();
				} catch (Exception ex) {
					;
				};
			if (br != null)
				try {
					br.close();
				} catch (Exception ex) {
					;
				}
		}
		return buffer.toString();
	}

	/**
	 * Returns the src.
	 * @return String
	 */
	public String getSrc() {
		return src;
	}

	/**
	 * Returns the type.
	 * @return int
	 */
	public int getType() {
		return type;
	}

	/**
	 * Sets the href.
	 * @param href The href to set
	 */
	public void setHref(String href) {
		this.href = href;
	}

	/**
	 * Sets the sqlText.
	 * @param sqlText The sqlText to set
	 */
	public void setSqlText(String sqlText) {
		this.sqlText = sqlText;
	}

	/**
	 * Sets the src.
	 * @param src The src to set
	 */
	public void setSrc(String src) {
		this.src = src;
	}

	/**
	 * Sets the sql stament type.
	 * @param type The type to set
	 */
	public void setType(int type) {
		this.type = type;
	}
	/**
	 * Sets the sql stament type.
	 * @param typeString The type to set
	 */
	public void setType(String typeString) {
		if (typeString != null && typeString.equalsIgnoreCase(PREPARED_STR)) {
			this.type = PREPARED;
		} else if (
			typeString != null && typeString.equalsIgnoreCase(CALLABLE_STR)) {
			this.type = CALLABLE;
		}
	}

	/**
	 * Sets the id.
	 * @param id The id to set
	 */
	public void setId(String id) {
		this.id = id;
	}

	/**
	 * Returns the xmlDir.
	 * @return String
	 */
	public String getXmlDir() {
		return xmlDir;
	}

	/**
	 * Sets the xmlDir.
	 * @param xmlDir The xmlDir to set
	 */
	public void setXmlDir(String xmlDir) {
		this.xmlDir = xmlDir;
	}
	
	protected void initStatementType(String typeString)
    {        
        if (typeString==null) {
            type = PREPARED;
        }else if (typeString.equalsIgnoreCase(REGULAR_STR)){
            type = REGULAR;
        }else if (typeString.equalsIgnoreCase(CALLABLE_STR)){
            type = CALLABLE;
        }else if (typeString.equalsIgnoreCase(PREPARED_STR)){
            type = PREPARED;
        }else{
            type = UNKNOWN;
        }        
    }

}
