package com.gonetix.framework.dbengine;

import java.sql.*;
import java.util.*;

public class XMLResultProcessor implements IResultProcessor 
{
  private final String XMLSTART = "<dataset>";
  private final String XMLEND = "</dataset>";
  private final String RSSTART = "<resultset>";
  private final String RSEND = "</resultset>";
  private final String ROWSTART = "<row>";
  private final String ROWEND = "</row>";
  private final String EL_START_START = "<";
  private final String EL_END_START = "</";
  private final String EL_START_END = ">";
  private final String EL_END_END = EL_START_END;
  private final String CDATA_END = " ]]>";
  private final String CDATA_START = "<![CDATA[ ";
  
  private DBDateFormater dateFormater = null; 
  private String xmlString = null;  
  private StringBuffer buffer = null;
  private boolean closed = false;

  
  public XMLResultProcessor(DBDateFormater formater) 
  {
      if (formater== null){
          dateFormater = new DBDateFormater();
      }else{
          dateFormater = formater;
      }
      
      buffer = new StringBuffer(1024);
      buffer.append(XMLSTART);
  }

  public XMLResultProcessor()
  {
      this(null);
  }

  public void process(ResultSet rs)
  {
      if ( closed == true || rs==null ) return;
      buffer.append(RSSTART);   
      int i= 0;
      try{
          ResultSetMetaData rsmd = rs.getMetaData();
          int colCount = rsmd.getColumnCount();
          String[] colNames = new String[colCount];
          int[] colTypes = new int[colCount];
          for(i=1; i<=colCount; i++){
              colNames[i-1] = rsmd.getColumnName(i);
              if (colNames[i-1].length()==0) colNames[i-1] = "column_" + i;
              colTypes[i-1] = rsmd.getColumnType(i);
          }
          while (rs.next()){
              buffer.append(ROWSTART);
              formateRow(buffer, rs, colCount, colNames, colTypes);
              buffer.append(ROWEND); 
          }
      }catch(Exception ex){
          ;
      }
      buffer.append(RSEND);
  }

  protected void formateRow(StringBuffer sb, ResultSet rs, int colCount, String[] colNames, int[] colTypes)
  {
      for(int i=0; i<colCount; i++){
            sb.append(EL_START_START);
            sb.append(colNames[i]);
            sb.append(EL_START_END);
            String data = "";
            try{
                switch (colTypes[i])
                {
                    case Types.DATE:
                        java.sql.Date d = rs.getDate(i+1);
                        if (d!=null) data = dateFormater.formatDate(d);
                        break;
                    case Types.TIME:
                        Time t = rs.getTime(i+1);
                        if ( t!=null) data = dateFormater.formatTime(t);
                        break;
                     case Types.TIMESTAMP:
                        Timestamp ts = rs.getTimestamp(i+1);
                        if (ts!=null) data = dateFormater.formatTimestamp(ts);
                        break;
                     default:
                        data = rs.getString(i+1);
                        if ( data == null ) data = "";
                        break;
                }                
            }catch(SQLException sex){
                data = sex.toString();
            } 
            if (XMLEntityHandler.hasIllegalEntity(data)) {
                sb.append(CDATA_START);
                sb.append(data);
                sb.append(CDATA_END);
            }else{
                sb.append(data);
            }
            sb.append(EL_END_START);
            sb.append(colNames[i]);
            sb.append(EL_END_END);
      }
  }

  
  public void close()
  {
      buffer.append(XMLEND);
      xmlString = buffer.toString();
      buffer = null;
      closed = true;
  }

  public String getXMLString()
  {
      if (closed == false) close();
      return xmlString;
  }

  public String toString()
  {
      return getXMLString();
  }
  
  public void process(int rowcount){}
  public void process(String svrMsg){}
  public void process(SQLException ex){}
  
  public static void main(String[] args)
  {
    XMLResultProcessor xMLResultProcessor = new XMLResultProcessor();
  }
  
}