/*
 * CSVWriter.java
 *
 * Created on August 8, 2002, 4:51 PM
 */

package com.gonetix.framework.dbpublish;

import java.io.*;
import java.util.*;
import java.sql.*;

/**
 *
 * @author  SLiu
 */
public class CSVWriter extends PublishWriter {
    
    protected BufferedWriter pw;
    protected String delimiter = ",";
    protected String varcharQuote = null;    
    protected int rowcount = 0;
    
    public CSVWriter(OutputStream out){
        super(out);
        try{
            pw =new BufferedWriter( new PrintWriter(this.out));
        }catch(Exception e){
            ;
        }
    }
    
    public CSVWriter(){
        super();       
    }
    
    
    protected void printTrailer() throws IOException 
    {
    	
    	if (customTrailer!=null){
            super.prepareCustomTrailer(rowcount);
            pw.write(customTrailer);
            pw.write(lineSeparator); 
        }
        pw.flush(); 
    }
    
    protected void resetRowCount() throws IOException 
    {
    	this.rowcount = 0;
    }
    
    public void write(ResultSet rs) throws IOException 
    {
        try{
            ResultSetMetaData rsmd = rs.getMetaData();
            int colcount = rsmd.getColumnCount();
            int[] colType = getColumnTypes(rsmd);            
            String[] colnames = getColumnNames(rsmd);
            
            String value = null;
            if (customHeader!=null && !headerApplied)
            {
                prepareCustomHeader();
                pw.write(customHeader);
                pw.write(lineSeparator);  
                if(!applyHeaderAll) headerApplied=true;              
            }
            if (doPrintHeader && !headerApplied ){            
                for(int i=0; i<colcount; i++){                   
                    pw.write((colnames[i]!=null)?colnames[i]:nullChar);
                    if (i!=colcount-1) pw.write(delimiter);
                } 
                pw.write(lineSeparator);                
            }
            
            long rowNum = rowNumberBase;            
            while(rs!=null && rs.next()){
                this.rowcount++;               
                for(int i=0; i<colcount; i++){
                    if (rowNumberColumn>0 && i==(rowNumberColumn-1))
                    {
                    	pw.write((rowNumberFormat==null)?String.valueOf(rowNum++):rowNumberFormat.format(rowNum++));
                		pw.write(delimiter);
                    }else{
	                    value = getString(rs, i+1, colType[i]);
	                    if (varcharQuote!=null && ( colType[i] == Types.VARCHAR || colType[i] == Types.CHAR)) pw.write(varcharQuote);
	                    pw.write((value!=null)?value:nullChar);
	                    if (varcharQuote!=null && (colType[i] == Types.VARCHAR || colType[i] == Types.CHAR)) pw.write(varcharQuote);                    
	                    if (i!=colcount-1) pw.write(delimiter);
                    }
                } 
                pw.write(lineSeparator);                
            }            
            pw.flush();           
        }catch(Exception ex){
            ex.printStackTrace();
            throw new IOException(ex.toString());
        }
       
    }
    
    public void setVarcharQuote(String value)
    {
      this.varcharQuote = value;  
    }
    public void setDelimiter(String value)
    {
        this.delimiter = value;
    }
    public void close() throws IOException
    {
        pw.close();
    } 
    
    public void initWriter(java.util.Properties prop)
    {
        super.initWriter(prop);
        if ( prop.getProperty("delimiter")!=null) setDelimiter(prop.getProperty("delimiter"));
        if ( prop.getProperty("varCharQuote")!=null) setVarcharQuote(prop.getProperty("varCharQuote"));                    
    }   
   
    
    public void setOutput(String filepath) throws IOException
    {        
        this.pw =new BufferedWriter( new FileWriter(filepath));
    }
   
    
}
