/*
 * DATWriter.java
 *
 * Created on August 8, 2002, 5:56 PM
 */

package com.gonetix.framework.dbpublish;

import com.gonetix.framework.util.StringUtil;
import java.io.*;
import java.util.*;
import java.sql.*;
/**
 *
 * @author  SLiu
 */
public class DATWriter extends PublishWriter {
    
    public static final int ALIGN_RIGHT = 1;
    public static final int ALIGN_LEFT = 0;
    
    protected BufferedWriter pw = null;
    
    private int[] aligns = null;
    private int[] lens = null;
    /** Creates a new instance of DATWriter */
    public DATWriter(OutputStream out) {
        super(out);
        try{
            pw =new BufferedWriter( new PrintWriter(this.out));
        }catch(Exception e){
            ;
        }
    }
    
    public DATWriter() {
        super();     
    }
    
    public void write(ResultSet rs) throws IOException 
    {
       
        int[] laligns = null;
    	int[] llens = null;
    
        try{
            ResultSetMetaData rsmd = rs.getMetaData();
            int colcount = rsmd.getColumnCount();
            int[] colType = getColumnTypes(rsmd);
            
            if (aligns == null){
                laligns = new int[colcount];
                for(int j=0; j<colcount; j++){
                   laligns[j] = ALIGN_LEFT;
                } 
            }else{
            	laligns = aligns;
            }
            if ( lens==null){
                llens = new int[colcount];
                for(int k=0; k<colcount; k++){
                   llens[k] = rsmd.getColumnDisplaySize(k+1);
                } 
            }else{
            	llens = lens;
            }
            while(rs!=null && rs.next()){
                for(int i=0; i<colcount; i++){                   
                    pw.write(fixLength(getString(rs, i+1, colType[i]), llens[i], laligns[i]));
                } 
                pw.write(lineSeparator);                
            }
            pw.flush();           
        }catch(Exception ex){
        	ex.printStackTrace();
            throw new IOException(ex.toString());
        }
       
    }
    
    private char[] fixLength(String value, int length, int align)
    {
        char[] ret = new char[length];
        for(int i=0; i<ret.length; i++) ret[i] = ' ';        
        if(value!=null){
            char[] src = value.toCharArray();
            if (src.length>=length){
                if(align == ALIGN_RIGHT){
                    System.arraycopy(src, (src.length-length), ret, 0, length);
                }else{
                    System.arraycopy(src, 0, ret, 0, length);
                }
            }else{
                if(align == ALIGN_RIGHT){
                    System.arraycopy(src, 0, ret, (length-src.length), src.length);
                }else{
                    System.arraycopy(src, 0, ret, 0, src.length);
                }                
            }            
        }
        return ret;
    }    
    
    public void close() throws IOException
    {
        pw.flush();
        pw.close();
    }
    
    public void setFieldLength(int[] value){
        this.lens = value;
    }
    
    public void setAligns(int[] value){
        this.aligns = value;
    }
    
    public void initWriter(java.util.Properties prop){
         super.initWriter(prop);
        String p = prop.getProperty("fieldlength");
        if ( p !=null) {
            int[] list = StringUtil.splitIntString(p, ",");
            if( list.length >0 ) setFieldLength(list);
        }
         p =  prop.getProperty("aligns");
        if (p!=null){
           int[] list = StringUtil.splitIntString(p, ",");
           if( list.length >0 ) setAligns(list);
        }
    }
    
    public void setOutput(String filepath) throws IOException
    {        
        this.pw =new BufferedWriter( new FileWriter(filepath));
    }
    
}
