/*
 * PDFWriter.java
 *
 * Created on October 2, 2002, 11:27 AM
 */

package com.gonetix.framework.dbpublish;


import java.io.*;
import java.util.*;
import java.sql.*;
import com.gonetix.framework.util.StringUtil;
import com.gonetix.framework.xsltEngine.*;

/**
 *
 * @author  SLiu
 */
public final class PDFWriter extends PublishWriter implements IPublishWriter2
{     
        
   private final String XML_TMP_FILE_NAME = ".PDFWriter2.temp.xml";
	
	private XMLWriter2 xmlWriter2 = null;	
	private String xslStylesheet = null;
	private String pdfOutput = null;
	private boolean isDev = false;
	
	/**
	 * Constructor for PDFWriter2.
	 */
	public PDFWriter(OutputStream out) 
	{
		super(out);		
		xmlWriter2 = new XMLWriter2();
		try{
			xmlWriter2.setOutput(XML_TMP_FILE_NAME);
		}catch(Exception ex){
			ex.printStackTrace();
		}
	
	}
	
	public PDFWriter() 
	{
		this(null);	
	}
	
	
	public void initWriter(java.util.Properties prop)
    {
        super.initWriter(prop);
        xmlWriter2.initWriter(prop);
		xslStylesheet = prop.getProperty("styleSheet");
        if ( xslStylesheet == null)
        {
            System.err.println("FATAL: XSL Style sheet is required for transformation.");
            System.exit(3);
        }
        String dev = prop.getProperty("isDev");
        if (dev!=null && (dev.equalsIgnoreCase("true") || dev.equalsIgnoreCase("yes")))
        {
        	this.isDev = true;
        }
               
    }
    
	
	public void write(ResultSet rs) throws IOException 
	{
	     xmlWriter2.write(rs);
	     generatePDF();
	}
        
   
    public void write(java.sql.Statement stmt, boolean hasFirstResult) throws java.io.IOException 
    {
        xmlWriter2.write(stmt, hasFirstResult);
        generatePDF();
    }
    
    
    
    protected void generatePDF() throws IOException 
    {	
    	try{    		
    		PDFRender pdfRender = new PDFRender();    		
    		pdfRender.renderFile(XML_TMP_FILE_NAME, this.xslStylesheet, this.out);
    	}catch(Exception ex){
    		ex.printStackTrace();
    		throw new IOException("Failed to generate PDF file. \n" + ex.toString());
    	}finally{    		
    		if ( isDev == false )
    		{
    			File f = new File(XML_TMP_FILE_NAME);
    			f.delete();
    		}
    	}   	
    	
    }
    
    public void close() throws IOException
    {
        super.close();
    }
    
}
