/*
 * PublishWriter.java
 *
 * Created on August 8, 2002, 3:56 PM
 */

package com.gonetix.framework.dbpublish;

import java.io.*;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.*;
import java.sql.*;
import com.gonetix.jextract.*;
import org.apache.poi.hssf.usermodel.*;
import com.gonetix.util.MathEvaluator;
import java.util.regex.*;
/**
 *
 * @author  SLiu
 */
public abstract class PublishWriter 
implements IPublishWriter
{
    
    protected OutputStream out = null;
    protected boolean doPrintHeader = false;
    protected String nullChar = "";
    protected Properties initProp = null;;
    protected java.text.SimpleDateFormat dateFormat = null;
    protected java.text.SimpleDateFormat timeFormat = null;
    protected java.text.SimpleDateFormat timestampFormat = null;
    protected String customHeader = null;
    protected String customTrailer = null;
    protected String[] formats = null;
    protected boolean[] doFormats = null;
    private final DecimalFormat df = new DecimalFormat("###.###");
    
    protected long rowNumberBase = -1;
    protected String rowNumberColumnName = null;
    protected int rowNumberColumn = 0;
    protected DecimalFormat rowNumberFormat = null;
    
    protected String[] cmdOutOptions = null;
    
    protected boolean applyHeaderAll = false;
    protected boolean applyTrailerAll = false;
    
    protected boolean headerApplied = false;
    protected boolean trailerApplied = false;
    
    protected String lineSeparator  = "\r\n";
    
    /** Creates a new instance of PublishWriter */
    
    public PublishWriter(OutputStream out) 
    {
        this();
        this.out = out;
    }
    
    public PublishWriter() 
    {  
        dateFormat = new java.text.SimpleDateFormat("yyyyMMdd");
        timeFormat = new java.text.SimpleDateFormat("HH:mm:ss");
        timestampFormat = new java.text.SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    	
    	//if runs on unix platform use unix convention;
    	if(File.pathSeparatorChar==':') lineSeparator = "\n";    
    }
    
    public String[] getColumnNames(ResultSetMetaData rsmd) throws SQLException
    {
        int colcount = rsmd.getColumnCount();
        String[] ret = new String[colcount];
        for(int h=0; h<colcount; h++){
           ret[h] = rsmd.getColumnName(h+1);
           ret[h] = (ret[h]!=null)?ret[h]:("column" + h+1);
           if(rowNumberColumnName!=null && rowNumberColumn==0 && rowNumberColumnName.equals(ret[h])) rowNumberColumn = h+1;
        } 
        return ret;
    }
    public int[] getColumnTypes(ResultSetMetaData rsmd) throws SQLException
    {
        int colcount = rsmd.getColumnCount();
        int[] ret = new int[colcount];
        for(int h=0; h<colcount; h++){
                   ret[h] = rsmd.getColumnType(h+1);                  
        } 
        return ret;
    }
    
    protected void parseDataFormat(boolean[] dofmts, String[] fmtStrings)
    {
    	if(fmtStrings==null || fmtStrings.length==0 ) return;
    	int len = dofmts.length;
    	if (len>fmtStrings.length) len = fmtStrings.length;    	    	
    	for(short j=0; j<len; j++)
        {
        	String formatString = fmtStrings[j];
        	if (formatString!=null && !formatString.trim().equals("--"))
        	{
        		dofmts[j] = true;
        		formatString = formatString.trim();
        	}
        }
    }
    
    protected void prepareCustomHeader()
    {
       
        Vector v = new Vector(20);        
        Variable[] vars = Util.parserPattern(customHeader, v);
        String[] tokens = new String[v.size()];
        v.copyInto(tokens);
        int i=0;
        for(i=0; i<vars.length; i++){
            int arrayIndex = vars[i].arrayIndex;
            String name = vars[i].varName;
            if (name.equalsIgnoreCase(Variable.DATE))            
                tokens[arrayIndex] = dateFormat.format(new java.util.Date());
            else if (name.equalsIgnoreCase(Variable.DATETIME))
                tokens[arrayIndex] = timestampFormat.format(new java.util.Date());
            else if (isIntegerName(name)>-1 && cmdOutOptions!=null){
            	int idx = isIntegerName(name);            	
            	tokens[arrayIndex] = cmdOutOptions[idx];
            }
        }
        StringBuffer buffer = new StringBuffer(100);
        for(i=0; i<tokens.length; i++){
            buffer.append(tokens[i]);
        }
        customHeader = buffer.toString();
    }
    
   protected void prepareCustomTrailer(int rowcount)
    {
       
        Vector v = new Vector(20);
        Variable[] vars = Util.parserPattern(customTrailer, v);
        String[] tokens = new String[v.size()];
        v.copyInto(tokens);
        int i=0;
        for(i=0; i<vars.length; i++){
            int arrayIndex = vars[i].arrayIndex;
            String name = vars[i].varName;
            if (name.equalsIgnoreCase(Variable.DATE))            
                tokens[arrayIndex] = dateFormat.format(new java.util.Date());
            else if (name.equalsIgnoreCase(Variable.DATETIME))
                tokens[arrayIndex] = timestampFormat.format(new java.util.Date());
            else if (name.equalsIgnoreCase(Variable.ROWCOUNT)){
                tokens[arrayIndex] = String.valueOf(rowcount);
            }else if (isIntegerName(name)>-1 && cmdOutOptions!=null){
            	int idx = isIntegerName(name);            	
            	tokens[arrayIndex] = cmdOutOptions[idx];
            }
        }
        StringBuffer buffer = new StringBuffer(100);
        for(i=0; i<tokens.length; i++){
            buffer.append(tokens[i]);
        }
        customTrailer = evalTrailer(buffer.toString());
    }
    
    protected String getString(ResultSet rs, int index, int type) throws SQLException
    {
        if ( rs == null ) return "";
        String ret = null;
        switch(type){
            case Types.DATE:               
                ret = dateFormat.format(rs.getDate(index));
                break;
            case Types.TIME:                
                ret = timeFormat.format(rs.getDate(index));
                break;
            case Types.TIMESTAMP:                
                ret = timestampFormat.format(rs.getDate(index));                
                break;
            case Types.DOUBLE:
            case Types.FLOAT:
            case Types.REAL:
            case Types.NUMERIC:            	
            case Types.DECIMAL:
            	ret = String.valueOf(rs.getDouble(index));            	
            	break;
            case Types.BIGINT:
            	ret = String.valueOf(rs.getLong(index));           	
            	break;
            case Types.INTEGER:
            case Types.SMALLINT:
            case Types.BIT:
            case Types.TINYINT:
            	ret = String.valueOf(rs.getInt(index));           	
            	break;
            default:
                ret = rs.getString(index);
                break;            
        }
        return ret;
    }
   
   /*
    protected String getString(Object obj, int type)
    {
        if ( obj == null ) return "";
        String ret = null;
        switch(type){
            case Types.DATE:               
                ret = dateFormat.format((java.util.Date)obj);
                break;
            case Types.TIME:                
                ret = timeFormat.format((java.util.Date)obj);
                break;
            case Types.TIMESTAMP:                
                ret = timestampFormat.format((java.util.Date)obj);                
                break;
            case Types.DOUBLE:
            case Types.FLOAT:
            case Types.BIGINT:
            case Types.INTEGER:
            	ret = DoubleOutputFormater.doubleToString(obj);            	
            	break;
            default:
                ret = obj.toString();
                break;            
        }
        return ret;
    }
    */
    public abstract void write(ResultSet rs)  throws IOException;    
    
    public void initWriter(java.util.Properties prop){
        this.initProp = prop;        
        if (prop.getProperty("dateFormat")!=null) dateFormat = new java.text.SimpleDateFormat(prop.getProperty("dateFormat"));
        if (prop.getProperty("timeFormat")!=null) timeFormat = new java.text.SimpleDateFormat(prop.getProperty("timeFormat"));
        if (prop.getProperty("timestampFormat")!=null) timestampFormat = new java.text.SimpleDateFormat(prop.getProperty("timestampFormat"));
        if (prop.getProperty("printHeader")!=null) setDoPrintHeader(Boolean.valueOf(prop.getProperty("printHeader")).booleanValue());
        if (prop.getProperty("nullChar")!=null) setNullChar(prop.getProperty("nullChar"));
        if (prop.getProperty("header")!=null) customHeader = prop.getProperty("header");              
        if (prop.getProperty("trailer")!=null) customTrailer = prop.getProperty("trailer");         
        if (prop.getProperty("applyTrailerAll")!=null) applyTrailerAll = Boolean.valueOf(prop.getProperty("applyTrailerAll")).booleanValue();        
        if (prop.getProperty("applyHeaderAll")!=null) applyHeaderAll = Boolean.valueOf(prop.getProperty("applyHeaderAll")).booleanValue();
        
        if (prop.getProperty("dataFormat")!=null) 
        {        	
        	formats =  prop.getProperty("dataFormat").split("\\|");
        }
        
        if ( prop.getProperty("rowNumberBase")!=null) rowNumberBase = Long.valueOf(prop.getProperty("rowNumberBase")).longValue(); 
        if ( prop.getProperty("rowNumberColumnName")!=null) rowNumberColumnName = prop.getProperty("rowNumberColumnName");
        if ( prop.getProperty("rowNumberColumn")!=null) rowNumberColumn = Integer.valueOf(prop.getProperty("rowNumberColumn")).intValue();
        if ( prop.getProperty("rowNumberFormat")!=null) {
        	rowNumberFormat = new DecimalFormat(prop.getProperty("rowNumberFormat")); 
        }        
         if ( prop.getProperty("cmdout")!=null) {        	
        	cmdOutOptions = (prop.getProperty("cmdout")).split(","); 
        }  
         
    }
    
    public void setDoPrintHeader(boolean value){
        this.doPrintHeader = value;
    }
    
    public void setNullChar(String value)
    {
        this.nullChar = value;
    }
    
    public void close() throws IOException
    {
        if (out !=null){
            out.flush();
            out.close();
        }
    }
    
    public void setOutput(String filepath) throws IOException
    {
        this.out = new FileOutputStream(filepath);
    }
    
    
    public String evalTrailer(String trailer)
    {
    	String pattern = "\\[EVAL<(.+)>\\[(.+)\\]\\]";
    	Pattern p = Pattern.compile(pattern);	
		Matcher m = p.matcher(trailer);
		MathEvaluator evaluator = new MathEvaluator();
		
		if ( m.find()){			
			String format = m.group(1);
			DecimalFormat df = new DecimalFormat(format);
			String exp = m.group(2);
			evaluator.parseExpression(exp);
			String valueString = String.valueOf(df.format(evaluator.getResult()));
			trailer = m.replaceFirst(valueString);
		}    	
    	return trailer;
    }
    
    
    public int isIntegerName(String name)
    {
    	int ret = -1;
    	try{
    		ret = Integer.valueOf(name).intValue();
    	}catch(Exception ex){
    		;
    	}
    	return ret;	
    }
    
}


final class DoubleOutputFormater
{
	public static String doubleToString(Object doubleObject)
	{
		String ret = doubleObject.toString();
		int index = ret.indexOf("E");
		if (index !=-1)
		{
			String part1 = ret.substring(0, index);
			String exp = ret.substring(index + 1);
			int expVal = Integer.valueOf(exp).intValue();
			StringBuffer buffer = new StringBuffer(15);			
			if (expVal>0)
			{
				int dl = decimalLength(buffer, part1);
				appendZero(buffer,expVal-dl);
			}else{				
				formatMinus(buffer, part1, expVal);
			}			
			ret = buffer.toString();			
		}
		
		
		return ret;
	}
	
	
	protected static int decimalLength(StringBuffer buffer, String dString)
	{
		int len = dString.length();
		boolean dpFound = false;
		int dl = 0;
		for(int i=0; i<len; i++)
		{
			char c = dString.charAt(i);
			if (c!='.' ) buffer.append(c);
			if (c=='.'){
				dpFound = true;
			}else if (dpFound){
				dl++;
			}
		}
		return dl;
	}
	
	
	protected static void appendZero(StringBuffer buffer,int length)
	{
		
		for(int i=0; i<length; i++)
		{			
			buffer.append("0");			
		}		
	}
	
	
	protected static void formatMinus(StringBuffer buffer, String dString, int dl)
	{
		int len = dString.length();
		for(int i=0; i<len; i++)
		{
			char c = dString.charAt(i);
			buffer.append(c);
			if (c=='.')
			{
				for(int k=0; k<dl; k++)
				{
					buffer.append("0");
				}
			}
		}
		
	}
	
	
	
	
	
	
}
