/*
 * PDFWriter.java
 *
 * Created on October 2, 2002, 11:27 AM
 */

package com.gonetix.framework.dbpublish;


import java.io.*;
import java.util.*;
import java.sql.*;
import java.awt.Color;
import com.lowagie.text.*;
import com.lowagie.text.rtf.RtfWriter;

/**
 *
 * @author  SLiu
 */
public final class RTFWriter extends PublishWriter implements IPublishWriter2
{      
    public static final Rectangle A0 = PageSize.A0;
    public static final Rectangle A1 = PageSize.A1;
    public static final Rectangle A2 = PageSize.A2;
    public static final Rectangle A3 = PageSize.A3;
    public static final Rectangle A4 = PageSize.A4;
    public static final Rectangle A5 = PageSize.A5;
    public static final Rectangle A6 = PageSize.A6;
    public static final Rectangle A7 = PageSize.A7;
    public static final Rectangle A8 = PageSize.A8;
    public static final Rectangle A9 = PageSize.A9;
    public static final int PORTRAIT = 0;
    public static final int LANDSCAPE = 1;
    
    private OutputStream out = null;
    private Document document = null;
    private Rectangle pageSize = A4;
    private int layout = 0;
    
    /** Creates a new instance of PDFWriter */
    public RTFWriter(OutputStream out) 
    {
        super(out);
        this.out = out;        
    }
    
    public RTFWriter() 
    {
        super();
    }
    
    public void setPageSize(Rectangle size)
    {
        if ( size == null ) return;
        pageSize = size;
    }
    
    public void setLayout(int layout)
    {
        this.layout = layout;
    }
    
    public void write(java.sql.ResultSet rs) throws java.io.IOException 
    {
        try{            
            if (layout == LANDSCAPE ) pageSize.rotate();
            document = new Document(pageSize);
            RtfWriter.getInstance(document, out);
            document.addAuthor("sunny.liu@gonetix.com");
            document.addCreationDate();
            document.open();
            write(rs, document);
            document.close();
        }catch(Exception ex){
            throw new java.io.IOException(ex.toString());
        }
    }
    
    private void write(java.sql.ResultSet rs, Document doc) throws java.io.IOException 
    {
        int k=0;
        try{
            ResultSetMetaData rsmd = rs.getMetaData();
            int colcount = rsmd.getColumnCount();
            int[] colType = getColumnTypes(rsmd);            
            String[] colnames = getColumnNames(rsmd);
            int[]  widths = new int[colcount];
            for(k=0; k<colcount; k++){
               widths[k] = getWidth(colType[k], rsmd.getColumnDisplaySize(k+1));
            } 
            Table datatable = new Table(colcount);
            datatable.setSpacing(0);
            datatable.setWidths(widths);
            
            datatable.setDefaultCellBorderWidth(1);
            datatable.setDefaultRowspan(1);
            datatable.setDefaultColspan(1);            
            datatable.setDefaultVerticalAlignment(Element.ALIGN_CENTER);
            datatable.setDefaultHorizontalAlignment(Element.ALIGN_CENTER);            
            for(k=0; k<colcount; k++){
               datatable.addCell(new Phrase(colnames[k], FontFactory.getFont(FontFactory.COURIER, 14, Font.BOLD)));
            }            
            while ( rs!=null && rs.next()){                
                for(k=0; k<colcount; k++){
                    if (  colType[k]==Types.VARCHAR || colType[k]==Types.LONGVARCHAR ) datatable.setDefaultHorizontalAlignment(Element.ALIGN_LEFT);
                    else datatable.setDefaultHorizontalAlignment(Element.ALIGN_CENTER);
                    datatable.addCell(getString(rs, k+1, colType[k]));
                }                
            }
            doc.add(datatable);
        }catch(Exception ex){
            ex.printStackTrace();
            throw new IOException(ex.getMessage());
        }
    }
    
    public void write(java.sql.Statement stmt, boolean hasFirstResult) throws java.io.IOException 
    {
        try{            
            if (layout == LANDSCAPE ) pageSize.rotate();
            document = new Document(pageSize);
            RtfWriter.getInstance(document, out);
            document.addAuthor("sunny.liu@gonetix.com");
            document.addCreationDate();
            document.open();
            int b = -1;            
            while ( hasFirstResult || (b=stmt.getUpdateCount())!=-1 ){            
              if ( hasFirstResult ){                  
                  ResultSet rs = stmt.getResultSet();
                  if ( rs!=null ) write(rs, document);                
              }else if (b!=-1){                  
                 document.add(new Paragraph("Total  "+ b + " row(s) are affected."));
                 b = -1;
              }
              hasFirstResult = stmt.getMoreResults();
            }
            document.close();
        }catch(Exception ex){
            throw new java.io.IOException(ex.toString());
        }
    }    
    
    protected int getWidth(int type, int size)
    {
        int ret = 10;
        switch(type)
        {
            case Types.CHAR:
                ret = size;
                break;
            case Types.VARCHAR:
            case Types.LONGVARCHAR:
                if ( size <= 25 ){
                    ret = size;
                }else{
                    ret = 25;
                }
                break;
            default:
                ret = 10;
                break;
        }
        return ret;   
    }       
        
    
    public void initWriter(java.util.Properties prop)
    {
        
        
    }
    
    public void setOutput(String filepath) throws java.io.IOException
    {
        out = new FileOutputStream(filepath);
    }
    
    public void close() throws java.io.IOException
    {
        super.close();
    }
    
}
