package com.gonetix.framework.dbpublish;


import java.io.*;
import java.util.*;
import java.sql.*;
import com.gonetix.framework.util.StringUtil;
import com.gonetix.framework.xsltEngine.*;
/**
 * @author SLiu
 *
  */
public class TransformWriter2 extends PublishWriter
implements IPublishWriter2 
{
	private final String XML_TMP_FILE_NAME = ".TransformWriter2.temp.xml";
	
	private XMLWriter2 xmlWriter2 = null;	
	private String xslStylesheet = null;
	private String resultOutput = null;
	private boolean isDev = false;
	
	/**
	 * Constructor for TransformWriter2.
	 * @param out
	 */
	public TransformWriter2(OutputStream out) {
		super(out);		
		xmlWriter2 = new XMLWriter2();
		try{
			xmlWriter2.setOutput(XML_TMP_FILE_NAME);
		}catch(Exception ex){
			ex.printStackTrace();
		}
	}

	/**
	 * Constructor for TransformWriter2.
	 */
	public TransformWriter2() {
		super(null);
	}
	
	public void initWriter(java.util.Properties prop)
    {
        super.initWriter(prop);
        xmlWriter2.initWriter(prop);
		xslStylesheet = prop.getProperty("styleSheet");
        if ( xslStylesheet == null)
        {
            System.err.println("FATAL: XSL Style sheet is required for transformation.");
            System.exit(3);
        }
        String dev = prop.getProperty("isDev");
        if (dev!=null && (dev.equalsIgnoreCase("true") || dev.equalsIgnoreCase("yes")))
        {
        	this.isDev = true;
        }
               
    }

	/**
	 * @see com.gonetix.framework.dbpublish.IPublishWriter2#write(java.sql.Statement, boolean)
	 */
	public void write(Statement stmt, boolean hasFirstResult)
	throws IOException 
	{
		 xmlWriter2.write(stmt, hasFirstResult);
		 transform() ;
	}

	/**
	 * @see com.gonetix.framework.dbpublish.IPublishWriter#write(java.sql.ResultSet)
	 */
	public void write(ResultSet rs) throws IOException 
	{
		 xmlWriter2.write(rs);
		 transform() ;
	}
	
	
	protected void transform() throws IOException 
    {	
    	try{    		
    		XSLTTransformer transformer = new XSLTTransformer();
    		transformer.init(this.xslStylesheet);
    		transformer.setOutput(this.out);    		
    		transformer.transform(XML_TMP_FILE_NAME);
    	}catch(Exception ex){
    		ex.printStackTrace();
    		throw new IOException("Failed to transform file. \n" + ex.toString());
    	}finally{    		
    		if ( isDev == false )
    		{
    			File f = new File(XML_TMP_FILE_NAME);
    			f.delete();
    		}
    	}   	
    	
    }

}
