/*
 * XLSWriter.java
 *
 * Created on August 8, 2002, 3:52 PM
 */

package com.gonetix.framework.dbpublish;

import java.io.*;
import java.util.*;
import java.sql.*;
import org.apache.poi.hssf.usermodel.*;
import com.gonetix.jextract.*;

/**
 *
 * @author  SLiu
 */
public class XLSWriter extends PublishWriter{
    
    
    private int maxRowPerSheet = 300;
    private final short MAX = Short.MAX_VALUE;
    private HSSFCellStyle[]  headerStyles = null;
    private HSSFCellStyle[]  dataStyles = null;
    private String[] headerAlignments = null;
    private String[] dataAlignments = null;
    private int defaultColumnWidth = 1;
    HSSFWorkbook wb = new HSSFWorkbook(); 
    
    /** Creates a new instance of XLSWriter */
    public XLSWriter(OutputStream out)
    {
        super(out);            
    }
    
    /** Creates a new instance of XLSWriter */
    public XLSWriter()
    {
        super(null);            
    }
    
    
    public void write(ResultSet rs) throws IOException
    {
        short rownum = 0;
        short rowcount = 0;        
        try{                    
            ResultSetMetaData rsmd = rs.getMetaData();
            int colcount = rsmd.getColumnCount();
            String[] colnames = new String[colcount];
            int[] displaySizes = new int[colcount];
            int[] sqlTypes = new int[colcount];
            doFormats = new boolean[colcount]; 
                        
            for(int i=0; i<colcount; i++)
            {
                colnames[i] = rsmd.getColumnName(i+1);                 
                displaySizes[i] = 1; //10 chars wide as default.
                sqlTypes[i] = rsmd.getColumnType(i+1);          
            } 
            
            this.parseDataFormat(doFormats, this.formats);                        
            HSSFSheet sheet = wb.createSheet();   
                     
            if (this.customHeader!=null){
            	headerStyles = createStyles(wb, colcount, this.headerAlignments, null);
            	printHeader(sheet, displaySizes, customHeader.split(","), headerStyles);
            	rowcount++;	
            }else if(this.doPrintHeader){
            	headerStyles = createStyles(wb, colcount, this.headerAlignments, null);
            	printHeader(sheet, displaySizes, colnames, headerStyles); 
            	rowcount++;	           	
            }
                      
            this.dataStyles = createStyles(wb, colcount, this.dataAlignments, this.formats);
            String value = null;            
            while (rs!=null && rs.next())
            {
                HSSFRow row = sheet.createRow(rowcount++);                
                setRowValue(wb, row,  rs, colcount, sqlTypes, displaySizes, 
                					doFormats, formats, dataStyles);
                if (rowcount==MAX)
                {                	
                	setColumnWidth(sheet, displaySizes);
                	sheet = wb.createSheet(); 
                	rowcount=0;           		
                }
            }
            setColumnWidth(sheet, displaySizes);
            
        }catch(Exception ex){
            ex.printStackTrace();
            throw new IOException(ex.toString());
        }finally{
            wb.write(out);                      
        }        
    } 
    
    private HSSFCellStyle[] createStyles(HSSFWorkbook wb, int colcount, 
    									  String[] alignments, String[] dataFormats)
    {
    	HSSFCellStyle[] styles = new HSSFCellStyle[colcount];
    	if (alignments==null) alignments = new String[0];
    	if (formats==null) formats = new String[0];
    	for(short i=0; i<colcount; i++)
        {
              HSSFCellStyle cellStyle = wb.createCellStyle();
			  if(dataFormats!=null && i<dataFormats.length) 
			  {			  	
			  	cellStyle.setDataFormat(HSSFDataFormat.getFormat(dataFormats[i]));
			  	String[] fs = dataFormats[i].split("\\$|(|)|;"); 
			  }
              if(alignments!=null && i<alignments.length)
              {
              	 char align = alignments[i].charAt(0);
              	 short cellAlignment = HSSFCellStyle.ALIGN_GENERAL;
              	 switch(align)
              	 {
              	 	case 'R':
              	 	case 'r':
              	 		cellAlignment = HSSFCellStyle.ALIGN_RIGHT;
              	 		break;
              	 	case 'L':
              	 	case 'l':
              	 		cellAlignment = HSSFCellStyle.ALIGN_LEFT;
              	 		break;
              	 	case 'C':
              	 	case 'c':
              	 		cellAlignment = HSSFCellStyle.ALIGN_CENTER;
              	 		break;
              	 	case 'J':
              	 	case 'j':
              	 		cellAlignment = HSSFCellStyle.ALIGN_JUSTIFY;
              	 		break;
              	 }
              	
              	 cellStyle.setAlignment(cellAlignment);
              }
        	styles[i] = cellStyle;
        } 
    	
    	return styles;
    }
    
    
    protected void printHeader(HSSFSheet sheet, int[] displaySizes, 
    							String[] headers, HSSFCellStyle[] styles)
    {
    	int len = displaySizes.length;
    	if (len>headers.length) len = headers.length;
    	HSSFRow row = sheet.createRow((short)0);
    	
    	for(short i=0; i<len; i++)
        {
              HSSFCell cell = row.createCell(i);
              String header = headers[i].trim(); 
              cell.setCellValue(header); 
              if (styles!=null && i<styles.length) cell.setCellStyle(styles[i]);
              int hlen = header.length() + 2;
              if(hlen>displaySizes[i]) 
              			displaySizes[i] = hlen;             
        } 
    }
    
    
    
    protected void setColumnWidth(HSSFSheet sheet, int[] displaySizes) 
    {
    	for(short i=0; i<displaySizes.length; i++)
        {
             int size =  displaySizes[i]*256;
             short colsize = (size>MAX)?MAX:(short)size;             
             sheet.setColumnWidth(i, colsize);       
        } 
    }  
    
     
    
    protected void setRowValue(HSSFWorkbook wb, HSSFRow row, 
    							ResultSet rs, int colcount, 
    							int[] types, int[] displaySizes, 
    							boolean[] dofmts, String[] dataFormats,
    							HSSFCellStyle[] styles)
    throws Exception
    {
    	for(short j=0; j<colcount; j++)
        {
            HSSFCell cell = row.createCell(j);                   
            String value = rs.getString(j+1);
            if (value!=null) value = value.trim();
            switch(types[j]) 
            {
            	case Types.DATE:
            	case Types.TIME:
            	case Types.TIMESTAMP:            		
            		cell.setCellValue(rs.getDate(j+1));
            		if (dofmts[j]){
            			displaySizes[j] = dataFormats[j].length() + 5;
            		}else{
            			displaySizes[j] = 20;
            		}
            		break;
            	case Types.BIGINT:
            	case Types.INTEGER:
            	case Types.SMALLINT:
            	case Types.FLOAT:
            	case Types.BIT:
            	case Types.DOUBLE:
            	case Types.DECIMAL:
            	case Types.REAL:
            	case Types.NUMERIC:
            		cell.setCellValue(rs.getDouble(j+1));            		
            		if ( dofmts[j] ){
            			 int len = getNumericFormatStringLength(dataFormats[j], rs.getDouble(j+1)) + 2;            			 
            			 if (len > displaySizes[j]) {
            			 	displaySizes[j]= len;
            			 }   
            		}else{
            			if (value!=null){
            				int len = value.length() + 2;
            				if (len > displaySizes[j]) {
            			 		displaySizes[j]= len;
            			 	}   
            				
            			}
            		}            		    		
            		break;
            	default:            		
            		cell.setCellValue(value);
            		if ( value!=null && value.length()>displaySizes[j]) displaySizes[j]= value.length()+2;
            		break;
            		
            }
            cell.setCellStyle(styles[j]);
                  
        } 
    }
   
    protected int getNumericFormatStringLength(String pattern, double dataValue)
	{
		String[] tokens = pattern.split("\\(|_|\\)|;");
		String p = null;
		for(int i=0; i<tokens.length; i++)
	 	{
	 		String token = tokens[i].trim();
	 		if (token.length()>0){
	 			 p = token;
	 			 break;
	 		}
	 	}
	 	if ( p!=null)
	 	{
	 		java.text.DecimalFormat df = new java.text.DecimalFormat(p);
	 		String out = df.format(dataValue);
	 		return out.length();	 		
	 	}		
		return 0;
	}

	/**
	 * @see com.gonetix.framework.dbpublish.IPublishWriter#initWriter(java.util.Properties)
	 */
	public void initWriter(Properties prop) 
	{
		super.initWriter(prop);
		String p = prop.getProperty("headerAlignment");
		if (p!=null)
		{
			this.headerAlignments = p.split(",");
		}
        p = prop.getProperty("dataAlignment");
        if (p!=null)
		{
			this.dataAlignments = p.split(",");
		}
		p = prop.getProperty("defaultColumnWidth");
		if (p!=null)
		{
			this.defaultColumnWidth = Integer.valueOf(p).intValue();
		}
	}

	/**
	 * @see com.gonetix.framework.dbpublish.IPublishWriter#close()
	 */
	public void close() throws IOException 
	{
		
		super.close();
	}

}
