/*
 * XMLWriter.java
 *
 * Created on August 8, 2002, 5:19 PM
 */

package com.gonetix.framework.dbpublish;

import java.io.*;
import java.util.*;
import java.sql.*;
import com.gonetix.framework.util.StringUtil;
/**
 *
 * @author  SLiu
 */
public class XMLWriter extends PublishWriter {
    
    public static final int STYLE_ELEMENT = 1;
    public static final int STYLE_ATTRIBUTE = 2;
    
    protected static final char[] ELSTART = new char[] {'<'};
    protected static final char[] ENDSTART = new char[] {'<', '/'};
    protected static final char[] ELGT = new char[] {'>'};
    protected static final char[] ELEND = new char[] {'/', '>'};
    protected static final char[] NEWLINE = new char[] {'\n'};
    
    protected char[] quote = new char[] {'\''};
    protected char[] space = new char[] {' '};
    protected char[] asignment = new char[] {'='};
    
    protected BufferedWriter pw = null;    
    protected String xmlRootName = "datas";
    protected String xmlRowName = "row";
    protected boolean enableDtType = false;    
    protected int xmlStyle = STYLE_ELEMENT;
    
    /** Creates a new instance of XMLWriter */
    public XMLWriter(OutputStream out) {
        super(out);
        try{
            pw =new BufferedWriter( new PrintWriter(this.out));
        }catch(Exception e){
            ;
        }
    }
    
   public XMLWriter() { super();  }
    
    public void write(ResultSet rs) throws IOException {
         try{
            ResultSetMetaData rsmd = rs.getMetaData();
            int colcount = rsmd.getColumnCount();
            int[] colType = getColumnTypes(rsmd);            
            String[] colnames = getColumnNames(rsmd);               
            String value = null;            
            int rowcount = 0;
            pw.write(ELSTART ); pw.write(xmlRootName); pw.write(ELGT);pw.write(NEWLINE);
            if (customHeader!=null) {
                pw.write(ELSTART ); pw.write("header"); pw.write(ELGT);
                pw.write(customHeader);
                pw.write(ENDSTART); pw.write("header"); pw.write(ELGT); 
                pw.write(lineSeparator);
            }
            while(rs!=null && rs.next()){
                rowcount++;
                switch( xmlStyle) {                    
                    case STYLE_ATTRIBUTE:
                        writeStyleAttribute(pw, colcount, colnames, colType, rs);
                    break;
                    case  STYLE_ELEMENT:                        
                    default:
                        writeStyleElement(pw, colcount, colnames, colType, rs);
                        break;
                }                            
            }
            if (customTrailer!=null) {
                super.prepareCustomTrailer(rowcount);
                pw.write(ELSTART ); pw.write("trailer"); pw.write(ELGT);
                pw.write(customTrailer);
                pw.write(ENDSTART); pw.write("trailer"); pw.write(ELGT); 
                pw.write(lineSeparator);
            }
            pw.write(ENDSTART ); pw.write(xmlRootName); pw.write(ELGT);
            pw.flush();
        }catch(Exception ex){
            throw new IOException(ex.getMessage());
        }
    }
    
    public void writeStyleElement(Writer w, int colcount, String[] colnames, int[] colType, ResultSet rs)  
    throws SQLException, IOException
    {
        String value = null;       
         w.write(ELSTART); w.write(xmlRowName); w.write(ELGT); 
        for(int i=0; i<colcount; i++){
            w.write(ELSTART); w.write(colnames[i]); w.write(ELGT);            
            value = getString(rs, i+1, colType[i]);
            w.write((value!=null)?StringUtil.encodeXMLEntity(value):nullChar);
             w.write(ENDSTART); w.write(colnames[i]); w.write(ELGT); 
         } 
         w.write(ENDSTART); w.write(xmlRowName); w.write(ELGT); 
         w.write(lineSeparator);
    }
    
    public void writeStyleAttribute(Writer w, int colcount, String[] colnames, int[] colType, ResultSet rs) 
    throws SQLException, IOException
    {
        String value = null;       
        w.write(ELSTART); w.write(xmlRowName); w.write(space); 
        for(int i=0; i<colcount; i++){
           w.write(colnames[i]); w.write(asignment);w.write(quote);            
            value = getString(rs, i+1, colType[i]);
            w.write((value!=null)?StringUtil.encodeXMLEntity(value):nullChar);
            w.write(quote); w.write(space); 
         } 
        w.write(ELEND);
        w.write(lineSeparator);
    }
    
    public void close() throws IOException
    {
        pw.flush();
        pw.close();
    }
    
    public void setXmlStyle(int value){
        this.xmlStyle = value;
    }
    
    public void setXmlRootName(String value)
    {
        this.xmlRootName = value;
    }
    
    public void setXmlRowName(String value){
        this.xmlRowName = value;
    }
    
    public void initWriter(java.util.Properties prop){
        super.initWriter(prop);
        if ( prop.getProperty("xmlStyle")!=null) setXmlStyle(Integer.valueOf(prop.getProperty("xmlStyle")).intValue());
        if ( prop.getProperty("xmlRootName")!=null) setXmlRootName(prop.getProperty("xmlRootName"));
        if ( prop.getProperty("xmlRowName")!=null) setXmlRootName(prop.getProperty("xmlRowName"));        
    }
    
    public void setQuote(char[] value){
        this.quote = value;
    }
    
    public void setSpace(char[] value){
        this.space = value;
    }
    
    public void setOutput(String filepath) throws IOException
    {        
        this.pw =new BufferedWriter( new FileWriter(filepath));
    }
   
}
