/*
 * XMLWriter.java
 *
 * Created on August 8, 2002, 5:19 PM
 */

package com.gonetix.framework.dbpublish;

import java.io.*;
import java.util.*;
import java.sql.*;
import com.gonetix.framework.util.StringUtil;
/**
 *
 * @author  SLiu
 */
public class XMLWriter2 extends PublishWriter implements IPublishWriter2
{
    
    public static final int STYLE_ELEMENT = 1;
    public static final int STYLE_ATTRIBUTE = 2;
    
    protected static final char[] ELSTART = new char[] {'<'};
    protected static final char[] ENDSTART = new char[] {'<', '/'};
    protected static final char[] ELGT = new char[] {'>'};
    protected static final char[] ELEND = new char[] {'/', '>'};
    protected static final char[] NEWLINE = new char[] {'\n'};
    
    protected char[] quote = new char[] {'\''};
    protected char[] space = new char[] {' '};
    protected char[] asignment = new char[] {'='};
    
    protected BufferedWriter pw = null; 
    protected String xmlContainerName = "dataset";
    protected String xmlRootName = "resultset";
    protected String xmlRowName = "row";
    protected boolean enableDtType = false;    
    protected int xmlStyle = STYLE_ELEMENT;
    private String encoding = null;
    private String dtd = null;
    private String nnsSchema = null;
    private String nsSchema = null;
    private String xsl = null;
    
    /** Creates a new instance of XMLWriter */
    public XMLWriter2(OutputStream out) {
        super(out);
        try{
            pw =new BufferedWriter( new PrintWriter(this.out));
        }catch(Exception e){
            ;
        }
    }
    
   public XMLWriter2() { super();  }
   
   public void setEncoding(String encodingName) throws IOException
   {
       if ( encoding != null ) throw new IOException("XML Encoding has been set already.");
       if ( encodingName == null ) encodingName = "UTF-8";
       this.encoding = encodingName;
       pw.write("<?xml version=\"1.0\" encoding=\"");
       pw.write(encodingName);
       pw.write("\"?>\n");       
   }
   
   public void setXSLStylesheet(String path) throws IOException
   {
       if ( xsl != null ) throw new IOException("XSL stylesheet has been set already.");
       if ( path == null ) return;
       pw.write("<?xml-stylesheet type=\"text/xsl\" href=\"");
       pw.write(path);
       pw.write("\"?>\n");
   }
   
   public void setXMLNoNameSpaceSchema(String path) throws IOException
   {
       if ( dtd != null || nsSchema!=null || nnsSchema!=null ) throw new IOException("XML DTD or Schema has been set already.");
   
   
   }
   
   public void setXMLDTD(String path) throws IOException
   {
       if ( dtd != null || nsSchema!=null || nnsSchema!=null ) throw new IOException("XML DTD or Schema has been set already.");
       if ( path == null ) return;
       pw.write("<!DOCTYPE ");
       pw.write(xmlContainerName);
       pw.write(" SYSTEM \"");
       pw.write(path);
       pw.write("\">\n");
   }
    
    public void write(ResultSet rs) throws IOException {
         try{
            ResultSetMetaData rsmd = rs.getMetaData();
            int colcount = rsmd.getColumnCount();
            int[] colType = getColumnTypes(rsmd);            
            String[] colnames = getColumnNames(rsmd);               
            String value = null;            
            int rowcount = 0;
            pw.write(ELSTART ); pw.write(xmlRootName); pw.write(ELGT);pw.write(NEWLINE);
            if (customHeader!=null) {
                pw.write(ELSTART ); pw.write("header"); pw.write(ELGT);
                pw.write(customHeader);
                pw.write(ENDSTART); pw.write("header"); pw.write(ELGT); 
                pw.write(NEWLINE);
            }
            while(rs!=null && rs.next()){
                rowcount++;
                switch( xmlStyle) {                    
                    case STYLE_ATTRIBUTE:
                        writeStyleAttribute(pw, colcount, colnames, colType, rs);
                    break;
                    case  STYLE_ELEMENT:                        
                    default:
                        writeStyleElement(pw, colcount, colnames, colType, rs);
                        break;
                }                            
            }
            if (customTrailer!=null) {
                super.prepareCustomTrailer(rowcount);
                pw.write(ELSTART ); pw.write("trailer"); pw.write(ELGT);
                pw.write(customTrailer);
                pw.write(ENDSTART); pw.write("trailer"); pw.write(ELGT); 
                pw.write(NEWLINE);
            }
            pw.write(ENDSTART ); pw.write(xmlRootName); pw.write(ELGT);
            pw.write(NEWLINE);
            pw.flush();
        }catch(Exception ex){
            throw new IOException(ex.getMessage());
        }
    }
    
    public void writeStyleElement(Writer w, int colcount, String[] colnames, int[] colType, ResultSet rs)  
    throws SQLException, IOException
    {
        String value = null;       
         w.write(ELSTART); w.write(xmlRowName); w.write(ELGT); 
        for(int i=0; i<colcount; i++){
            w.write(ELSTART); w.write(colnames[i]); w.write(ELGT);            
            value = getString(rs, i+1, colType[i]);
            w.write((value!=null)?StringUtil.encodeXMLEntity(value):nullChar);
             w.write(ENDSTART); w.write(colnames[i]); w.write(ELGT); 
         } 
         w.write(ENDSTART); w.write(xmlRowName); w.write(ELGT); 
         w.write(NEWLINE);
    }
    
    public void writeStyleAttribute(Writer w, int colcount, String[] colnames, int[] colType, ResultSet rs) 
    throws SQLException, IOException
    {
        String value = null;       
        w.write(ELSTART); w.write(xmlRowName); w.write(space); 
        for(int i=0; i<colcount; i++){
           w.write(colnames[i]); w.write(asignment);w.write(quote);            
            value = getString(rs, i+1, colType[i]);
            w.write((value!=null)?StringUtil.encodeXMLEntity(value):nullChar);
            w.write(quote); w.write(space); 
         } 
        w.write(ELEND);
        w.write(NEWLINE);
    }
    
    public void close() throws IOException
    {
        pw.flush();
        pw.close();
    }
    
    public void setXmlStyle(int value){
        this.xmlStyle = value;
    }
    
    public void setXmlRootName(String value)
    {
        this.xmlRootName = value;
    }
    
    public void setXmlRowName(String value){
        this.xmlRowName = value;
    }
    
    public void initWriter(java.util.Properties prop)
    {
        super.initWriter(prop);
        if ( prop.getProperty("xmlStyle")!=null) setXmlStyle(Integer.valueOf(prop.getProperty("xmlStyle")).intValue());
        try{
            if ( prop.getProperty("xmlEncoding")!=null) this.setEncoding(prop.getProperty("xmlEncoding"));
            if ( prop.getProperty("dtd")!=null) this.setXMLDTD(prop.getProperty("dtd"));
            if ( prop.getProperty("stylesheet")!=null) this.setXSLStylesheet(prop.getProperty("stylesheet"));
        }catch(Exception ex){
            ;
        }
        if ( prop.getProperty("xmlRootName")!=null) setXmlRootName(prop.getProperty("xmlRootName"));
        if ( prop.getProperty("xmlRowName")!=null) setXmlRootName(prop.getProperty("xmlRowName"));        
    }
    
    public void setQuote(char[] value){
        this.quote = value;
    }
    
    public void setSpace(char[] value){
        this.space = value;
    }
    
    public void setOutput(String filepath) throws IOException
    {        
        this.pw =new BufferedWriter( new FileWriter(filepath));
    }
    
    public void setOutput(Writer writer) throws IOException
    {        
        this.pw = new BufferedWriter(writer);
    }
   
    public void write(java.sql.Statement stmt, boolean hasFirstResult) throws java.io.IOException 
    {
        int b = -1;
        pw.write(ELSTART ); pw.write(xmlContainerName); pw.write(ELGT);pw.write(NEWLINE);
        try{
            while ( hasFirstResult || (b=stmt.getUpdateCount())!=-1 ){            
                  if ( hasFirstResult ){                  
                      ResultSet rs = stmt.getResultSet();
                      write(rs);             
                  }else if (b!=-1){                  
                     pw.write("\n<executeUpdate>");
                     pw.write("Total "); 
                     pw.write(b); 
                     pw.write(" rows affected."); 
                     pw.write("</executeUpdate>\n");
                     b = -1;
                  }else{
                     pw.write("\n<executeDDL>");                 
                     pw.write("This command does not return anything."); 
                     pw.write("</executeDDL>\n");
                  }
                  hasFirstResult = stmt.getMoreResults();
            }
        }catch(SQLException ex){
            throw new IOException(ex.toString());
        }
        pw.write(ENDSTART ); pw.write(xmlContainerName); pw.write(ELGT);
        pw.write(NEWLINE);
        pw.flush();
    }
    
}
