package com.gonetix.framework.textengine;

import java.io.*;
import java.util.*;

/**
 * @author SLiu
 *
 */
public class PlainTextDataFileReader extends TextDataFileReader {

	/**
	 * Constructor for PlainTextDataFileReader.
	 * @param systemId
	 * @param fileSpec
	 * @throws IOException
	 */
	public PlainTextDataFileReader(String systemId, FileSpec fileSpec)
		throws IOException {
		super(systemId, fileSpec);
	}

	/**
	 * Constructor for PlainTextDataFileReader.
	 * @param file
	 * @param fileSpec
	 * @throws IOException
	 */
	public PlainTextDataFileReader(File file, FileSpec fileSpec)
		throws IOException {
		super(file, fileSpec);
	}

	/**
	 * Constructor for PlainTextDataFileReader.
	 * @param reader
	 * @param fileSpec
	 * @throws IOException
	 */
	public PlainTextDataFileReader(Reader reader, FileSpec fileSpec)
		throws IOException {
		super(reader, fileSpec);
	}

	/**
	 * @see com.gonetix.framework.textengine.TextDataFileReader#openToReader()
	 */
	protected void openToRead() throws IOException
	{
		SectionSpec headerSpec = fileSpec.getHeaderSpec();
		SectionSpec trailerSpec = fileSpec.getTrailerSpec();
		
		int rowcount = -1;		
		if (headerSpec!=null || trailerSpec!=null)
		{
			rowcount = lineCount(this.reader);
			int xcount = 0;
			if (headerSpec!=null) xcount = xcount + headerSpec.getRowcount();
			if (trailerSpec!=null) xcount = xcount + trailerSpec.getRowcount();
			rowcount = rowcount - xcount;
		}
		
		BufferedReader breader = new BufferedReader(this.reader);
		
		if (headerSpec!=null)
		{
			handleRawData(breader, this.headerHandlers, headerSpec.getRowcount());
			
		}
		
		if (rowcount==-1 || rowcount>0)
		{
			handleRawData(breader, this.contentHandlers, rowcount);
			
		}
		
		if (trailerSpec!=null)
		{
			handleRawData(breader, this.trailerHandlers, trailerSpec.getRowcount());
			
		}
		
		
	}
	
	
	protected void handleRawData(BufferedReader breader, Vector handlers, int rowcount) throws IOException
	{
		String line = null;
		if(rowcount==-1){
			while((line=breader.readLine())!=null)
			{
				RawRowData row = new RawRowData();
				row.setRawCharacterData(line);
				int size = handlers.size();
				for(int j=0; j<size; j++){
					DefaultSectionHandler handler = (DefaultSectionHandler) handlers.elementAt(j);
					handler.handle(row);
				}				
			}			
		}else{		
			for(int i=0; i<rowcount; i++){
			    line = breader.readLine();
				if(line==null){
					throw new IOException("Line number out of bound exception.");
				}else{
					RawRowData row = new RawRowData();
					row.setRawCharacterData(line);
					int size = handlers.size();
					for(int j=0; j<size; j++){
						DefaultSectionHandler handler = (DefaultSectionHandler) handlers.elementAt(j);
						handler.handle(row);
					}
					
				}			
			}
		}		
	}
	
	
	
	protected int lineCount(Reader reader) throws IOException
	{
		BufferedReader breader = new BufferedReader(reader);
		String line = null;
		int count=0;
		while((line=breader.readLine())!=null){
			count++;
		}
		reader.reset();
		return count;		
	}
	

}
