package com.gonetix.framework.textengine;

/**
 * @author SLiu
 * 
 */
import java.io.*;
import java.util.*;

public abstract class TextDataFileReader 
{	
	
	Reader reader = null;
	FileSpec fileSpec = null;
	Vector headerHandlers = null;
	Vector contentHandlers = null;
	Vector trailerHandlers = null;
	
	/**
	 * Method TextDataFileReader.
	 * @param systemId
	 * @param fileSpec
	 * @throws IOException
	 */
	/**
	 * Constructor for TextDataFileReader.
	 */
	public TextDataFileReader(String systemId, FileSpec fileSpec) throws IOException
	{
		this(new File(systemId), fileSpec); 
	}
	
	
	/**
	 * Method TextDataFileReader.
	 * @param file
	 * @param fileSpec
	 * @throws IOException
	 */
	public TextDataFileReader(File file, FileSpec fileSpec) throws IOException
	{
		this(new FileReader(file), fileSpec);
	}
	
	
	/**
	 * Method TextDataFileReader.
	 * @param reader
	 * @param fileSpec
	 * @throws IOException
	 */
	public TextDataFileReader(Reader reader, FileSpec fileSpec) throws IOException
	{
		this.reader = reader;
		this.fileSpec = fileSpec;
		headerHandlers = new Vector();
		contentHandlers = new Vector();
		trailerHandlers = new Vector();
		openToRead();
	}
	
	
	
	/**
	 * Method openToReader is needed to be implemented by concreted subclass.
	 * 
	 */
	protected abstract void openToRead() throws IOException;
	
	
	
	
	/**
	 * Method close.
	 * @throws IOException
	 */
	public void close() throws IOException
	{
		if (reader!=null) reader.close();
	}
	
	/**
	 * Method addHeaderHandler.
	 * @param handler
	 */
	public void addHeaderHandler(DefaultSectionHandler handler)
	{
		headerHandlers.add(handler);
	}	
	
	/**
	 * Method addTrailerHandlers.
	 * @param handler
	 */
	public void addTrailerHandlers(DefaultSectionHandler handler)
	{
		trailerHandlers.add(handler);
	}	
	
	/**
	 * Method addContentHandlers.
	 * @param handler
	 */
	public void addContentHandlers(DefaultSectionHandler handler)
	{
		contentHandlers.add(handler);
	}	

}
