/*
 * StringUtil.java
 *
 * Created on August 9, 2002, 3:06 PM
 */

package com.gonetix.framework.util;


import java.util.*;
import java.io.*;
/**
 *
 * @author  SLiu
 */
public class StringUtil {
    
    public static final String LINE_SEP = System.getProperty("line.separator");
    
    /** Creates a new instance of StringUtil */
    public StringUtil() {
    }
    
    public static String strip(String src){
        if(src==null || src.length()==0 ) return src;
        return src.replaceAll("\\s+", " ");
    }
    
    public static String decodeEntity(String src, String pattern, String value)
    {
        if(src==null || src.length() ==0 ) return src;
        src = src.replaceAll(pattern, value);
        return src;
    }
    
    public static String encodeXMLEntity(String src){
        src = src.replaceAll("&", "&#38;");
        src = src.replaceAll(">", "&#62;");
        src = src.replaceAll("<", "&#60;");
        src = src.replaceAll("'", "&#39;");
        src = src.replaceAll("/", "&#47;");
        return src;
    }
    
    
    public static void  printCharCompare(String src)
    {
        if ( src== null || src.length() ==0){
            System.out.println("NULL or Empty String.");
            return;
        }
        char[] cs = src.toCharArray();
        for(int i=0; i<cs.length; i++){
            System.out.println("Decimal Value=" + (int)cs[i] + " Char Reps=" + cs[i]);
        }
    }
    
    public static String getStackTrace(Throwable t) {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw, true);
        t.printStackTrace(pw);
        pw.flush();
        pw.close();
        return sw.toString();
    }
    
    
    public static java.util.Properties getOptions(String[] args) throws Exception
    {
        Properties prop = new Properties();
        try{
           for(int i=0; i<args.length; i++){
                 if ( args[i].startsWith("-") && (i+1)<args.length && !args[i+1].startsWith("-")){                     
                     prop.setProperty(args[i].substring(1), args[i+1]);
                 }
           }
        }catch(Exception e){
            throw new Exception("FATAL: Invalid command line parameter format.");
        }        
        return prop;
    }
    
    public static int[] splitIntString(String str, String regx)
    {
        String[] list = str.split(regx);
        int[] ret = new int[list.length];
        for(int i=0; i<list.length; i++){
            ret[i] = Integer.valueOf(list[i]).intValue();
        }
        return ret;
    }
    
    public static double[] splitDoubleString(String str, String regx)
    {
        String[] list = str.split(regx);
        double[] ret = new double[list.length];
        for(int i=0; i<list.length; i++){
            ret[i] = Double.valueOf(list[i]).doubleValue();
        }
        return ret;
    }
    
    public static void printProperties(Properties prop)
    {
         Enumeration enum = prop.propertyNames();
         while ( enum.hasMoreElements()){
            String key = (String)enum.nextElement();
            System.out.println(key + "=" + prop.getProperty(key));
         }
    }
    
    public static void printArrayString(String[] list)
    {
        for(int i=0; i<list.length; i++){
           System.out.print(list[i]);
           System.out.print(" ");
        }
        System.out.println();
    }
    

}
