/*
 * SimpleSQLReader.java
 *
 * Created on August 9, 2002, 11:36 AM
 */

package com.gonetix.framework.xmlEngine;

import com.gonetix.framework.util.*;

import java.io.*;
import java.util.*;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
/**
 *
 * @author  SLiu
 */
public class SimpleSQLReader extends DefaultHandler
{    
    private static final String dbUrlKey = "db.Url";
    private static final String dbDvrKey = "db.Driver";
    private static final String dbUserKey = "db.Username";
    private static final String dbPwdKey = "db.Password";   
    private static final String dbStmtKey = "db.Statement"; 
    
    protected String xmlfile;
    protected SQLStatement stmt;
    protected Vector params;
    
    protected Properties props;
    /** Creates a new instance of SimpleSQLReader */
    public SimpleSQLReader(String filepath) {
        this.xmlfile = filepath;
        this.props = new Properties();
        this.params = new Vector();
        initReader();
    }
    
    private void initReader() {
        XMLReader xmlReader = null;
        if ( xmlfile == null || xmlfile.length()<= 3) return;
        try{            
            SAXParserFactory spfactory = SAXParserFactory.newInstance();
            spfactory.setValidating(false);        
            SAXParser saxParser = spfactory.newSAXParser();
            xmlReader = saxParser.getXMLReader();            
            xmlReader.setContentHandler(this);
            xmlReader.setEntityResolver(this);
            xmlReader.setErrorHandler(this);            
            InputSource source = new InputSource(xmlfile);
            xmlReader.parse(source);            
        }catch(Exception ex){
            ex.printStackTrace();
        }
    }
    
    boolean started = false;    
    String key = null;
    StringBuffer value = new StringBuffer(100);
    String stmtType = null;
    String stmtId=null;
    
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
    throws SAXException 
    {
       value.setLength(0);
        if (qName.equals("driver")){
           key = dbDvrKey;
           started = true;
       }else if (qName.equals("url")){
            key = dbUrlKey;
           started = true;
       }else if (qName.equals("user")){
            key = dbUserKey;
           started = true;
       }else if (qName.equals("password")){            
           key = dbPwdKey;
           started = true;
       } else if (qName.equals("statement")){           
            stmtType = atts.getValue("type");
            stmtId = atts.getValue("id"); 
            started = true;
       } else if (qName.equals("param")){
            started = true;
            String type = atts.getValue("type");
            String foramt = atts.getValue("format");
            String paramValue = atts.getValue("value");
            String cmdIndex = atts.getValue("cmdIndex");            
            try{
                Parameter param;
                if (paramValue!=null && paramValue.equals("CMDLINE")){                    
                    param = new Parameter(type, paramValue, foramt);                     
                    param.setCmdIndex(cmdIndex);                    
                }else{
                    param = new Parameter(type, paramValue, foramt);
                }
                params.add(param);               
            }catch(Exception e){
                throw new SAXException("FATAL: " + e.toString());
            }            
            
       }
    }
    
    public void characters(char[] ch, int start, int length)
    throws SAXException 
    {       
        String content = (new String(ch, start, length)); 
        if(content!=null && length>0){            
         content=StringUtil.decodeEntity(content, "&#60;|&lt;", "<");         
         content=StringUtil.decodeEntity(content, "&#62;|&lt;", "<");        
         content=StringUtil.decodeEntity(content, "&#47;", "/");
        }
        if( started) value.append(content);
    }
    
    public void endElement(String namespaceURI, String localName, String qName)
    throws SAXException 
    {
        if ( started ){
            String pvalue = value.toString();
            value.setLength(0);
            pvalue = StringUtil.strip(pvalue);            
            if (qName.equals("statement")){
                try{
                    stmt = new SQLStatement(null, stmtType, pvalue);
                }catch(Exception e){
                    throw new SAXException("FATAL: " + e.toString());
                }
                stmtType = null;
            }else if(qName.equals("param")){
                ;
            }else if ( key != null && pvalue != null ){
                props.setProperty(key, pvalue);                 
            }            
        }
        key = null ;        
        started = false;
    }
    
    
    public Properties getProperties()
    {
        return this.props;
    }
    
    public SQLStatement getSQLStatement()
    {
        return this.stmt;
    }
    
    public Parameter[] getParams()
    {
        Parameter[] ps = new Parameter[params.size()];
        params.copyInto(ps);
        return ps;
    }
    
}
