/*
 * PDFRender.java
 *
 * Created on August 13, 2002, 11:01 AM
 */

package com.gonetix.framework.xsltEngine;


import com.gonetix.framework.util.*;
import com.gonetix.util.DebugUtil;

import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

import org.apache.fop.apps.Driver;
import org.apache.fop.apps.Version;
import org.apache.fop.apps.XSLTInputHandler;
import org.apache.fop.messaging.MessageHandler;
import org.apache.avalon.framework.logger.ConsoleLogger;
import org.apache.avalon.framework.logger.Logger;

import com.gonetix.framework.dbpublish.*;
import java.io.*;
import java.sql.*;
/**
 *
 * @author  SLiu
 */
public class PDFRender {
    
    private OutputStream pdfOut = null;    
    private StringReader foReader = null;
    private StringWriter foWriter = null;
    private XSLTTransformer xlstTransformer = null;
    private XMLBuilder xmlBuilder = null;
    
    /** Creates a new instance of PDFRender */
    public PDFRender() throws Exception
    {
            this(null);
    }
    
    public PDFRender(OutputStream out) throws Exception
    {
            this.pdfOut = out;            
            foWriter = new StringWriter();            
            xlstTransformer = new XSLTTransformer();
            xlstTransformer.setOutput(foWriter);
            xmlBuilder = new XMLBuilder();
    }
    
    public void setStyleSheet(Reader reader) throws Exception
    {
        xlstTransformer.init(reader);
    }
    
    public void setStyleSheet(String xslfile) throws Exception
    {
        xlstTransformer.init(xslfile);
    }
    
        
    
    public void render(Reader foin) throws Exception
    {
        InputSource fosource = new InputSource(foin);
        renderFo(fosource , pdfOut);
    }
    
    public void render(ResultSet rs) throws Exception
    {
        StringWriter xmlw = new StringWriter();        
        xmlBuilder.buildXML(xmlw, rs);
        xlstTransformer.transform(new StringReader(xmlw.toString()));
        InputSource fosource = new InputSource(new StringReader(foWriter.toString()));
        renderFo(fosource , pdfOut); 
        xmlw.close();        
    }
    
    private void renderFo(InputSource foFile, OutputStream out)throws Exception
    {
        Logger log = new ConsoleLogger(ConsoleLogger.LEVEL_WARN);
	    MessageHandler.setScreenLogger(log);
        Driver driver = new Driver(foFile, out);
        driver.setLogger(log);
        driver.setRenderer(Driver.RENDER_PDF);
        driver.run(); 
    }
    
    public void renderFile(String xmlfile, String xslFile, OutputStream out)throws Exception
    {
        if ( out==null ) out = pdfOut;        
        XSLTInputHandler input = new XSLTInputHandler(new File(xmlfile), new File(xslFile));
        Logger log = new ConsoleLogger(ConsoleLogger.LEVEL_WARN);
	    MessageHandler.setScreenLogger(log);
        Driver driver = new Driver();
        driver.setLogger(log);
        driver.setRenderer(Driver.RENDER_PDF);
        driver.setOutputStream(out);
        driver.render(input.getParser(), input.getInputSource());
    }
    
    public void close()  throws IOException
    {
        pdfOut.close();
        foReader.close();
        foWriter.close();
    }
    
    
    public static void main(String[] args) throws Exception
    {   
        java.util.Properties options = StringUtil.getOptions(args);
        String foFile = options.getProperty("fo");
        String xmlFile = options.getProperty("xml");
        String xslFile = options.getProperty("xsl");
        String pdfFile = options.getProperty("pdf");
        PDFRender pdfRender = null;
        if (pdfFile!=null) pdfRender = new PDFRender(new FileOutputStream(pdfFile));       
        if ( foFile != null && pdfRender!=null ){            
            pdfRender.render(new FileReader(foFile));
        }else if (xmlFile!=null && xslFile!=null && pdfRender!=null ){
            pdfRender.renderFile(xmlFile, xslFile,null);
        }else{
            printUsage();
        }
        if (pdfRender!=null ) pdfRender.close();
    } 
    
     public static void printUsage()
    {
        String usage="Usage:\n"
                     + "     java [options] com.gonetix.framework.xsltEngine.XSLTTransformer -xml XML_File_Path -xsl Style_Sheet_Path -pdf out_file_path"
                     + "\n\n"
                     + "-fo   File path to fo file\n"
                     + "-xml  File path to XML file\n"
                     + "-xsl  File path to XSL Style-Sheet file\n"
                     + "-pdf  File path to output PDF file\n";                     
        System.out.println(usage);
    }

}
