/*
 * RTFRender.java
 *
 * Created on August 13, 2002, 11:02 AM
 */

package com.gonetix.framework.xsltEngine;


import com.gonetix.framework.dbpublish.*;
import com.gonetix.framework.util.*;

import java.sql.*;
import java.io.*;
import org.jfor.jfor.converter.Converter;
import org.xml.sax.InputSource;
/**
 *
 * @author  SLiu
 */
public class RTFRender {
    
    private Writer rtfOut = null;
    private PipedReader foReader = null;
    private PipedWriter foWriter = null;
    private XSLTTransformer xlstTransformer = null;
    private XMLBuilder xmlBuilder = null;
   
    /** Creates a new instance of RTFRender */
    public RTFRender(Writer out) throws Exception
    {
        this.rtfOut = out;            
        foWriter = new PipedWriter();
        foReader = new PipedReader(foWriter);
        xlstTransformer = new XSLTTransformer();
        xlstTransformer.setOutput(foWriter);
        xmlBuilder = new XMLBuilder();
    }    
   
    
    public void setStyleSheet(Reader reader) throws Exception
    {
        xlstTransformer.init(reader);
    }
    
    public void setStyleSheet(String xslfile) throws Exception
    {
        xlstTransformer.init(xslfile);
    }
    
        
    
    public void render(Reader foin) throws Exception
    {
        InputSource fosource = new InputSource(foin);
        renderFo(fosource , rtfOut);
    }
    
    public void render(ResultSet rs) throws Exception
    {
        PipedWriter xmlw = new PipedWriter();
        PipedReader pr = new PipedReader(xmlw);
        xmlBuilder.buildXML(xmlw, rs);
        xlstTransformer.transform(pr);
        InputSource fosource = new InputSource(foReader);
        renderFo(fosource , rtfOut); 
        xmlw.close();
        pr.close();
    }
    
    private void renderFo(InputSource foFile, Writer out)throws Exception
    {
        
        Converter converter = new Converter(foFile, out, Converter.createConverterOption());
        
    }
    
    public void renderFile(String xmlfile, String xslFile, Writer out)throws Exception
    {
        if ( out==null ) out = rtfOut;
        StringWriter sw = new StringWriter();
        XSLTTransformer trans = new XSLTTransformer();
        trans.setOutput(sw);
        trans.init(xslFile);
        trans.transform(xmlfile);
        StringReader sr = new StringReader(sw.toString());
        InputSource fosource = new InputSource(sr);
        renderFo(fosource, out);
    }
    
    public void close()  throws IOException
    {
        rtfOut.close();
        foReader.close();
        foWriter.close();
    }
    
    
    public static void main(String[] args) throws Exception
    {   
        java.util.Properties options = StringUtil.getOptions(args);
        String foFile = options.getProperty("fo");
        String xmlFile = options.getProperty("xml");
        String xslFile = options.getProperty("xsl");
        String rtfFile = options.getProperty("rtf");
        RTFRender rtfRender = null;
        if (rtfFile!=null) rtfRender = new RTFRender(new FileWriter(rtfFile));
        if ( foFile != null && rtfFile!=null){            
            rtfRender.render(new FileReader(foFile));
        }else if (xmlFile!=null && xslFile!=null && rtfFile!=null){
            rtfRender.renderFile(xmlFile, xslFile,null);
        }else{
            printUsage();
        }
        if (rtfRender!=null) rtfRender.close();
    }  
    
    public static void printUsage()
    {
        String usage="Usage:\n"
                     + "     java [options] com.gonetix.framework.xsltEngine.XSLTTransformer -xml XML_File_Path -xsl Style_Sheet_Path -rtf out_file_path"
                     + "\n\n"
                     + "-fo   File path to fo file\n"
                     + "-xml  File path to XML file\n"
                     + "-xsl  File path to XSL Style-Sheet file\n"
                     + "-rtf  File path to output RTF file\n";                     
        System.out.println(usage);
    }

    
}
