/*
 * XSLTTransformer.java
 *
 * Created on August 13, 2002, 10:00 AM
 */

package com.gonetix.framework.xsltEngine;


import com.gonetix.framework.util.StringUtil;
// Imported TraX classes
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.*;

// Imported java classes
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.*;
/**
 *
 * @author  SLiu
 */
public final class XSLTTransformer {    
    
    private Transformer xsltTransformer = null;
    private javax.xml.transform.Result result = null;
    
    /** Creates a new instance of XSLTTransformer */
    public XSLTTransformer() {
    }
    
    public void init(String xslFilePath)
    throws TransformerException, TransformerConfigurationException, FileNotFoundException, IOException
    {
        initTransformer(new StreamSource(xslFilePath));
        
    }
    public void init(File xslFile)
    throws TransformerException, TransformerConfigurationException, FileNotFoundException, IOException
    {
        initTransformer( new StreamSource(xslFile));
        
    }
    public void init(InputStream xslFile)
    throws TransformerException, TransformerConfigurationException, FileNotFoundException, IOException
    {
        initTransformer( new StreamSource(xslFile));
        
    }
    
    public void init(Reader xslFile)
    throws TransformerException, TransformerConfigurationException, FileNotFoundException, IOException
    {
        initTransformer(new StreamSource(xslFile));        
    }
    
    
    public void setOutput(String value)
    throws TransformerException, TransformerConfigurationException, FileNotFoundException, IOException
    {
        result = new StreamResult(value);
    }
    
    public void setOutput(File value)
    throws TransformerException, TransformerConfigurationException, FileNotFoundException, IOException
    {
        result = new StreamResult(value);
    }
    
    public void setOutput(OutputStream value)
    throws TransformerException, TransformerConfigurationException, FileNotFoundException, IOException
    {
        result = new StreamResult(value);
    }
    
    public void setOutput(Writer value)
    throws TransformerException, TransformerConfigurationException, FileNotFoundException, IOException
    {
        result = new StreamResult(value);
    }
    
    
    public void transform(String value)
    throws TransformerException, TransformerConfigurationException, FileNotFoundException, IOException
    {
        Source xml = new StreamSource(value);
        xsltTransformer.transform(xml, result);
    }
    
    public void transform(File value)
    throws TransformerException, TransformerConfigurationException, FileNotFoundException, IOException
    {
        Source xml = new StreamSource(value);
        xsltTransformer.transform(xml, result);
    }
    
    public void transform(InputStream value)
    throws TransformerException, TransformerConfigurationException, FileNotFoundException, IOException
    {
        Source xml = new StreamSource(value);
        xsltTransformer.transform(xml, result);
    }
    
    public void transform(Reader value)
    throws TransformerException, TransformerConfigurationException, FileNotFoundException, IOException
    {
        Source xml = new StreamSource(value);
        xsltTransformer.transform(xml, result);
    }
     
    protected void initTransformer(Source src) throws TransformerException, TransformerConfigurationException
    {
        TransformerFactory tFactory = TransformerFactory.newInstance();	
	xsltTransformer = tFactory.newTransformer(src);
    }
    
    
    public static void main(String[] args) 
    throws Exception
    {   
	java.util.Properties options = StringUtil.getOptions(args);       
        String xmlFile = options.getProperty("xml");
        String xslFile = options.getProperty("xsl");
        String outFile = options.getProperty("out");
        if ( xmlFile!=null && xslFile!=null && outFile!=null){
            XSLTTransformer trans = new XSLTTransformer();
            trans.init(xslFile);
            trans.setOutput(outFile);
            trans.transform(xmlFile);
        }else{
            printUsage();
        }
        
    }
    
    
    public static void printUsage()
    {
        String usage="Usage:\n"
                     + "     java [options] com.gonetix.framework.xsltEngine.XSLTTransformer -xml XML_File_Path -xsl Style_Sheet_Path -out out_file_path"
                     + "\n\n\n"
                     + "-xml  File path to XML file\n"
                     + "-xsl  File path to XSL Style-Sheet file\n"
                     + "-out  File path to output file\n";
        System.out.println(usage);
    }

    
    
}
