/*
 * Main.java
 *
 * Created on August 13, 2002, 9:40 AM
 */

package com.gonetix.jextract;

import com.gonetix.framework.dbengine.QueryEngine;
import com.gonetix.framework.xmlEngine.*;
import com.gonetix.framework.dbpublish.*;
import com.gonetix.framework.util.StringUtil;

import java.io.*;
import java.util.*;
import java.sql.*;


/**
 *
 * @author  SLiu
 */
public class Main { 
    
    private static final String dbStmtKey = "db.Statement";
    private static final String XLS = "XLS";
    private static final String PDF = "PDF";
    private static final String PDF2 = "PDF2";
    private static final String DAT = "DAT";
    private static final String CSV = "CSV";
    private static final String RTF = "RTF";    
    private static final String XML = "XML"; 
    private static final String TRAN = "TRAN"; 
    
    public static void main(String[] args) throws Exception
    {                
        
        java.util.Properties options = StringUtil.getOptions(args);
        int paramIndex = -1;
        String[] params = null;        
        if ( args.length > 2 ){
            for(int i=2; i<args.length; i++){
                 if ( args[i].equals("-params")){
                     paramIndex = i;
                     break;
                 }
            }            
            if (paramIndex!= -1) {
                int pslength = args.length - paramIndex-1;                
                params = new String[pslength];
                System.arraycopy(args, paramIndex+1, params, 0, pslength); 
            }
        }        
        doExport(options, params);       
    }
    
    public static void doExport(Properties options, String[] params)
    throws Exception
    {
        
        String xmlFile = options.getProperty("src");
        String outFile = options.getProperty("out"); 
        String taskId = options.getProperty("task");
        
        if (xmlFile != null){
            XMLProcessor processor = new XMLProcessor();
            try{
                processor.parse(xmlFile);            
            }catch(Exception e1){
                e1.printStackTrace();
                System.exit(3704);
            }
            Properties globalProp = processor.getGlobalProperties();            
            QueryEngine qengine = new QueryEngine(globalProp);
            try{
                qengine.connect();
            }catch(Exception ex){
                ex.printStackTrace();
                System.exit(3702);
            }
            Task task = null;
            Task[] tasks = null;
            String[] tidlist = new String[0];
            if ( taskId!=null){
                tidlist = taskId.split(",");
                tasks = new Task[tidlist.length];
                for(int k = 0; k<tidlist.length; k++){
                    tasks[k] = processor.getTask(tidlist[k]);
                }
            }else{
                tasks = processor.getAllTask();
            }
            if ( tasks == null || tasks.length == 0 ){
                System.err.println("FATAL: No task found in XML file. --" + xmlFile);
                System.exit(3802);
            }
            for(int i=0; i<tasks.length; i++){
                try{
                    task = tasks[i];
                    if(task!=null) processTask(qengine, task, options, globalProp, params);  
                }catch(Exception ex){
                    ex.printStackTrace();
                }
            }           
            qengine.close();
        
        }else{
            printUsage();            
        }
    } 
    
    public static void processTask(QueryEngine qengine, Task task, Properties cmdProp, Properties globalProp, String[] params)
    throws Exception
    {
        boolean isQuery = true;
        IPublishWriter writer = null;
        String taskType = task.getProperty("type");
        if (taskType!=null && taskType.equals("insertOrUpdate") ) isQuery = false;        
        String format = cmdProp.getProperty("format");        
        String outFile = task.getProperty("out");
        
        if (isQuery){            
            if (outFile == null ){
                outFile = cmdProp.getProperty("out");
            }else{            
                if (outFile.indexOf("{")!= -1  && outFile.indexOf("}")!= -1){
                    outFile = StringFormater.format(outFile, cmdProp.getProperty("out").split(","));                
                }
            }
            if ( outFile == null){
              System.err.println("FATAL: output file path must be specified in command line or XML file.");
              return;
            }
            String writerClass = null;
            if (format!=null){
                  writerClass = getWriterClass(format);
            }else{
                 writerClass = task.getProperty("writer");
                if (writerClass==null || writerClass.length()==0 ) writerClass = getWriterClass(CSV);
            }
        
            writer =(IPublishWriter)Class.forName(writerClass).newInstance();
        }
        globalProp.putAll(task.getProperties());
        
        if (writer!=null){
            writer.initWriter(globalProp);
            writer.setOutput(outFile);
        }
        Parameter[] sqlParameters = task.getParams();
        adjustSqlParameters(sqlParameters, params);
        ResultSet rs = null; 
        switch (task.getStatementType())
        {
          case SQLStatement.PREPARED_STATEMENT:
                        qengine.prepare(task.getStatement());
                        if(isQuery){                            
                            rs = qengine.queryCache(sqlParameters);
                        }else{
                            qengine.executeCache(sqlParameters);
                        }
                        break;
         case SQLStatement.REGULAR_STATEMENT:
                        if(isQuery){
                            rs = qengine.query(task.getStatement());
                        }else{
                            qengine.execute(task.getStatement());
                        }
            
                        break;
         case SQLStatement.CALLABLE_STATEMENT:
                        break;
        }            
        if ( writer!=null &&  rs!=null) writer.write(rs);
        if ( rs!=null) qengine.closeResultSet(rs);           
        if ( writer!=null) writer.close();
    }
   
    private static void adjustSqlParameters(Parameter[] sqlParameters, String[] params) throws Exception
    {
      
        if (sqlParameters==null || params==null ) return;
        for(int i=0; i<sqlParameters.length; i++){            
           if ( sqlParameters[i].isFromCMDLine()){              
               sqlParameters[i].setCmdValue(params[sqlParameters[i].getCmdIndex()]);               
           }
       }
    }
    
    private static String getWriterClass(String format)
    {
             String writerClass = null;            
            if (format.equals(CSV)){
                writerClass = "com.gonetix.framework.dbpublish.CSVWriter";
            }else if (format.equals(DAT) ){
               writerClass = "com.gonetix.framework.dbpublish.DATWriter";
            }else if (format.equals(XML)){
               writerClass = "com.gonetix.framework.dbpublish.XMLWriter";
            }else if (format.equals(XLS)){
               writerClass = "com.gonetix.framework.dbpublish.XLSWriter";
            }else if (format.equals(PDF)){
               writerClass = "com.gonetix.framework.dbpublish.PDFWriter2";
            }else if (format.equals(PDF2)){
               writerClass = "com.gonetix.framework.dbpublish.PDFWriter2";
            }else if (format.equals(RTF)){ 
               writerClass = "com.gonetix.framework.dbpublish.RTFWriter";
            }else if (format.equals(TRAN)){ 
               writerClass = "com.gonetix.framework.dbpublish.TransformWriter";
            }else{
               writerClass = "com.gonetix.framework.dbpublish.CSVWriter"; 
            }            
            return writerClass;      
    }
    
    public static void printUsage()
    {
        String usage ="\n\nUsage:\n\t com.esd.Jextract.Main xmlCfgFilePath outFilepath [Format] [Delimiter] [-params [parameter..]]"
                       + "\n\n\n" 
                       + "xmlCfgIFilePath - path to XML file which contains username, password, jdbc url, jdbc drive\n"
                       + "                      and Sql Statement\n"
                       + "XML File Format:\n"
                       + "<root>\n"
                       + "    <driver>JDBC Driver Name</driver>\n"
                       + "    <url>JDBC URL</url>\n"
                       + "    <user>username</user>\n"
                       + "    <password>username</password>\n"
                       + "    <statement>SQL Statement Here</statement>\n"
                       + "</root>\n"
                       + "outFormat - one of CSV, DAT, XLS, XML, PDF, RTF"
                       + "outFilepath - path to output file\n"
                       + "Delimiter - optional default delimiter is comma (,)\n\n";
        
        System.out.println(usage);
                       
    }
    
}
