/*
 * XMLProcessor.java
 *
 * Created on August 13, 2002, 4:40 PM
 */

package com.gonetix.jextract;


import java.io.*;
import java.util.*;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.FactoryConfigurationError;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

import com.gonetix.framework.xmlEngine.*;
import com.gonetix.framework.dbpublish.*;
import com.gonetix.framework.util.*;
import com.gonetix.framework.xsltEngine.*;
/**
 *
 * @author  SLiu
 */
public class XMLProcessor extends DefaultHandler {
    
    private Properties globalProps;
    private ArrayList taskList;
    private XMLReader xmlReader;
    
    public XMLProcessor() 
    {
        globalProps = new Properties();
        taskList = new ArrayList();
    }
   
    public void parse(String file) throws Exception
    {
        parse(new InputSource(file));
    }
    
    public void parse(Reader reader) throws Exception
    {
        parse(new InputSource(reader));
    }
    public void parse(InputStream byteStream) throws Exception
    {
        parse(new InputSource(byteStream));
    }
    
    private void parse(InputSource source) throws Exception
    {                 
        SAXParserFactory spfactory = SAXParserFactory.newInstance();
        spfactory.setValidating(false);        
        SAXParser saxParser = spfactory.newSAXParser();
        xmlReader = saxParser.getXMLReader();            
        xmlReader.setContentHandler(this);
        xmlReader.setEntityResolver(this);
        xmlReader.setErrorHandler(this);
        xmlReader.parse(source);
    }
    
    public void addTask(Task task)
    {
        taskList.add(task);
    }
    
    public void setHandler(DefaultHandler handler)
    {
        xmlReader.setContentHandler(handler);
    }
    
    //--------------------------------------------------------------------------
    //XML SAX methods
    //--------------------------------------------------------------------------
    boolean started = false;
    StringBuffer value = new StringBuffer(100);    
    String key = null;
    
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
    throws SAXException 
    {
       value.setLength(0);
       if ( qName.equals("task")){
           Task task = new Task();
           this.addTask(task);
           TaskHandler thandler = new TaskHandler(this, task);
           thandler.init(qName, atts); 
           this.setHandler(thandler);
           started = true;
       }else if (!qName.equals("root")){
           key = qName;
           started = true;
       }else{
           started = false;
       }
        
    }
    
    public void characters(char[] ch, int start, int length)
    throws SAXException 
    {       
        String content = (new String(ch, start, length)); 
        if(content!=null && length>0){            
         content=StringUtil.decodeEntity(content, "&#60;|&lt;", "<");         
         content=StringUtil.decodeEntity(content, "&#62;|&gt;", ">");        
         content=StringUtil.decodeEntity(content, "&#47;", "/");
        }
        if( started)value.append(content);
    }
    
    public void endElement(String namespaceURI, String localName, String qName)
    throws SAXException 
    {
         
        if ( key!=null && value.length()>0 && started){
            String pvalue = value.toString();           
            pvalue = StringUtil.strip(pvalue);
            
            
            globalProps.setProperty(key, pvalue);            
        }
        started = false;
        key = null;
        value.setLength(0);
    }
    
    public String getProperty(String key)
    {
        return globalProps.getProperty(key);
    }
    
    public void setProperty(String key, String value)
    {
        globalProps.setProperty(key, value);
    }
    
    public void addAllProperties(Properties prop)
    {
        this.globalProps.putAll(prop);
    }
    
    public Properties getGlobalProperties()
    {
        return this.globalProps;
    }
    
    public Task getTask(int index)
    {
        return (Task)taskList.get(index);
    }
    
    
    public Task getTask(String taskId)
    {
        Task[] tasks = getAllTask();
        for (int i=0; i<tasks.length; i++){
            String id = tasks[i].getProperty("id");
            if (id !=null && id.equals(taskId)) return tasks[i];            
        }
        return null;
    }
    
    public Task[] getAllTask()
    {
        Task[] tasks = new Task[taskList.size()];
        taskList.toArray(tasks);
        return tasks;
    }
    
    
    
}
