/**
 * Sendmail Java Implementation
 */
package com.gonetix.mail;

import java.io.*;
import java.sql.SQLException;
import java.sql.*;
import java.util.*;
import java.util.logging.*;

// EMAIL
import javax.mail.*;
import javax.mail.internet.*;
import javax.mail.event.*;
import javax.activation.*;


/**
 *
 */
public class Mail implements ConnectionListener, TransportListener {
    
    // ConnectionListener interface
    public void opened(ConnectionEvent e) {
        System.out.println( e );
    }
    
    public void disconnected(ConnectionEvent e) {
        System.out.println( e );
    }
    
    public void closed(ConnectionEvent e) {
        System.out.println( e );
    }
    
    // TransportListener interface
    public void messageDelivered(TransportEvent e) {
        System.out.println( e );
    }
    
    public void messageNotDelivered(TransportEvent e) {
        System.out.print(">>> TransportListener.messageNotDelivered().");
        System.out.println(" Invalid Addresses:");
        Address[] invalid = e.getInvalidAddresses();
        if( invalid != null ) {
            for( int i = 0; i < invalid.length; i++ )
                System.out.println("     " + invalid[i]);
        }
    }
    
    public void messagePartiallyDelivered(TransportEvent e) {
        System.out.println( e );
    }
    
    /**
     * These need to be defined else where, i.e. we need a drop mail box where all mail goes
     */
    private static final String USER                = "DeadLetter@citco.com";
    private static final String RECIPIENTS          = "DLattimoreGay@citco.com, fundhelp@citco.com";
    private static final String HOST                = "xmail.citcoweb.com";
    
    /**
     *
     */
    private String getMailProperties() {
        String filename = System.getProperty("mail.properties");
        
        if ( filename == null || filename.length() == 0 ) {
            filename = "." + "/" + "mail.properties";
        }
        
        return filename;
    }
    
    /**
     * Members
     */
    private String strMsgSubject;
    private String strMsgBody;
    private String strAttachment;
    private java.util.Properties configuration;
    
    /**
     * Constructors
     */
    public void MailReport(String strMsgSubj, String strMsgBody, java.util.Properties conf) {
        strMsgSubject = strMsgSubj;
        this.strMsgBody = strMsgBody;
        this.configuration = conf;
        this.strAttachment = null;
    }
    
    /**
     * This will read from the properties file specified on the command line
     */
    public void MailReport(String propertiesfile, java.util.Properties conf) {
        this.configuration = conf;
        
        if ( propertiesfile != null && propertiesfile.length() > 0 ) {
            Properties props = new Properties();
            
            try {
                props.load(new java.io.BufferedInputStream(new java.io.FileInputStream(propertiesfile)));
                
                this.strMsgSubject = props.getProperty("subject", "UNKNOWN");
                this.strMsgBody = props.getProperty("body", "UNKNOWN");
                this.strAttachment = props.getProperty("attachment");
                
            } catch ( Exception ex ) {
                ex.printStackTrace();
                return;
            }
        }
    }
    
    public void MailReport(String strMsgSubj, String strMsgBody, String filename, java.util.Properties conf) {
        strMsgSubject = strMsgSubj;
        this.strMsgBody = strMsgBody;
        this.configuration = conf;
        this.strAttachment = filename;
    }
    
    public void MailReport(String strMsgSubj, String strMsgBody) {
        strMsgSubject = strMsgSubj;
        this.strMsgBody = strMsgBody;
        this.configuration = null;
        this.strAttachment = null;
    }
    
    public void sendmail() throws Exception {
        if ( configuration == null ) {
            Properties props = new Properties();
            
            String mailProperties = getMailProperties();
            
            try {
                props.load(new java.io.BufferedInputStream(new java.io.FileInputStream(mailProperties)));
            } catch ( Exception ex ) {
                props.put("mail.smtp.user", USER);
                props.put("mail.smtp.host", HOST);
                props.put("mail.smtp.recipients", RECIPIENTS);
            }
            
            configuration = props;
        }
        
        sendmail( this.strMsgSubject, this.strMsgBody, this.strAttachment, this.configuration );
    }
    
    private void sendmail(String strMsgSubj, String strMsgBody, String attachment, java.util.Properties conf) throws Exception {
        
        InternetAddress[] addr = null;
        
        addr = InternetAddress.parse(conf.getProperty("mail.smtp.recipients", RECIPIENTS));
        
        Session session = Session.getInstance(conf, null);
        session.setDebug(true);
        
        Transport trans = null;
        Message msg = new MimeMessage(session);
        
        trans = session.getTransport(addr[0]);
        trans.addConnectionListener(this);
        trans.addTransportListener(this);
        trans.connect();
        
        String from = (String)conf.get("mail.smtp.user");
        try {
            msg.setFrom(new InternetAddress(from,from + " on behalf of " ));
        } catch ( Exception ex ) {
            ex.printStackTrace();
            msg.setFrom();
        }
        
        msg.setRecipients(Message.RecipientType.TO, addr);
        msg.setSubject(strMsgSubj);
        msg.setSentDate(new java.util.Date());
        msg.setContent(strMsgBody, "text/plain");
        
        // create the message part
        MimeBodyPart messageBodyPart =
        new MimeBodyPart();
        
        //fill message
        messageBodyPart.setText("Hi");
        
        Multipart multipart = new MimeMultipart();
        multipart.addBodyPart(messageBodyPart);
        
        // Part two is attachment
        messageBodyPart = new MimeBodyPart();
        
        if ( attachment != null && attachment.length() > 0 ) {
            DataSource source = new FileDataSource(attachment);
            messageBodyPart.setDataHandler( new DataHandler(source) );
            messageBodyPart.setFileName(attachment);
            multipart.addBodyPart(messageBodyPart);
        }
        
        // Put parts in message
        msg.setContent(multipart);
        
        trans.sendMessage(msg, addr);
        
        try { Thread.sleep(5); } catch (InterruptedException e) {}
    }
    
    /**
     * Simple Test Example.
     */
    public static void main(String[] args) {
        /** Send Email out of there is a problem with the load, i.e. processed != loaded **/
        Properties props = new Properties();
        
        try {
            props.load(new java.io.BufferedInputStream(new java.io.FileInputStream(args[0])));
        } catch ( Exception ex ) {
            props.put("mail.smtp.user", USER);
            props.put("mail.smtp.host", HOST);
            props.put("mail.smtp.recipients", RECIPIENTS);
        }
        
        Mail mail = new Mail();
        
        String messagefile = System.getProperty("message");
        if ( messagefile != null && messagefile.length() >0 ) {
            mail.MailReport(messagefile, props);
        } else {
            StringBuffer html = new StringBuffer(1024);
            
            html.append("<HTML>");
            html.append("<BODY>");
            
            html.append("<h1>");
            
            html.append(new java.util.Date().toString());
            
            html.append("</h1>");
            
            html.append("</BODY>");
            html.append("</HTML>");
            
            mail.MailReport("TEST EMAIL PLEASE IGNORE " + (new java.util.Date()).toString(), html.toString(), props);
            
        }
        
        try {
            mail.sendmail();
        } catch ( Exception ex ) {
            ex.printStackTrace();
        }
        
    }
}