/*
 * RegExProperties.java
 *This is a replacement for java.util.Properties. it supports inlime comment
 *and space in key or value as well.
 *such as
 *key = value #this is properties for somewhat I do not care;
 *
 * Created on October 8, 2002, 9:36 AM
 */

package com.gonetix.properties;

import java.util.*;
import java.io.*;
import java.util.regex.*;

/**
 *
 * @author  SLiu
 */
public class RegExProperties extends Properties
{
    private static final Pattern comment = Pattern.compile("^\\s*#\\.*|^\\s*!\\.*");
    private static final Pattern continueLine = Pattern.compile("\\s*([^\\\\]+)\\\\");
    private static final Pattern singleLine = Pattern.compile("\\s*([^=]+)=([^#]+)#?.*");
    private static final Pattern variable = Pattern.compile("\\$\\{(\\w+)\\}");


    private File pfileDir = new File(".");
    private Vector loadListeners = new Vector();

    public synchronized void addLoadListener(IPropertiesLoadListener listener)
    {
       loadListeners.addElement(listener);
    }

    public synchronized void removeLoadListener(IPropertiesLoadListener listener)
    {
       loadListeners.remove(listener);
    }

    protected void fireNewPropertyEvent(String key, String value)
    {
        synchronized(loadListeners){
            for(int i=0; i<loadListeners.size(); i++){
               IPropertiesLoadListener listener = (IPropertiesLoadListener)loadListeners.elementAt(i);
               listener.onNewProperty(key, value);
            }
        }
    }

    public synchronized void loadFromPath(String systemid) throws IOException
    {
        File  file = new File(systemid);
        pfileDir = file.getParentFile();
        FileInputStream fin = new FileInputStream(file);
        load(fin);
        fin.close();
    }

    public synchronized void load(InputStream inStream) throws IOException {

        BufferedReader in = new BufferedReader(new InputStreamReader(inStream, "8859_1"));
        StringBuffer buffer = new StringBuffer(256);
        String line = null;
        Matcher m = null;
        while((line=in.readLine())!=null)
        {
            if ( isComment( line) == false ){
                if ( isImport(line)){
                    File ifile = new File(pfileDir, line.substring(1));
                    if ( ifile.exists() && ifile.isFile() ){
                        FileInputStream fin = new FileInputStream(ifile);
                        load(fin);
                        fin.close();
                    }
                }else {
                    if ( continueLine(line)){
                        while(line!=null && continueLine(line)){
                                m = continueLine.matcher(line);
                                if (m.find())buffer.append(m.group(1));
                                line = in.readLine();
                        }
                        if ( line!=null ) buffer.append(line); //last line of continue Line;
                        line = buffer.toString();
                        buffer.setLength(0);
                    }
                    m = singleLine.matcher(line);
                    if ( m.find()){
                        String key = m.group(1);
                        String value = m.group(2);
                        if (key!=null){
                            put(key.trim(), (value==null)?"":value.trim());
                            fireNewPropertyEvent(key.trim(), (value==null)?"":value.trim());
                        }
                    }
                }
            }//end not comment;
        }
    }

    protected boolean isImport(String line)
    {
        return line.startsWith("@");
    }

    protected boolean isComment(String line)
    {
        Matcher m = comment.matcher(line);
        return m.find();
    }

    protected boolean continueLine(String line)
    {
        Matcher m = continueLine.matcher(line);
        return m.find();
    }
     public int getIntProperty(String key) throws java.lang.NumberFormatException
    {
        return (Integer.valueOf(this.getProperty(key)).intValue());
    }

    public long getLongProperty(String key) throws java.lang.NumberFormatException
    {
        return (Long.valueOf(this.getProperty(key)).longValue());
    }

    public double getDoubleProperty(String key) throws java.lang.NumberFormatException
    {
        return (Double.valueOf(this.getProperty(key)).doubleValue());
    }

    public String getProperty(String key, String defaultValue)
    {
    	Matcher m = null;
		while ((m=variable.matcher(key)).find())
		{
			String subkey = m.group(1);
			key = m.replaceFirst(super.getProperty(subkey)==null?"":super.getProperty(subkey));
		}
    	return super.getProperty(key, defaultValue);
    }

    public String getProperty(String key)
    {

    	Matcher m = null;
		while ((m=variable.matcher(key)).find())
		{
			String subkey = m.group(1);
			key = m.replaceFirst(super.getProperty(subkey)==null?"":super.getProperty(subkey));
		}
		//System.out.println(key);
    	return super.getProperty(key);
    }

}
