package com.gonetix.swing.EditorComponents.SQLUI;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.text.*;
import javax.swing.event.DocumentListener;

public class SQLEditorPane extends JEditorPane {

  public static final String KEYWORD_COLOR = "keyword.color";
  public static final String COMMON_WORD_COLOR = "commonWord.color";
  public static final String MULTI_LINE_COMMENT_COLOR = "multilineComment.color";
  public static final String SINGLE_LINE_COMMENT_COLOR = "singlelineComment.color";
  public static final String STRING_LITERAL_COLOR = "stringLiteral.color";
  public static final String CHAR_LITERAL_COLOR = "charLiteral.color";
  public static final String NUMERIC_LITERAL_COLOR = "numericLiteral.color";
  public static final String OPERATOR_COLOR = "operator.color";
  /**
   * Key used for Boolean client property.
   */
  public static final String INDENT_ON_INSERT_BREAK = "indentOnInsertBreak";
  /**
   * Key used for Boolean client property.
   */
  public static final String AUTO_TRIM_ON_INSERT_BREAK = "trimLineOnInsertBreak";    
//  private static CommentAttributeSetter commentAttributeSetter;

  public SQLEditorPane() {
    setupKeymap();
    EditorKit kit = new StyledEditorKit();
    setEditorKitForContentType("text/sql", kit);
    setContentType("text/sql");

    setBackground(Color.white);
    setFont(new Font("Courier", 0, 12));
    setEditable(true);
    setIndentOnBreak(true);
    setAutoTrimOnInsertBreak(true);

    setKeywordColor(Color.blue);
    setCommonWordColor(new Color(26,168,110));
    setCharLiteralColor(Color.red);
    setMultiLineCommentColor(Color.magenta.darker());
    setNumericLiteralColor(Color.red);
    setSingleLineCommentColor(Color.green.darker());
    setStringLiteralColor(Color.red);
    setOperatorColor(Color.green.darker().darker());
  }
/*
  protected static DocumentListener getCommentAttributeSetter() {
    if(commentAttributeSetter == null)
        commentAttributeSetter = new CommentAttributeSetter();

    return commentAttributeSetter;
  }*/

  private void setupKeymap() {
    Keymap map = JTextComponent.getKeymap("MyKeymap");
    if(map == null) {
      Keymap parent = getKeymap();
      map = JTextComponent.addKeymap("MyKeymap",parent);
      
      KeyStroke insertBreakKeyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0,false);
      map.addActionForKeyStroke(insertBreakKeyStroke,new SQLEditorKit.InsertBreakAction());
      
      KeyStroke insertTabKeyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_TAB,0,false);
      map.addActionForKeyStroke(insertTabKeyStroke,new SQLEditorKit.InsertTabAction());
      
      KeyStroke insertShiftedTabKeyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_TAB,java.awt.Event.SHIFT_MASK,false);
      map.addActionForKeyStroke(insertShiftedTabKeyStroke,new SQLEditorKit.InsertShiftedTabAction());
    }
    setKeymap(map);
    
 }
   
  protected  EditorKit createDefaultEditorKit() {
    return new SQLEditorKit();
  }
  /**
   * Enable or disable auto indent. The same result can be obtained with
   * <br><br><code>putClientProperty(JavaEditorPane.INDENT_ON_INSERT_BREAK,new Boolean(b));</code><br>
   */  
  public void setIndentOnBreak(boolean b) {
    putClientProperty(INDENT_ON_INSERT_BREAK,new Boolean(b));
  }
  /**
  * Return auto actual setting for indentOnBreak property.
  * */
  public boolean getIndentOnBreak() {
    Boolean b = (Boolean)getClientProperty(INDENT_ON_INSERT_BREAK);
    if(b!=null)
      return false;
    else
      return b.booleanValue();
  }
  /**
   * Enable or disable auto trim. The same result can be obtained with
   * <br><br><code>putClientProperty(JavaEditorPane.AUTO_TRIM_ON_INSERT_BREAK,new Boolean(b));</code><br>
   */  
  public void setAutoTrimOnInsertBreak(boolean b) {
    putClientProperty(AUTO_TRIM_ON_INSERT_BREAK,new Boolean(b));
  }
  /**
  * Return auto actual setting for autoTrimOnInsertBreak property.
  * */
  public boolean getAutoTrimOnInsertBreak() {
    Boolean b = (Boolean)getClientProperty(AUTO_TRIM_ON_INSERT_BREAK);
    if(b!=null)
      return false;
    else
      return b.booleanValue();
  }
  public void setKeywordColor(Color c) {
    putClientProperty(KEYWORD_COLOR,c);
  }
  public Color getKeywordColor() {
    return (Color)getClientProperty(KEYWORD_COLOR);
  }
  public void setCommonWordColor(Color c) {
    putClientProperty(COMMON_WORD_COLOR,c);
  }
  public Color getCommonWordColor() {
    return (Color)getClientProperty(COMMON_WORD_COLOR);
  }
  public void setCharLiteralColor(Color c) {
    putClientProperty(CHAR_LITERAL_COLOR,c);
  }
  public Color getCharLiteralColor() {
    return (Color)getClientProperty(CHAR_LITERAL_COLOR);
  }
  public void setMultiLineCommentColor(Color c) {
    putClientProperty(MULTI_LINE_COMMENT_COLOR,c);
  }
  public Color getMultiLineCommentColor() {
    return (Color)getClientProperty(MULTI_LINE_COMMENT_COLOR);
  }
  public void setNumericLiteralColor(Color c) {
    putClientProperty(NUMERIC_LITERAL_COLOR,c);
  }
  public Color getNumericLiteralColor() {
    return (Color)getClientProperty(NUMERIC_LITERAL_COLOR);
  }
  public void setSingleLineCommentColor(Color c) {
    putClientProperty(SINGLE_LINE_COMMENT_COLOR,c);
  }
  public Color getSingleLineCommentColor() {
    return (Color)getClientProperty(SINGLE_LINE_COMMENT_COLOR);
  }
  public void setStringLiteralColor(Color c) {
    putClientProperty(STRING_LITERAL_COLOR,c);
  }
  public Color getStringLiteralColor() {
    return (Color)getClientProperty(STRING_LITERAL_COLOR);
  }
  public void setOperatorColor(Color c) {
    putClientProperty(OPERATOR_COLOR,c);
  }
  public Color getOperatorColor() {
    return (Color)getClientProperty(OPERATOR_COLOR);
  }
  /**
  * Visualize a JFrame with a JavaEditorPane inside. Used for testing.
  */
  public static void main(String[] argv) {
    JFrame f = new JFrame("SQLEditorPane");    
    f.setSize(500,400);
    SQLEditorPane editor = new SQLEditorPane();
    f.getContentPane().add(new JScrollPane(editor));
    f.setVisible(true);
  }
}
