package com.gonetix.swing.SQLUI;

import javax.swing.event.*;
import javax.swing.text.*;


public class SQLDocument extends PlainDocument {

  public static final Object CommentStartAttribute = new AttributeKey("CommentStart");

  public static final Object CommentEndAttribute = new AttributeKey("CommentEnd");

  public static final Object MultilineCommentAttribute = new AttributeKey("MultilineComment");
    
  static class AttributeKey {
    private String attributeName;
    public AttributeKey(String attributeName) {
      this.attributeName = attributeName;
    }
    public String toString() {
      return attributeName;
    }
  }

  protected void insertUpdate(DefaultDocumentEvent chng, AttributeSet attr) {
    super.insertUpdate(chng, attr);
     DocumentEvent.ElementChange change = chng.getChange(getDefaultRootElement());
     int startOffset;
     int length;

     if(change==null) {
       startOffset=chng.getOffset();
       length=chng.getLength();
     }
     else {
       Element[] elements = change.getChildrenAdded();
       startOffset=elements[0].getStartOffset();
       length=elements[elements.length-1].getEndOffset()-elements[0].getStartOffset();
     }
     try {
       _scanRange(startOffset,length);
     }
     catch(Exception e) {
       System.out.println("Unespected exception: "+e+". Please contact ginian@tiscali.it");
     }
  }
  protected void postRemoveUpdate(DefaultDocumentEvent chng) {
    super.postRemoveUpdate(chng);
     try {
       _scanRange(chng.getOffset(),chng.getLength());
     }
     catch(Exception e) {}
  }

  private void _scanRange(int offset,int len) throws BadLocationException {
    Element root = getDefaultRootElement();
    int firstParagraph = root.getElementIndex(offset);
    int lastParagraph = root.getElementIndex(offset+len);

    boolean inComment = __getCommentStatus(firstParagraph);
    for(int i=firstParagraph;i<=lastParagraph;i++)
      inComment = __scanParagraph(root.getElement(i),inComment);

    if(inComment)
      __commentFollowingLines(lastParagraph+1);
    else
      __uncommentFollowingLines(lastParagraph+1);
  }
  private boolean __getCommentStatus(int paragraphNumber) {
    if(paragraphNumber>0) {
      Element previousElement = getDefaultRootElement().getElement(paragraphNumber-1);
      return containsMultilineCommentAttribute(previousElement);
    }
    return false;
  }
  private void __commentFollowingLines(int startParagraph) {
    int lastParagraphToScan = getDefaultRootElement().getElementCount();
    for(int i=startParagraph;i<lastParagraphToScan;i++) {
      if(containsCommentEndAttribute(getDefaultRootElement().getElement(i))) {
        Element e = getDefaultRootElement().getElement(i);
        fireChangedUpdate(new AbstractDocument.DefaultDocumentEvent(e.getStartOffset(),e.getEndOffset()-e.getStartOffset(),DocumentEvent.EventType.CHANGE));
        break;
      }
      else
        addMultilineCommentAttribute(getDefaultRootElement().getElement(i));
    }
  }
  private void __uncommentFollowingLines(int startParagraph) {
    int lastParagraphToScan = getDefaultRootElement().getElementCount();
    for(int i=startParagraph;i<lastParagraphToScan;i++) {
      if(containsMultilineCommentAttribute(getDefaultRootElement().getElement(i))
          && !containsCommentStartAttribute(getDefaultRootElement().getElement(i)))
        removeMultilineCommentAttribute(getDefaultRootElement().getElement(i));
      else {
        Element e = getDefaultRootElement().getElement(i);
        fireChangedUpdate(new AbstractDocument.DefaultDocumentEvent(e.getStartOffset(),e.getEndOffset()-e.getStartOffset(),DocumentEvent.EventType.CHANGE));
        break;
      }
    }
  }
  private boolean __scanParagraph(Element paragraph,boolean inComment) throws BadLocationException {
    int startPosition = paragraph.getStartOffset();
    int length = (paragraph.getEndOffset()<getLength() ? paragraph.getEndOffset() : getLength())-startPosition;
    StringSegment content = new StringSegment();
    getText(startPosition,length,content);

    boolean slashFound=false;
    boolean starFound=false;
    boolean commentStartFound=false;
    boolean commentEndFound=false;
    boolean stringLiteral = false;
    boolean charLiteral = false;

    for(int wordIndex=0;wordIndex<content.length();wordIndex++)  {
      char indexedChar = content.charAt(wordIndex);

      //aggiungere il controllo del commento singola linea
      if(!inComment && slashFound && indexedChar == '/')
        break;
      else if(!inComment && indexedChar=='\"')
        stringLiteral = !stringLiteral;
      else if(!inComment && indexedChar=='\'')
        charLiteral = !charLiteral;
      else if(!starFound && !stringLiteral && !charLiteral && indexedChar=='/')
        slashFound=true;
      else if(!slashFound && !stringLiteral && !charLiteral && indexedChar=='*')
        starFound=true;
      else if(starFound && !stringLiteral && !charLiteral && indexedChar=='/') {
       inComment=false;
       commentEndFound=true;
       starFound=false;
      }
      else if(slashFound && !stringLiteral && !charLiteral && indexedChar=='*') {
       inComment=true;
       commentStartFound=true;
       slashFound=false;
      }
      else {
        starFound=false;
        slashFound=false;
      }
    }

    if(inComment)
      addMultilineCommentAttribute(paragraph);
    else
      removeMultilineCommentAttribute(paragraph);

    if(commentStartFound)
      addCommentStartAttribute(paragraph);
    else
      removeCommentStartAttribute(paragraph);

    if(commentEndFound)
      addCommentEndAttribute(paragraph);
    else
      removeCommentEndAttribute(paragraph);

    return inComment;
}  
  protected static boolean containsCommentStartAttribute(Element e) {
    return e.getAttributes().containsAttribute(CommentStartAttribute,CommentStartAttribute);
  }
  protected static boolean containsMultilineCommentAttribute(Element e) {
    return e.getAttributes().containsAttribute(MultilineCommentAttribute,MultilineCommentAttribute);
  }
  protected static boolean containsCommentEndAttribute(Element e) {
    return e.getAttributes().containsAttribute(CommentEndAttribute,CommentEndAttribute);
  }
  protected void addCommentStartAttribute(Element e) {
    addAttribute(e,CommentStartAttribute, CommentStartAttribute);
  }
  protected void removeCommentStartAttribute(Element e) {
    removeAttribute(e, CommentStartAttribute);
  }
  protected void addMultilineCommentAttribute(Element e) {
    addAttribute(e,MultilineCommentAttribute, MultilineCommentAttribute);
  }
  protected void removeMultilineCommentAttribute(Element e) {
    removeAttribute(e, MultilineCommentAttribute);
  }
  protected void addCommentEndAttribute(Element e) {
    addAttribute(e,CommentEndAttribute, CommentEndAttribute);
  }
  protected void removeCommentEndAttribute(Element e) {
    removeAttribute(e, CommentEndAttribute);
  }
  protected final void addAttribute(Element e,Object name,Object value) {
    ((MutableAttributeSet)e.getAttributes()).addAttribute(name,value);
    fireChangedUpdate(new AbstractDocument.DefaultDocumentEvent(e.getStartOffset(),e.getEndOffset()-e.getStartOffset(),DocumentEvent.EventType.CHANGE));
  }
  protected final void removeAttribute(Element e,Object name) {
    ((MutableAttributeSet)e.getAttributes()).removeAttribute(name);
    fireChangedUpdate(new AbstractDocument.DefaultDocumentEvent(e.getStartOffset(),e.getEndOffset()-e.getStartOffset(),DocumentEvent.EventType.CHANGE));
  }
}






