package com.gonetix.swing.SQLUI;

import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

public class SQLEditorKit extends DefaultEditorKit {

  public static final String insertShiftedTabAction = "insertShiftedTabAction";
  private ViewFactory sqlViewFactory = new SQLViewFactory();

  /** @link dependency */
  /*#SQLDocument lnkSQLDocument;*/

  public String getContentType() {
    return "text/sql";
  }

  public Document createDefaultDocument() {
    Document doc = new SQLDocument();
    return doc;
  }

  public final ViewFactory getViewFactory() {
    return sqlViewFactory;
  }


  public static class InsertShiftedTabAction extends TextAction {

   /**
    * Creates this object with the appropriate identifier.
    */
    public InsertShiftedTabAction() {
      super("InsertShiftedTabAction");
    }

    /**
     * The operation to perform when this action is triggered.
     *
     * @param e the action event
     */
    public void actionPerformed(ActionEvent e) {
      JTextComponent target = getTextComponent(e);
        if (target != null) {
          if ((! target.isEditable()) || (! target.isEnabled())) {
            target.getToolkit().beep();
            return;
          }
          if(target.getSelectionStart()!=target.getSelectionEnd()) {
            int selectionStart = target.getSelectionStart();
            int selectionEnd = target.getSelectionEnd();
            int firstElementIndex = target.getDocument().getDefaultRootElement().getElementIndex(selectionStart);        
            int lastElementIndex = target.getDocument().getDefaultRootElement().getElementIndex(selectionEnd);                    
            Element firstElement = target.getDocument().getDefaultRootElement().getElement(firstElementIndex);
            Element lastElement = target.getDocument().getDefaultRootElement().getElement(lastElementIndex);
            int newSelectionStart = firstElement.getStartOffset();
            int newSelectionEnd = lastElement.getEndOffset()-1;
            target.select(newSelectionStart,newSelectionEnd);    

            String content = target.getSelectedText();
            StringBuffer replacement = new StringBuffer();
            
            while(content.length()!=0) {
              int crIndex = content.indexOf('\n');
              if(crIndex==-1)
                crIndex = content.length();
              else
                crIndex = crIndex+1;
              String line = content.substring(0,crIndex);
              if(line.startsWith(" "))
                line = line.substring(1);
  
              replacement.append(line);
              content=content.substring(crIndex);
            }
            target.replaceSelection(replacement.toString());
            target.select(newSelectionStart,newSelectionStart+replacement.length());          
            return;
          }
          target.replaceSelection("\t");
        }
    }
 }

  
  public static class InsertTabAction extends TextAction {

   /**
    * Creates this object with the appropriate identifier.
    */
    public InsertTabAction() {
      super(insertTabAction);
    }

    /**
     * The operation to perform when this action is triggered.
     *
     * @param e the action event
     */
    public void actionPerformed(ActionEvent e) {
      JTextComponent target = getTextComponent(e);
        if (target != null) {
          if ((! target.isEditable()) || (! target.isEnabled())) {
            target.getToolkit().beep();
            return;
          }
          if(target.getSelectionStart()!=target.getSelectionEnd()) {
            int selectionStart = target.getSelectionStart();
            int selectionEnd = target.getSelectionEnd();
            int firstElementIndex = target.getDocument().getDefaultRootElement().getElementIndex(selectionStart);        
            int lastElementIndex = target.getDocument().getDefaultRootElement().getElementIndex(selectionEnd);                    
            Element firstElement = target.getDocument().getDefaultRootElement().getElement(firstElementIndex);
            Element lastElement = target.getDocument().getDefaultRootElement().getElement(lastElementIndex);
            int newSelectionStart = firstElement.getStartOffset();
            int newSelectionEnd = lastElement.getEndOffset()-1;
            target.select(newSelectionStart,newSelectionEnd);    

            String content = target.getSelectedText();
            StringBuffer replacement = new StringBuffer();
              
              while(content.length()!=0) {
                int crIndex = content.indexOf('\n');
                if(crIndex==-1)
                  crIndex = content.length();
                else
                  crIndex = crIndex+1;
                String line = content.substring(0,crIndex);

                replacement.append(" ");
                replacement.append(line);
                content=content.substring(crIndex);
              }
              target.replaceSelection(replacement.toString());
              target.select(newSelectionStart,newSelectionStart+replacement.length());          
              return;
          }
          target.replaceSelection("\t");
        }
    }
 }  
  
  public static class InsertBreakAction extends TextAction {

    public InsertBreakAction() {
      super(DefaultEditorKit.insertBreakAction);
    }

    /**
     * The operation to perform when this action is triggered.
     *
     * @param e the action event
     */
    public void actionPerformed(ActionEvent e) {
      JTextComponent target = getTextComponent(e);
      if (target != null) {
        if ((! target.isEditable()) || (! target.isEnabled())) {
          target.getToolkit().beep();
          return;
      }
      Boolean insertOnBreak = (Boolean)target.getClientProperty(SQLEditorPane.INDENT_ON_INSERT_BREAK);
      if(insertOnBreak!=null && insertOnBreak.booleanValue()) {
        Document document = target.getDocument();
        int selectionStart = target.getSelectionStart();
        int selectionEnd = target.getSelectionEnd();
        
        int lineNumber = document.getDefaultRootElement().getElementIndex(selectionStart);
        Element lineElement = document.getDefaultRootElement().getElement(lineNumber);

        int lastLineNumber = document.getDefaultRootElement().getElementIndex(selectionEnd);
        Element lastLineElement = document.getDefaultRootElement().getElement(lastLineNumber);

        int startOffset = lineElement.getStartOffset();
        int endOffset = selectionStart;
        int length = endOffset-startOffset;

        try {
          Segment lineHead = new Segment();
          target.getDocument().getText(startOffset,length,lineHead);

          StringBuffer spaces = new StringBuffer();

          for(int i = lineHead.offset;i<lineHead.offset+lineHead.count;i++) {
            if(lineHead.array[i]==' ')
              spaces.append(" ");
            else if(lineHead.array[i]=='\t')
              spaces.append("\t");
            else
               break;
          }

		  String newLine = document.getText(selectionEnd,Math.min(lastLineElement.getEndOffset(),document.getLength())-selectionEnd);

          Boolean trimOnInsert = (Boolean)target.getClientProperty(SQLEditorPane.AUTO_TRIM_ON_INSERT_BREAK);
          if(trimOnInsert!=null && trimOnInsert.booleanValue())
            newLine = newLine.trim();
          target.select(target.getSelectionStart(),Math.min(lastLineElement.getEndOffset(),document.getLength()));
          target.replaceSelection("\n"+spaces.toString()+newLine+"\n");
          target.setCaretPosition(endOffset+spaces.length()+1);

          return;
        }
        catch(BadLocationException ble) {
          ble.printStackTrace();
        }             
      }
      else
        target.replaceSelection("\n");
     }
    }
  }
}
