package com.gonetix.swing.SQLUI;

import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.Vector;
import java.util.*;
import javax.swing.text.*;


public class SQLViewFactory implements ViewFactory {

public static final String KEYWORD = "0";
public static final String OPERATOR = "1";
public static final String COMMON_LIBRARY_CLASS = "2";

private static Map keywordMap;

  public SQLViewFactory() {
    super();
    if(keywordMap==null)
      keywordMap = createKeywordMap();
  }
  public static Map createKeywordMap() {
    HashMap sqlKeywordMap = new HashMap();
    sqlKeywordMap.put("select",KEYWORD);
    sqlKeywordMap.put("update",KEYWORD);
    sqlKeywordMap.put("insert",KEYWORD);
    sqlKeywordMap.put("into",KEYWORD);
    sqlKeywordMap.put("from",KEYWORD);
    sqlKeywordMap.put("as",KEYWORD);
    sqlKeywordMap.put("where",KEYWORD);
    sqlKeywordMap.put("order",KEYWORD);
    sqlKeywordMap.put("by",KEYWORD);
    sqlKeywordMap.put("join",KEYWORD);
    sqlKeywordMap.put("on",KEYWORD);
    sqlKeywordMap.put("and",KEYWORD);
    sqlKeywordMap.put("or",KEYWORD);
    sqlKeywordMap.put("inner",KEYWORD);
    sqlKeywordMap.put("outer",KEYWORD);
    sqlKeywordMap.put("full",KEYWORD);
    sqlKeywordMap.put("create",KEYWORD);
    sqlKeywordMap.put("table",KEYWORD);
    sqlKeywordMap.put("drop",KEYWORD);
    sqlKeywordMap.put("alter",KEYWORD);
    sqlKeywordMap.put("modify",KEYWORD);
    sqlKeywordMap.put("database",KEYWORD);
    sqlKeywordMap.put("declare",KEYWORD);
    sqlKeywordMap.put("int",KEYWORD);
    sqlKeywordMap.put("bigint",KEYWORD);
    sqlKeywordMap.put("integer",KEYWORD);
    sqlKeywordMap.put("decimal",KEYWORD);
    sqlKeywordMap.put("number",KEYWORD);
    sqlKeywordMap.put("float",KEYWORD);
    sqlKeywordMap.put("varchar",KEYWORD);
    sqlKeywordMap.put("char",KEYWORD);
    sqlKeywordMap.put("real",KEYWORD);
    sqlKeywordMap.put("date",KEYWORD);
    sqlKeywordMap.put("datetime",KEYWORD);
    sqlKeywordMap.put("timestamp",KEYWORD);    
    return sqlKeywordMap;
  }
  
  public View create(Element elem) {
    return new SQLView(elem);
  }

  /**
   * View that uses the lexical information to determine the
   * style characteristics of the text that it renders.  This
   * simply colorizes the various tokens and assumes a constant
   * font family and size.
   */
  class SQLView extends PlainView {

  public SQLView(Element elem) {
    super(elem);
  }

/**
 * Renders the given range in the model as normal unselected
 * text.  This is implemented to paint colors based upon the
 * token-to-color translations.  To reduce the number of calls
 * to the Graphics object, text is batched up until a color
 * change is detected or the entire requested range has been
 * reached.
 *
 * @param g the graphics context
 * @param x the starting X coordinate
 * @param y the starting Y coordinate
 * @param p0 the beginning position in the model
 * @param p1 the ending position in the model
 * @returns the location of the end of the range
 * @exception BadLocationException if the range is invalid
 */
  protected int drawUnselectedText(Graphics g, int x, int y,int startOffset, int endOffset) throws BadLocationException {
    int length = (endOffset<getDocument().getLength() ? endOffset : getDocument().getLength())-startOffset;
    return scanParagraph(g,x,y,startOffset,length);
  }
  private boolean isParagraphInComment(int startOffset) {
    if(startOffset>0) {
      Element root = getDocument().getDefaultRootElement();
      Element previousElement = root.getElement(root.getElementIndex(startOffset-1));
      if(previousElement.getAttributes().containsAttribute(SQLDocument.MultilineCommentAttribute,SQLDocument.MultilineCommentAttribute))
        return true;   
    }
    return false;
  }
private int scanParagraph(Graphics g, int x, int y,int startPosition,int length) throws BadLocationException {
  StringSegment content = new StringSegment();
  JTextComponent host = (JTextComponent) getContainer();

  getDocument().getText(startPosition,length,content);
  boolean inComment = isParagraphInComment(startPosition);
  for(int wordIndex=0;wordIndex<content.length();)  {
    char indexedChar = content.charAt(wordIndex);
	
    if(Character.isLetter(indexedChar) && !inComment) {
      String scannedIdentifier = scanIdentifier(content,wordIndex);
      String tokenType = (String)keywordMap.get(scannedIdentifier.toLowerCase());
      int scannedIdentifierLength = scannedIdentifier.length();

      Color color = getColorForTokenType(tokenType);

      Segment text = getLineBuffer();
      getDocument().getText(startPosition+wordIndex, scannedIdentifierLength, text);
      g.setColor(color);
      x = Utilities.drawTabbedText(text, x, y, g, this, startPosition+wordIndex);
 
      wordIndex=wordIndex+scannedIdentifierLength;     
    }
    else if(Character.isDigit(indexedChar) && !inComment) {
      String scannedNumericLiteral = scanNumericLiteral(content,wordIndex);

      int scannedNumericLiteralLength = scannedNumericLiteral.length();

      Color color = (Color)host.getClientProperty(SQLEditorPane.NUMERIC_LITERAL_COLOR);

      Segment text = getLineBuffer();
      getDocument().getText(startPosition+wordIndex, scannedNumericLiteralLength, text);
      g.setColor(color);
      x = Utilities.drawTabbedText(text, x, y, g, this, startPosition+wordIndex);
 
      wordIndex=wordIndex+scannedNumericLiteralLength;
    }
    else if(indexedChar=='\"' && !inComment) {
      int scannedStringLength = scanStringLiteral(content,wordIndex);

      Segment text = getLineBuffer();
      getDocument().getText(startPosition+wordIndex, scannedStringLength, text);
      g.setColor((Color)host.getClientProperty(SQLEditorPane.STRING_LITERAL_COLOR));
      x = Utilities.drawTabbedText(text, x, y, g, this, startPosition+wordIndex);
      
      wordIndex=wordIndex+scannedStringLength;      
    }
    else if(indexedChar=='\'' && !inComment) {
      int scannedCharLength = scanCharLiteral(content,wordIndex);
      
      Segment text = getLineBuffer();
      getDocument().getText(startPosition+wordIndex, scannedCharLength, text);
      g.setColor((Color)host.getClientProperty(SQLEditorPane.CHAR_LITERAL_COLOR));
      x = Utilities.drawTabbedText(text, x, y, g, this, startPosition+wordIndex);
      
      wordIndex=wordIndex+scannedCharLength;      
    }    
    else if(isSingleLineCommentStart(content,wordIndex)&& !inComment) {
      int scannedCommentLength = scanSingleLineComment(content,wordIndex);

      Segment text = getLineBuffer();
      getDocument().getText(startPosition+wordIndex, scannedCommentLength, text);
      g.setColor((Color)host.getClientProperty(SQLEditorPane.SINGLE_LINE_COMMENT_COLOR));
      x = Utilities.drawTabbedText(text, x, y, g, this, startPosition+wordIndex);
      
      wordIndex=wordIndex+scannedCommentLength;       
    }
    else if(isMultiLineCommentEnd(content,wordIndex) && inComment) {
      Segment text = getLineBuffer();
      getDocument().getText(startPosition+wordIndex, 2, text);
      g.setColor((Color)host.getClientProperty(SQLEditorPane.MULTI_LINE_COMMENT_COLOR));
      x = Utilities.drawTabbedText(text, x, y, g, this, startPosition+wordIndex);

      inComment = false;
      wordIndex = wordIndex+2;
    }
    else if(isMultiLineCommentStart(content,wordIndex)) {
      int scannedCommentLength = scanMultiLineComment(content,wordIndex);

      Segment text = getLineBuffer();
      getDocument().getText(startPosition+wordIndex, scannedCommentLength, text);
      g.setColor((Color)host.getClientProperty(SQLEditorPane.MULTI_LINE_COMMENT_COLOR));
      x = Utilities.drawTabbedText(text, x, y, g, this, startPosition+wordIndex);
      
      wordIndex=wordIndex+scannedCommentLength;       
    }
    else if(isOperator(indexedChar) && !inComment) {
      Color color = (Color)host.getClientProperty(SQLEditorPane.OPERATOR_COLOR);

      Segment text = getLineBuffer();
      getDocument().getText(startPosition+wordIndex, 1, text);
      g.setColor(color);
      x = Utilities.drawTabbedText(text, x, y, g, this, startPosition+wordIndex);
 
      wordIndex=wordIndex+1;
    }
    else {
      Color color;
      if(inComment)
        color = (Color)host.getClientProperty(SQLEditorPane.MULTI_LINE_COMMENT_COLOR);
      else
        color = host.getForeground();
        
      Segment text = getLineBuffer();
      getDocument().getText(startPosition+wordIndex, 1, text);
      g.setColor(color);
      x = Utilities.drawTabbedText(text, x, y, g, this, startPosition+wordIndex);
      
      wordIndex++;
    }
  }
    return x;
  }
  private Color getColorForTokenType(String tokenType) {
    JTextComponent host = (JTextComponent) getContainer();
    if(tokenType==KEYWORD)
      return (Color)host.getClientProperty(SQLEditorPane.KEYWORD_COLOR);
    else if(tokenType==COMMON_LIBRARY_CLASS)
      return (Color)host.getClientProperty(SQLEditorPane.COMMON_WORD_COLOR);
    else
      return host.getForeground();
  }
  private boolean isSingleLineCommentStart(StringSegment string,int index) {
    if(string.charAt(index)=='/' &&
       index+1<string.length() &&
       string.charAt(index+1)=='/')
    return true;

    return false;
  }
  private int scanSingleLineComment(StringSegment string,int index) {
      int commentLength = 0;
      for(commentLength = 0;commentLength<(string.length()-index);commentLength++) {
        char commentChar = string.charAt(index+commentLength);
        if(commentChar=='\n')
          break;
      }
      return commentLength;
  }
  private boolean isMultiLineCommentStart(StringSegment string,int index) {
    if(string.charAt(index)=='/' &&
       index+1<string.length() &&
       string.charAt(index+1)=='*')
    return true;

    return false;
  }
  private boolean isMultiLineCommentEnd(StringSegment string,int index) {
    if(string.charAt(index)=='*' &&
       index+1<string.length() &&
       string.charAt(index+1)=='/')
    return true;

    return false;
  }
  private int scanMultiLineComment(StringSegment string,int index) {
      int commentLength = 0;
      boolean starFound = false;
      for(commentLength = 0;commentLength<(string.length()-index);commentLength++) {
        char commentChar = string.charAt(index+commentLength);
        if(starFound && commentChar=='/') {
          commentLength++;
          break;
        }
        starFound = false;
        if(commentChar=='\n')
          break;
        else if(commentChar == '*')
          starFound = true;
      }
      return commentLength;
  }
  private int scanStringLiteral(StringSegment string,int index) {
      int stringLength = 1;
      boolean backslash = false;
      for(stringLength = 1;stringLength<(string.length()-index);stringLength++) {
        char stringChar = string.charAt(index+stringLength);
        if(stringChar=='\\')
          backslash=true;
        else if(stringChar=='\n')
          break;
        else if(stringChar=='"' && !backslash) {
          stringLength++;
          break;          
        }
        else if(backslash)
          backslash=false;
      }
      return stringLength;
  }
  private int scanCharLiteral(StringSegment string,int index) {
      int charLength = 1;
      boolean backslash = false;
      for(charLength = 1;charLength<(string.length()-index);charLength++) {
        char charChar = string.charAt(index+charLength);
        if(charChar=='\\')
          backslash=true;
        else if(charChar=='\n')
          break;
        else if(charChar=='\'' && !backslash) {
          charLength++;
          break;          
        }
        else if(backslash)
          backslash=false;
      }
      return charLength;
  }
  private String scanIdentifier(StringSegment stringSegment,int index) {
    String string = new String(stringSegment.array,stringSegment.offset,stringSegment.count);
    int identifierLength = 0;
    if(!Character.isJavaIdentifierStart(string.charAt(index)))
      return "";
    else
      for(identifierLength = 1;identifierLength<(string.length()-index);identifierLength++) {
        char identifierChar = string.charAt(index+identifierLength);
        if(!Character.isJavaIdentifierPart(string.charAt(index+identifierLength)))
          break;
      }

    return string.substring(index,index+identifierLength);
  }
  private String scanNumericLiteral(StringSegment stringSegment,int index) {
    String string = new String(stringSegment.array,stringSegment.offset,stringSegment.count);
    int identifierLength = 0;
    for(identifierLength = 0;identifierLength<(string.length()-index);identifierLength++) {
      char identifierChar = string.charAt(index+identifierLength);
      if(!Character.isDigit(string.charAt(index+identifierLength)))
        break;
    }
    return string.substring(index,index+identifierLength);
  }
  private boolean isOperator(char c) {
    return (c == '-' || c == '+' || c == '*' || c == '/' || c=='<' || c=='>' || c == '!' || c == '~' || c=='%' || c=='^' || c== '&' || c == '|' || c == '=');
  }
}
}
