package com.gonetix.swing.sql;

import java.io.*;
import java.sql.*;
import com.gonetix.swing.sql.base.*;
import com.gonetix.swing.sql.util.*;
import com.gonetix.swing.sql.swing.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.text.*;
import javax.swing.border.*;

public class AppMain extends JFrame
{
  private Container container;
  private JTabbedPane tabbedPane;
  private ResultPanel resultPane;
  private JMenuBar menuBar;   
  private JToolBar toolBar;
  protected final String CUT = "Cut";
  protected final String COPY = "Copy";
  protected final String PASTE = "Paste";
  protected JMenu editMenu, fileMenu;
  protected  SQLCompositionPane sqlp;
  
  
  public AppMain()
  {
      super("SQL Front -- Sunny Liu");
      this. addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                ResourceManager.getResourceManager().saveResource();
                if (sqlp!=null) sqlp.dispose();
                System.exit(0);
            }
      });
      container = this.getContentPane();
      container.setLayout(new BorderLayout());
      tabbedPane = new JTabbedPane();
      sqlp = new SQLCompositionPane();

      Border blackline1 = BorderFactory.createLineBorder(Color.black);

      Border blackline = BorderFactory.createLoweredBevelBorder();
      JEditorPane xslPane = new JEditorPane();
      xslPane.setBorder(blackline);

      resultPane = new ResultPanel();
      resultPane.setBorder(blackline);
      SQLHandler.getInstance().setQueryResultRender(resultPane);
      tabbedPane.add("SQL ", sqlp);
      tabbedPane.add("XSL ", xslPane);
      tabbedPane.add("Result ", resultPane);    

      tabbedPane.setBorder(blackline1);
      container.add(tabbedPane, BorderLayout.CENTER);
      menuBar = new JMenuBar(); 
      initMenuBar(menuBar);
      setJMenuBar(menuBar);
      
      toolBar = new JToolBar();
      initJToolBar(toolBar);
      container.add(toolBar, BorderLayout.NORTH);
  }


  public void initMenuBar(JMenuBar jmb)
  {
        JMenu menu;
        JMenuItem menuItem;
        //jmb.setLayout(new FlowLayout());
        fileMenu = new JMenu("File");
        fileMenu.setMnemonic('f');        
        jmb.add(fileMenu);
        
        editMenu = new JMenu("Edit");
        menuItem = new JMenuItem("Undo");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, KeyEvent.CTRL_MASK));
        //menuItem.addActionListener(undoListner);
        editMenu.add(menuItem);
        menuItem = new JMenuItem("Redo");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, KeyEvent.CTRL_MASK));
        //menuItem.addActionListener(undoListner);
        editMenu.add(menuItem);
        editMenu.addSeparator();
        menuItem = new JMenuItem("Cut");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, KeyEvent.CTRL_MASK));
        //menuItem.addActionListener(ccpListener);
        editMenu.add(menuItem);        
        menuItem = new JMenuItem("Copy");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, KeyEvent.CTRL_MASK));
        //menuItem.addActionListener(ccpListener);
        editMenu.add(menuItem);        
        menuItem = new JMenuItem("Paste");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, KeyEvent.CTRL_MASK));
        //menuItem.addActionListener(ccpListener);        
        editMenu.add(menuItem);        
        editMenu.addSeparator();        
        
        //FormatListener ffl = new FormatListener();
        menu = new JMenu("Format");
        menuItem = new JMenuItem("Editor Font");
        //menuItem.addActionListener(ffl);
        menu.add(menuItem);
        menuItem = new JMenuItem("Result Font");
        //menuItem.addActionListener(ffl);
        menu.add(menuItem);
        menuItem = new JMenuItem("Result Information");
        //menuItem.addActionListener(new ResultFormatListener());
        menu.add(menuItem); 
        jmb.add(menu);
        
        menu = new JMenu("Run");
        menuItem = new JMenuItem("Execute");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F5, KeyEvent.CTRL_MASK));
        menuItem.addActionListener(new RunListener());
        menu.add(menuItem);        
        jmb.add(menu);
        
        menu = new JMenu("Help");
        menuItem = new JMenuItem("Help Content");        
        //menuItem.addActionListener(new HelpListener());
        //menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0));
        menu.add(menuItem);
        menu.addSeparator();
        menuItem = new JMenuItem("About SQL Front");
        //menuItem.addActionListener(aboutListener);
        menu.add(menuItem);        
        jmb.add(menu);

  }

  public void initJToolBar(JToolBar jtb)
  {
        JButton jbutton = null;
        jbutton = new JButton("New");
        jbutton.setToolTipText("New");        
        //jbutton.addActionListener(newListener); getImageIcon("images/new.gif")
        jtb.add(jbutton);
        jbutton = new JButton("Open"/*getImageIcon("images/open.gif")*/);
        jbutton.setToolTipText("Open");
        //jbutton.addActionListener(openListener);
        jtb.add(jbutton);
        jbutton = new JButton("Exec"/*getImageIcon("images/run.gif")*/);
        jbutton.setToolTipText("Execute");
        jbutton.addActionListener(new RunListener());
        jtb.add(jbutton);
        jbutton = new JButton("Save"/*getImageIcon("images/save.gif")*/);
        jbutton.setToolTipText("Save");
        //jbutton.addActionListener(saveListener);
        jtb.add(jbutton);        
 }
 

  public static void main(String[] args)
  {
     AppMain appMain = new AppMain();
     appMain.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
     appMain.setSize(800, 700);
     appMain.setVisible(true);
  }


  class RunListener implements ActionListener
  {
        public void actionPerformed(ActionEvent actionevent)
        {
            SQLHandler.getInstance().handleSQLExecutionEvent();
        }      
  }//end of  RunListener;
  
}