package com.gonetix.swing.sql.base;

import java.util.*;
import java.sql.*;

import javax.swing.tree.TreeNode;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.event.*;


public final class ConnNode extends DefaultMutableTreeNode

{
  private ConnInfo connInfo;
  private Connection conn;
  
  public ConnNode(ConnInfo connInfo)
  {
      super(connInfo);     
      this.connInfo = connInfo;     
  }

  public String getName()
  {
      return connInfo.getName();
  }

  public ConnInfo getInfo()
  {
      return this.connInfo;
  }

  public String toString()
  {
      return connInfo.getName();
  }

  public Connection getConnection()
  {
      return this.conn;
  }
  
  public void close() 
  {
      try{
            conn.close();
      }catch(Exception ex){

      }
  }
  public void connect() throws Exception
  {     
      conn = DBConnectionFactory.createConnection(connInfo);
      DatabaseMetaData  dbmd = conn.getMetaData();
      exposeMeta(dbmd);
  }

  protected void exposeMeta(DatabaseMetaData dbmd)  throws SQLException
  {
      CatalogNode cnode = null;
      SchemaNode snode = null;
      TableNode tnode = null;
      ViewNode vnode = null;
      TablesCollectionNode tnodes = null;
      ViewsCollectionNode vnodes = null;
      
      String[] catalogs = getColumnInString(dbmd.getCatalogs(), 1);
      String[] schemas = getColumnInString(dbmd.getSchemas(), 1);
      int i=0, j=0, k=0;
      for(i=0; i<catalogs.length; i++){
            cnode = new CatalogNode(catalogs[i]);
            this.add(cnode);
            for(j=0; j<schemas.length; j++){
                  snode = new SchemaNode(schemas[j]);                  
                  cnode.add(snode);
                  tnodes = new TablesCollectionNode();
                  snode.add(tnodes);
                  String[] tables = getColumnInString(dbmd.getTables(catalogs[i], schemas[j], "%", new String[]{"table"}), 3);
                  for(k=0; k<tables.length; k++){
                        tnode = new TableNode(tables[k]);
                        tnodes.add(tnode);                        
                  }
                  vnodes = new ViewsCollectionNode();
                  snode.add(vnodes);
                  String[] views = getColumnInString(dbmd.getTables(catalogs[i], schemas[j], "%", new String[]{"view"}), 3);
                  for(k=0; k<views.length; k++){
                        vnode = new ViewNode(views[k]);
                        vnodes.add(vnode);                        
                  }
            }
      }
      
  }
  
  protected String[] getColumnInString(ResultSet rs, int index) throws SQLException
  {
      if ( rs == null )  return null;
      Vector al = new Vector(20);      
      while (rs.next()){
          al.add(rs.getString(index));    
      }      
      String[] ret = new String[al.size()];
      al.copyInto(ret);
      al = null;
      return ret;
  }

  public void removeAllChildren()
  {
      super.removeAllChildren();
      super.children.clear();
  }

  public void removeFromParent()
  {
      super.removeFromParent();
      if(children!=null) children.clear();
  }

}