package com.gonetix.swing.sql.base;

import java.sql.ResultSetMetaData;
import java.sql.*;

public final class TableViewInfo 
{
  private int colCount;
  private String tableName;
  private String[] colNames;
  private int[] colTypes;
  private String[] typeNames;
  private boolean[] nullables;
  private int[] sizes;
  private int[] scales;
  private int[] precisions;
  private String catalogName;
  private String schemaName;

  public TableViewInfo(String name, ResultSetMetaData rsmd) throws SQLException
  {
      if (name.indexOf(".")>0){
          this.tableName = name.substring(name.lastIndexOf(".")+1);
      }else{
          this.tableName = name;
      }
      extractInfo(rsmd);
  }

  private void extractInfo(ResultSetMetaData rsmd) throws SQLException
  {
      colCount = rsmd.getColumnCount();
      catalogName = rsmd.getCatalogName(1);
      schemaName = rsmd.getSchemaName(1);
      colNames = new String[colCount];
      colTypes = new int[colCount];
      nullables = new boolean[colCount];
      sizes = new int[colCount];
      scales = new int[colCount];
      precisions = new int[colCount];
      typeNames = new String[colCount];
      
      for(int i=0; i<colCount; i++){
          colNames[i] = rsmd.getColumnName(i+1);
          colTypes[i] = rsmd.getColumnType(i+1);
          nullables[i] = (rsmd.isNullable(i+1)!=0)?true:false;
          sizes[i] = rsmd.getColumnDisplaySize(i+1);
          scales[i] = rsmd.getScale(i+1);
          precisions[i] = rsmd.getPrecision(i+1);
          typeNames[i] = rsmd.getColumnTypeName(i+1);
      }
  }

  

  public static String getQueryString(String tableName)
  {
      return "SELECT * FROM " + tableName + " WHERE 1>2";
  }

  public int getColCount()
  {
    return colCount;
  }

  public String[] getColNames()
  {
    return colNames;
  }

  public String getColName(int index)
  {
    return colNames[index-1];
  }

  public int[] getColTypes()
  {
    return colTypes;
  }

  public int getColType(int index)
  {
    return colTypes[index-1];
  }

  public boolean[] getNullables()
  {
    return nullables;
  }
  public boolean getNullable(int index)
  {
    return nullables[index-1];
  }
  public int[] getPrecisions()
  {
    return precisions;
  }

  public int getPrecision(int index)
  {
    return precisions[index-1];
  }

  public int[] getScales()
  {
    return scales;
  }
  public int getScale(int index)
  {
    return scales[index-1];
  }
  public int getSize(int index)
  {
    return sizes[index-1];
  }
  public int[] getSizes()
  {
    return sizes;
  }
  public String getTableName()
  {
    return tableName;
  }

  public String getCatalogName()
  {
    return catalogName;
  }

  public String getSchemaName()
  {
    return schemaName;
  }

  public Object[][] getTableInfo()
  {
        Object[][] ret = new Object[colCount][4];
        for(int j=0; j<colCount; j++){
            ret[j][0] = colNames[j];
            ret[j][1] = typeNames[j];
            ret[j][2]= new Boolean(nullables[j]);
            ret[j][3] = new Integer(sizes[j]);            
        }
        return ret;
  }
  public String toString()
  {
      StringBuffer sb = new StringBuffer(200);
      sb.append("\n"); 
      if (catalogName!=null) sb.append(catalogName).append(".");
      if (schemaName!=null) sb.append(schemaName).append(".");
      sb.append(tableName).append("  (\n");
      for(int i=0; i<colCount; i++){
          sb.append(colNames[i]);
          if (colTypes[i]==Types.VARCHAR || colTypes[i]==Types.CHAR){
                sb.append("(").append(sizes[i]).append(")");
          }else if (colTypes[i]==Types.NUMERIC || colTypes[i]==Types.DECIMAL) {
                sb.append("(").append(scales[i]).append(", ").append(precisions[i]).append(")");
          }
          if ( !nullables[i]) sb.append(" NOT NULL");
          if (i!=(colCount-1)) sb.append(",");
      }
      sb.append(tableName).append(")\n");
      return sb.toString();
  }
  
}