package com.gonetix.swing.sql.swing;

import javax.swing.JScrollPane;
import java.util.*;
import java.sql.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;

import com.gonetix.swing.sql.util.*;
import com.gonetix.swing.sql.base.*;
import javax.swing.tree.*;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.event.*;

public class DBExplorerPane extends JScrollPane 
{
  private JTree tree = null;
  private ConnRootNode root = null;
  private JPopupMenu connPopup = null;
  private JPopupMenu tablePopup = null;
  private TreePath currPath = null;
  private Object currSelectedObject = null;
  private ConnNode currCurrConnNode = null;
  private TablePropertiesPane tablePropertiesPane = null;
  private JTabbedPane parent = null;
  
  public DBExplorerPane(JTabbedPane parent)
  {
        super();
        this.parent = parent;
        root = new ConnRootNode();
        initTree(root);
        tree = new JTree(root);
        tree.setEditable(true);
        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.setShowsRootHandles(true);
        tree.addTreeSelectionListener(new TreeSelectionMonitor());
        getViewport().add(tree);
        initPopupMenu();
        tree.addMouseListener(new PopupMonitor());
  }

  protected void initTree(ConnRootNode rootNode)
  {
      ConnInfo[] infos = ResourceManager.getResourceManager().getConnInfoes();
      for(int i=0; i<infos.length; i++){
            ConnNode cnode = new ConnNode(infos[i]);
            rootNode.add(cnode);
      }
  }

  public void initPopupMenu()
  {
      JMenuItem menuItem;
      connPopup = new JPopupMenu();
      tablePopup = new JPopupMenu();
      MenuMonitor menuMonitor = new MenuMonitor();

      menuItem = new JMenuItem("Connect");
      menuItem.addActionListener(menuMonitor);
      connPopup.add(menuItem);
      menuItem = new JMenuItem("Add Conntion");
      menuItem.addActionListener(menuMonitor);
      connPopup.add(menuItem);
      menuItem = new JMenuItem("Delete Conntion");
      menuItem.addActionListener(menuMonitor);
      connPopup.add(menuItem);
      connPopup.addSeparator();
      menuItem = new JMenuItem("Close Connection");
      menuItem.addActionListener(menuMonitor);
      connPopup.add(menuItem); 
      menuItem = new JMenuItem("Property");
      menuItem.addActionListener(menuMonitor);
      connPopup.add(menuItem);    
      menuItem = new JMenuItem("Refresh");
      menuItem.addActionListener(menuMonitor);
      connPopup.add(menuItem);      
      
      
      menuItem = new JMenuItem("Refresh");
      menuItem.addActionListener(menuMonitor);
      tablePopup.add(menuItem);      
      
      menuItem = new JMenuItem("Property");
      menuItem.addActionListener(menuMonitor);
      tablePopup.add(menuItem);    
      
  }

  public void setTablePropertiesPane(TablePropertiesPane pane)
  {
        this.tablePropertiesPane = pane;
  }

  public void addConnection(ConnInfo info)
  {
      ConnNode conn1 = new ConnNode(info);
      root.add(conn1);
  } 

  private class TreeSelectionMonitor implements TreeSelectionListener
  {
        public void valueChanged(TreeSelectionEvent e) 
        {
            
            currPath = e.getPath();
            if (currPath ==null ) return;
            Object[] nodes = currPath.getPath();
            if (nodes!=null && nodes.length>=2 && nodes[1] instanceof ConnNode ){
                ConnNode selectedConnNode = (ConnNode)nodes[1];
                if (currCurrConnNode == null ){
                    currCurrConnNode = selectedConnNode;
                }else{
                    if (currCurrConnNode!=selectedConnNode)currCurrConnNode = selectedConnNode; 
                }              
            }
            currSelectedObject = tree.getLastSelectedPathComponent();
            if (currCurrConnNode != null ) 
                    SQLHandler.getInstance().setConnNode(currCurrConnNode);

        }

  }

  private class PopupMonitor extends MouseAdapter
  {
         public void mousePressed(MouseEvent e) {
            maybeShowPopup(e);
        }

        public void mouseReleased(MouseEvent e) {
            maybeShowPopup(e);
        }

        private void maybeShowPopup(MouseEvent e) {
            if (e.isPopupTrigger()) {
                if (currSelectedObject !=null && (currSelectedObject instanceof ConnNode || currSelectedObject instanceof  ConnRootNode)){
                      connPopup.show(e.getComponent(), e.getX(), e.getY());
                }else if ( currSelectedObject instanceof TableNode ){
                      tablePopup.show(e.getComponent(), e.getX(), e.getY());
                }else if (currSelectedObject instanceof ViewNode ){
                      tablePopup.show(e.getComponent(), e.getX(), e.getY());
                }
            }
        }
  }

  private class MenuMonitor implements ActionListener 
  {
      public void actionPerformed(ActionEvent e)
      {
          if ( e==null ) return;
          if (e.getActionCommand().equals("Connect")){
                if (currSelectedObject !=null &&  currSelectedObject instanceof ConnNode )
                {
                     ConnNode connNode =  (ConnNode)currSelectedObject;
                     try{
                            connNode.connect();
                     }catch(Exception ex){

                     }
                }
          }else if (e.getActionCommand().equals("Add Conntion")){
                AddConnDialog dialog = new AddConnDialog(tree, root);
                dialog.show();
          }else if (e.getActionCommand().equals("Delete Conntion")){
                if (currSelectedObject !=null &&  currSelectedObject instanceof ConnNode )
                {
                     ConnNode connNode = (ConnNode)currSelectedObject;
                     ConnInfo info = connNode.getInfo();
                     ResourceManager.getResourceManager().removeConn(info);
                     connNode.close();                      
                     currPath = currPath.getParentPath();
                     connNode.removeFromParent();
                     tree.expandPath(currPath);                     
                     tree.updateUI();
                }       
          }else if (e.getActionCommand().equals("Refresh")){
                      tree.updateUI();
          }else if (e.getActionCommand().equals("Property")){                
                if ( tablePropertiesPane !=null && currCurrConnNode !=null 
                && currSelectedObject !=null &&  currSelectedObject instanceof TableNode  )
                {                     
                    TreePath path = tree.getSelectionPath();
                    Object[] objs = path.getPath();                   
                     try{
                         CatalogNode catalog = (CatalogNode)objs[2];
                         SchemaNode schema = (SchemaNode)objs[3];
                        TableNode tbNode =  (TableNode)currSelectedObject;
                        Connection conn = currCurrConnNode.getConnection();
                        String tbName = catalog.getName() + "." + schema.getName() + "." +  tbNode.getName();                       
                        tablePropertiesPane.addTable(SQLHandler.getTableViewInfo(conn, tbName));
                        parent.setSelectedIndex(1);
                     }catch(Exception ex){
                        ex.printStackTrace();
                     }
                     
                }else  if ( tablePropertiesPane !=null && currCurrConnNode !=null 
                && currSelectedObject !=null &&  currSelectedObject instanceof ConnNode )
                {
                      try{                        
                        ConnNode tbNode =  (ConnNode)currSelectedObject;
                        tablePropertiesPane.addTable(tbNode.getInfo().getInfoes());
                        parent.setSelectedIndex(1);
                     }catch(Exception ex){
                        ex.printStackTrace();
                     }
                }
          }else if (e.getActionCommand().equals("Close Connection")){
                if (currSelectedObject !=null &&  currSelectedObject instanceof ConnNode )
                {
                     ConnNode connNode =  (ConnNode)currSelectedObject;                     
                     connNode.close();
                     connNode.removeAllChildren();
                     tree.updateUI();
                }     
          }
          if (currPath!=null) tree.expandPath(currPath);
          tree.updateUI();          
      }
  }

}