package com.gonetix.util;

import java.io.*;
import java.util.zip.*;
import java.util.ArrayList;
import java.util.regex.*;


/**
 * @author SLiu
 *
 * This class contains some common file level operation utility
 * 
 */
public class FileUtil 
{
	
	/**
	 * Method isWindowsPlatform.
	 * @return boolean
	 */
	public static boolean isWindowsPlatform()
	{
		return (System.getProperty("file.separator").equals("\\"));
	}
	
	/**
	 * Method getAbsolutePath.
	 * @param parent
	 * @param child
	 * @return String
	 */
	public static String getAbsolutePath(String parent, String child)
	{
		child = child.trim();
		child = child.replaceAll("\\", "/");
		if (child.startsWith("/") || child.indexOf(":")!=-1)
		{
			 //it is absoluted path already;
		}else{
			File dir = new File(parent);
			File childf = new File(dir, child);
			try{
				child = childf.getCanonicalPath();
			}catch(IOException ex){
				child = childf.getAbsolutePath();
			}		
		}
		return child;
	}
	
	
	/**
	 * Method readTextFile. reads text file and return file content as string.
	 * 
	 * @param file
	 * @return String
	 * @throws IOException
	 */
	public static String readTextFile(String file) throws IOException
	{
		StringWriter sw = new StringWriter(2048);
		PrintWriter pw = new PrintWriter(sw);
		BufferedReader reader = null;
				
		reader = new BufferedReader(new FileReader(file));
		String line = null;
		while((line=reader.readLine())!=null)
		{
			pw.println(line);
			pw.flush();
		}
		pw.close();
		reader.close();
		return sw.toString();
	}
	
	/**
	 * Method extract. extracts sub portion of text file content from a large text
	 * file.
	 * @param reader 
	 * @param writer
	 * @param startLineNumber
	 * @param endLineNumber
	 * @throws IOException
	 */
	public static void extract(Reader reader, Writer writer, int startLineNumber, int endLineNumber)
	throws IOException
	{
		int lineNumber = 0;
		String line = null;
		PrintWriter pw = new PrintWriter(writer);
		BufferedReader br = new BufferedReader(reader);
		while ((line=br.readLine())!=null && lineNumber <= endLineNumber)
		{			
			if (lineNumber>=startLineNumber)
			{
				pw.println(line);
			}
			lineNumber++;
		}
	}
	
	
	/**
	 * Method append read source file content and append to target file.
	 * @param source
	 * @param target
	 * @return boolean
	 */
	public static boolean append(File source, File target)
	{
		boolean su = false;
		FileInputStream fin = null;
		FileOutputStream fout = null;
		try{
			byte[] buffer = new byte[1024];
			fin = new FileInputStream(source);
			fout = new FileOutputStream(target, true);
			int len = 0;
			while ((len=fin.read(buffer))>0)
			{
				fout.write(buffer, 0, len);
			}
			su = true;			
		}catch(Exception ex){
			su = false;
		}finally{
			try{
				if (fout!=null) fin.close();			
			}catch(Exception ex1){
					;
			}
			try{
				if (fout!=null) fin.close();			
			}catch(Exception ex2){
					;
			}				
		}
		return su;
	}
	
	
	/**
	 * Method copy copys file from one location to the other location.
	 * 
	 * @param source
	 * @param target
	 * @param overwriteTarget
	 * @return boolean
	 */
	public static boolean copy(File source, File target, boolean overwriteTarget)
	{
		boolean su = false;		
		if (!overwriteTarget && target.exists()) return su;		
		FileInputStream fin = null;
		FileOutputStream fout = null;
		try{
			byte[] buffer = new byte[1024];
			fin = new FileInputStream(source);
			fout = new FileOutputStream(target);
			int len = 0;
			while ((len=fin.read(buffer))>0)
			{
				fout.write(buffer, 0, len);
			}
			su = true;				
		}catch(Exception ex){
			su = false;
		}finally{
			try{
				if (fout!=null) fin.close();			
			}catch(Exception ex1){
					;
			}
			try{
				if (fout!=null) fin.close();			
			}catch(Exception ex2){
					;
			}				
		}
		return su;
	}
	
	
	/**
	 * Method addToZip a list files to a existing Zip file 
	 * or create a new Zip file.
	 * @param zipFile
	 * @param filenames
	 * @throws IOException
	 */
	public static void addToZip(String zipFile, String[] filenames) 
	throws IOException
	{
		ZipOutputStream zipOut = null;
		File zf = new File(zipFile);
		if (zf.exists())
		{		
			File nzf = new File("." + zipFile + ".temp");
			nzf.deleteOnExit();
			move(zf, nzf, true);
			ZipFile zfile = new ZipFile(nzf);
			zipOut = new ZipOutputStream(new FileOutputStream(zipFile));
			java.util.Enumeration enum = zfile.entries();
			byte[] buffer = new byte[512];
			while (enum.hasMoreElements())
			{
				ZipEntry zentry = (ZipEntry)enum.nextElement();
				InputStream zein = zfile.getInputStream(zentry);			
				zipOut.putNextEntry(zentry);
				int len=0;
			    while((len=zein.read(buffer)) != -1) 
			    {
			       zipOut.write(buffer,0,len);
			    }
			    zein.close();
			    zipOut.closeEntry();
			}
			zfile.close();	
			zfile = null;
		}else{
			zipOut = new ZipOutputStream(new FileOutputStream(zipFile));
		}
		
		ArrayList al = new ArrayList();
		for(int i=0; i<filenames.length; i++)
		{
			if (filenames[i]==null) continue;
			File[] fs = listFilesFromDirectory(".", filenames[i]);
			for(int k=0; k<fs.length; k++) al.add(fs[k]);			
		}
		File[] files = new File[al.size()];
		al.toArray(files);					
		archive(zipOut, files, null);		
		zipOut.close();
	}
	
	public static String[] listFromDirectory(String directory, String mask)
	{
		return listFromDirectory(new File(directory), mask);		
	}
	
	public static String[] listFromDirectory(File directory, String mask)
	{
		if (mask.indexOf("*")!=-1 || mask.indexOf("?")!=-1)
		{
			return directory.list(new FileNameMaskFilter(mask));	
		}else{
			File f = new File(mask);
			return (f.exists())?(new String[]{mask}):(new String[0]);
		}	
	}
	
	public static File[] listFilesFromDirectory(String directory, String mask)
	{
		return listFilesFromDirectory(new File(directory), mask);		
	}
	
	public static File[] listFilesFromDirectory(File directory, String mask)
	{
		
		if (mask.indexOf("*")!=-1 || mask.indexOf("?")!=-1)
		{
			return directory.listFiles(new FileNameMaskFilter(mask));
		}else{
			File f = new File(mask);			
			return (f.exists())?(new File[]{new File(mask)}):(new File[0]);
		}			
	}
	
	public static void archive(String zipfile, String filename)
	{
		if (filename.indexOf("*")!=-1 || filename.indexOf("?")!=-1){			
			archive(zipfile, listFilesFromDirectory(".", filename), null);
		}else{
			archive(zipfile, new File(filename));
		}
	}
	
	public static void archive(String zipfile, String[] filenames)
	{
		ArrayList al = new ArrayList();
		for(int i=0; i<filenames.length; i++)
		{
			if (filenames[i]==null) continue;
			File[] fs = listFilesFromDirectory(".", filenames[i]);
			for(int k=0; k<fs.length; k++) al.add(fs[k]);			
		}
		File[] files = new File[al.size()];
		al.toArray(files);
		archive(zipfile, files, null);		
	}
	
	public static void archive(String zipfile, File file)
	{
		archive(zipfile, new File[]{file} , null);
	}
		
	public static void archive(String zipfile, File[] files, String parent)
	{
		ZipOutputStream zipOut = null;
		try{
			zipOut = new ZipOutputStream(new FileOutputStream(zipfile));			
			archive(zipOut, files, parent);
		}catch(Exception ex){
			ex.printStackTrace();
		}finally{
			if (zipOut != null) try{zipOut.close();}catch(Exception ex){;}
		}	
		
	}	
	
	protected static void archive(ZipOutputStream zipOut, File[] files, String parent)
	{
		parent = ( parent==null)?"":(parent+File.separator);
		FileInputStream fin = null;
		byte[] buffer = new byte[512];
		for(int i=0; i<files.length; i++)
		{				
			if (files[i].exists()==false) continue;
			String entryName = parent + files[i].getName();			
			//System.out.println(entryName);
			if (files[i].isFile())
			{
				try{					
					fin = new FileInputStream(files[i]);
					ZipEntry zentry = new ZipEntry(entryName);				
					zipOut.putNextEntry(zentry);
					int len=0;
				    while((len=fin.read(buffer)) != -1) 
				    {
				       zipOut.write(buffer,0,len);
				    }
				    zipOut.closeEntry();				    					
				}catch(IOException ioex){
					ioex.printStackTrace();
				}finally{
					if (fin != null) try{fin.close();}catch(Exception ex){;}					
				}
			}else{
				File[] fs = listFilesFromDirectory(files[i], "*.*");
				//for(int k=0; k<fs.length; k++) System.out.println(fs[k].getAbsolutePath());
				
				archive(zipOut, getChildrenOnly(files[i],fs), entryName);
			}
		}	
	}
	
	protected static File[] getChildrenOnly(File dir, File[] children)
	{
		ArrayList al = new ArrayList();
		File dirParent = dir.getParentFile();				
		for(int k=0; k<children.length; k++)
		{
			if (!children[k].equals(dirParent) && !children[k].equals(dir)) 
						al.add(children[k]);
		}		
		File[] files = new File[al.size()];
		al.toArray(files);
		return files;
	}
	
	public static boolean copy(File source, File target)
	{	
		return copy(source, target, true);
	}
	
	public static boolean move(File source, File target, boolean overwriteTarget)
	{
		boolean su = copy(source, target, overwriteTarget);
		if (su)
		{
			source.delete();
		}
		return su;
	}
	
	public static boolean move(File source, File target)
	{	
		return move(source, target, true);
	}
	
	public static void main(String[] args) throws Exception
	{
		int argsLength = args.length;
				
		CmdLineOptions cmdOptions = CmdLineOptions.getCmdLineOptions(args);		
		if (argsLength < 3 ) return;
		String[] list = new String[argsLength - 2];
		System.arraycopy(args, 2, list, 0, list.length);
		if (cmdOptions.getOption("z")!=null || cmdOptions.getOption("Z")!=null)
		{
			archive(args[1], list);
		}else if (cmdOptions.getOption("za")!=null || cmdOptions.getOption("Za")!=null)
		{
			addToZip(args[1], list);
		}
		
	}

}

final class FileNameMaskFilter implements java.io.FilenameFilter
{
    private String mask = null;
    /** Creates a new instance of FileMaskFilter */
    public FileNameMaskFilter(String mask1) 
    {
       
        StringBuffer buffer = new StringBuffer(128);
        for(int i=0; i<mask1.length(); i++){
            char c = mask1.charAt(i);
            if ( c=='*' ) buffer.append(".*");
            else if ( c=='?') buffer.append(".");
            else if ( c=='.') buffer.append("\\.");
            else buffer.append(c);
        }
        this.mask = buffer.toString();       
    }
    
    public boolean accept(java.io.File file, String str) 
    {        
        return Pattern.matches(mask, str);
    }
    
    public String toString()
    {
        return mask;
    }
    
}

