package com.gonetix.xml.parser;


import org.xml.sax.ContentHandler;

import java.io.*;
import java.util.*;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.FactoryConfigurationError;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

import com.gonetix.properties.RegExProperties;

/**
 * @author SLiu
 *
 */
public final class PropertyElementHandler 
extends AbstractChainableHandler
{

	private final String NAME = "name";
	private final String VALUE = "value";
	private final String FILE = "file";
	private final String XMLFILE = "xml-file";
	
	private String name = null;
	private String value = null;
	private String file = null;
	private String xmlfile = null;
	private Properties prop = null;
	private AbstractChainableHandler parent = null;
	StringBuffer buffer = null;
	/**
	 * Constructor for PropertyElementHandler.
	 */
	public PropertyElementHandler() {
		super();
		this.buffer = new StringBuffer(128); 
	}
	
	public void init(AbstractChainableHandler parent, Properties prop, Attributes attrs) 
	throws SAXException 
	{
		this.parent = parent;
		this.prop = prop;
		name = attrs.getValue(NAME);
		value = attrs.getValue(VALUE);
		file = attrs.getValue(FILE);
		xmlfile = attrs.getValue(XMLFILE);
		if (name!=null && value!=null)
		{
			prop.setProperty(name, value);
		}else if (file!=null && xmlfile==null){
			RegExProperties regExProp = new RegExProperties();
			try{
				regExProp.loadFromPath(file);
			}catch(IOException ex){				
				throw new SAXException(ex.toString());
			}
			prop.putAll(regExProp);			
		}else if (file==null && xmlfile!=null){
			PropertyTreeRoot proot = new PropertyTreeRoot();
			XMLPropertyParser xpParser = new XMLPropertyParser(proot);
			try{
				xpParser.parse(xmlfile);				
			}catch(Exception ex){
				ex.printStackTrace();
				throw new SAXException(ex.toString());				
			}
			prop.putAll(proot.convert());
		}else if (name==null){
			throw new SAXException("name attribute is required for property element.");
		}
	}
	
	public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
    throws SAXException 
    {      
       throw new SAXException("property element can not contain nested elements.");    
    }
	
	public void characters(char[] ch, int start, int length)
    throws SAXException 
    {       
        String pvalue = new String(ch, start, length);
        buffer.append(pvalue);                
    }
    
    
    public void endElement(String namespaceURI, String localName, String qName)
    throws SAXException 
    {
         String pvalue = buffer.toString().trim();
         if (pvalue.length()>0 && name!=null)
         {
         	prop.setProperty(name, pvalue);         	
         }         
         setContentHandler(this.parent);        
    }

	/**
	 * @see com.gonetix.xml.parser.AbstractChainableHandler#setContentHandler(org.xml.sax.ContentHandler)
	 */
	public void setContentHandler(ContentHandler handler) 
	{
		parent.setContentHandler(handler);
	}

}
