package com.gonetix.xml.parser;

import java.util.*;
import java.util.regex.*;
import com.gonetix.properties.RegExProperties;

/**
 * @author SLiu
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class PropertyTreeNode 
{
	
	protected PropertyTreeNode parent = null;
	protected String key = null;
	protected String value = null;
	protected Hashtable children = null;
	
	
	public PropertyTreeNode(String key, PropertyTreeNode parent)
	{
		if (key==null || key.length() == 0 ) 
				throw new InvalidPropertyKeyException("Invalid Property key [" + key + "]");
		this.key = key;
		this.parent = parent;
		children = new Hashtable();			
	}
	
	
	public void addChild(PropertyTreeNode child)
	{
		children.put(child.getKey(), child);
	}
	
	public PropertyTreeNode getChild(String hkey)
	{
		if (hkey==null || hkey.length() == 0 ) 
				throw new InvalidPropertyKeyException("Invalid Property key [" + hkey + "]");		
		PropertyTreeNode currNode = (PropertyTreeNode)children.get(hkey);				
		return currNode;
	}
	
	public PropertyTreeNode getChildByPath(String path)
	{
		if (path==null || path.length() == 0 ) 
				throw new InvalidPropertyKeyException("Invalid Property key path [" + path + "]");
		PropertyTreeNode currNode = this;		
		String[] paths = path.split("\\.");		
		for(int i=0; i<paths.length; i++)
		{
			if ( currNode==null) break;			
			currNode = currNode.getChild(paths[i]);
		}			
		return currNode;
	}
	
	protected Hashtable getChildren()
	{
		return children;	
	}
	
	public Iterator children()
	{
		return children.values().iterator();	
	}
	
	
	
	public String getProperty(String hkey)
	{		
		PropertyTreeNode currNode = getChild(hkey);		
		return (currNode==null)?null:currNode.getValue();	
	}
	
	public String getPropertyByPath(String hkey)
	{		
		PropertyTreeNode currNode = getChildByPath(hkey);		
		return (currNode==null)?null:currNode.getValue();	
	}
	
	/**
	 * Returns the period seperated hirarchical property key .
	 * @return String
	 */	
	public String getKeyPath()
	{
		if ( parent==null) return key;
		else{
			if (parent.getKeyPath()==null) return key;
			else{
				return (parent.getKeyPath()+ "." + key);
			}			
		}				
	}	
	
	/**
	 * Returns the value.
	 * @return String
	 */
	public String getValue() {
		return value;
	}	
	/**
	 * Sets the value.
	 * @param value The value to set
	 */
	public void setValue(String value) {
		this.value = value;
	}

	/**
	 * Returns the key.
	 * @return String
	 */
	public String getKey() {
		return key;
	}
	
	public Properties convert()
	{
		RegExProperties prop = new RegExProperties();
		if (this.getKeyPath()!=null && this.value!=null) prop.setProperty(this.getKeyPath(), value);
		Iterator iter = children();
		while (iter.hasNext())
		{
			PropertyTreeNode child = (PropertyTreeNode)iter.next();
			prop.putAll(child.convert());
		}	
		return prop;
	}
	
	/**
	 * Returns String representation of this property node.
	 * @return String
	 */		
	public String toString()
	{
		StringBuffer buffer = new StringBuffer("Property key [" + getKeyPath() + "] value [" + value + "]\n");
		Iterator iter = children();
		while (iter.hasNext())
		{
			PropertyTreeNode child = (PropertyTreeNode)iter.next();
			buffer.append(child.toString());
		}		
		return buffer.toString();		
	}

}
