package com.gonetix.xml.parser;

import org.xml.sax.ContentHandler;

import com.gonetix.xml.parser.AbstractChainableHandler;
import java.io.*;
import java.util.*;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.FactoryConfigurationError;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;


/**
 * @author SLiu
 *
 */
public class XMLPropertyHandler extends AbstractXMLHandler
implements IPropertyElementHandler 
{

	AbstractChainableHandler parent = null;
	PropertyTreeNode parentNode = null;	
	PropertyTreeNode currNode = null;
	StringBuffer buffer = null;
	
	/**
	 * Constructor for XMLPropertyHandler.
	 */
	public XMLPropertyHandler(AbstractChainableHandler parent, PropertyTreeNode parentNode) 
	{
		super();
		this.buffer = new StringBuffer(128); 
		this.parent = parent;
		this.parentNode = parentNode;
	}
	
	public void init(String elementName, Attributes attrs)
	{
		currNode = new PropertyTreeNode(elementName, parentNode);
		for(int i=0; i<attrs.getLength(); i++)
		{
			String key = attrs.getQName(i);
			String value = attrs.getValue(i);
			PropertyTreeNode child = new PropertyTreeNode(key, currNode);
			currNode.addChild(child);			
		}
		parentNode.addChild(currNode);		   
	}
	
	
	public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
    throws SAXException 
    {
      
        XMLPropertyHandler childHandler = new XMLPropertyHandler(this, currNode);
        childHandler.init(qName, atts);
        setContentHandler(childHandler);
    }
    
    public void characters(char[] ch, int start, int length)
    throws SAXException 
    {       
        String value = new String(ch, start, length);
        buffer.append(value);                
    }
    
    
    public void endElement(String namespaceURI, String localName, String qName)
    throws SAXException 
    {
         currNode.setValue(buffer.toString().trim());
         buffer.setLength(0);
         setContentHandler(this.parent);        
    }
    
	/**
	 * @see com.gonetix.xml.parser.AbstractChainableHandler#setContentHandler(org.xml.sax.ContentHandler)
	 */
	public void setContentHandler(ContentHandler handler) 
	{
		
		parent.setContentHandler(handler);
	}

}
