package com.gonetix.xml.parser.automation;

import com.gonetix.automation.*;
import com.gonetix.xml.parser.*;
import com.gonetix.util.*;

import java.io.*;
import java.util.*;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.FactoryConfigurationError;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;


/**
 * @author SLiu
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class XMLActionHandler extends AbstractXMLHandler {
	private static final String VALUE_ATTR = "value";
	private static final String NAME_ATTR = "name";
	private static final String FILE_ATTR = "file";
	private static final String XMLFILE_ATTR = "xmlfile";
	private final String PROPERTY_TAG = "property";
	private Task task = null;
	private ActionWrapper wrapper = null;
	private AbstractXMLHandler parent = null;
	private String tagName = null;
	private PropertyTreeRoot actionPropRoot = null;
	
	/**
	 * Constructor for XMLActionHandler.
	 */
	public XMLActionHandler(AbstractXMLHandler parent,Task task, ActionWrapper wrapper) 
	{
		this.task = task;
		this.parent = parent;
		this.wrapper = wrapper;
		actionPropRoot = new PropertyTreeRoot();
	}
	
	public void init(String tagName, Attributes attrs)
	throws SAXException
	{
		this.tagName = tagName;
		int count = attrs.getLength();
		for(int i=0; i<count; i++)
		{
			String name = attrs.getQName(i);
			String value = attrs.getValue(i);
			PropertyTreeNode node = new PropertyTreeNode(name, actionPropRoot);
       		node.setValue(value);
       		actionPropRoot.addChild(node);
		}		
		
	}
	
	private StringBuffer buffer = new StringBuffer(1024);	
	
	public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
    throws SAXException 
    {
       	if (task.getActionRegistry().isActionDefined(qName)){       		
       		ActionWrapper aw = wrapper.createChild(qName);       		
       		aw.setActionProperties(actionPropRoot.convert());
       		XMLActionHandler handler = new XMLActionHandler(this, task, aw);
       		this.setContentHandler(handler);
       	}else if (qName.equals(PROPERTY_TAG)){
       		String name = atts.getValue(NAME_ATTR);
       		String value = atts.getValue(VALUE_ATTR);
       		String file = atts.getValue(FILE_ATTR);
       		String xmlfile = atts.getValue(XMLFILE_ATTR);
       		if (name!=null && value!=null){
       			PropertyTreeNode node = new PropertyTreeNode(name, actionPropRoot);
       			node.setValue(value);
       			actionPropRoot.addChild(node);
       		}else if (file!=null){
       			TxtPropertyHandler txtHandler = new TxtPropertyHandler(actionPropRoot);       			
       			try{
	       			txtHandler.load(name, FileUtil.getAbsolutePath(this.getXmlBaseDir(), file));
	       		}catch(IOException ex){
	       			throw new SAXException(ex.toString());
	       		} 
       		}else if (xmlfile!=null){
       			PropertyTreeNode pNode = actionPropRoot;
       			if ( name!=null){
       				pNode = new PropertyTreeNode(name, actionPropRoot); 
       				actionPropRoot.addChild(pNode);      				
       			}
       			       			
       			XMLPropertyParser xmlParser = new XMLPropertyParser(pNode);       			
       			try{
	       			xmlParser.parse(this.getXmlBaseDir(), xmlfile);
	       		}catch(Exception ex){
	       			throw new SAXException(ex.toString());
	       		} 
       			
       		}       		
       }else{
       		throw new SAXException("Action " + qName + " is undefined.");       		
       	}       
    }
    
    public void characters(char[] ch, int start, int length)
    throws SAXException 
    {       
       buffer.append(new String(ch, start, length));    
    }
    
    public void endElement(String namespaceURI, String localName, String qName)
    throws SAXException 
    {
        if (qName.equals(tagName)) {            	
        	wrapper.setText(buffer.toString().trim());
        	wrapper.addProperties(this.actionPropRoot.convert());
        	setContentHandler(parent);
        }  
    }   
    
    public void setContentHandler(ContentHandler handler) 
	{
		
		parent.setContentHandler(handler);
	}

}
