package com.gonetix.xml.parser.report;


import org.xml.sax.ContentHandler;

import com.gonetix.xml.parser.*;
import com.gonetix.util.*;
import com.gonetix.report.*;
import java.io.*;
import java.util.*;


import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.FactoryConfigurationError;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;


public class ReportConfigReader extends AbstractChainableHandler
{

	/** Namespaces feature id (http://xml.org/sax/features/namespaces). */
    protected static final String NAMESPACES_FEATURE_ID = "http://xml.org/sax/features/namespaces";

    /** Namespace prefixes feature id (http://xml.org/sax/features/namespace-prefixes). */
    protected static final String NAMESPACE_PREFIXES_FEATURE_ID = "http://xml.org/sax/features/namespace-prefixes";

    /** Validation feature id (http://xml.org/sax/features/validation). */
    protected static final String VALIDATION_FEATURE_ID = "http://xml.org/sax/features/validation";

    /** Schema validation feature id (http://apache.org/xml/features/validation/schema). */
    protected static final String SCHEMA_VALIDATION_FEATURE_ID = "http://apache.org/xml/features/validation/schema";

    /** Schema full checking feature id (http://apache.org/xml/features/validation/schema-full-checking). */
    protected static final String SCHEMA_FULL_CHECKING_FEATURE_ID = "http://apache.org/xml/features/validation/schema-full-checking";

    /** Dynamic validation feature id (http://apache.org/xml/features/validation/dynamic). */
    protected static final String DYNAMIC_VALIDATION_FEATURE_ID = "http://apache.org/xml/features/validation/dynamic";

	
	private String rootTagName = "reports";
	private String taskTagName = "task";
	private String propertyTagName = "property";
	
	private Properties globalProps;
    private ArrayList taskList;
    private XMLReader xmlReader;
    
    public ReportConfigReader() 
    {
        globalProps = new Properties();
        taskList = new ArrayList();
    }
   
    public void parse(String file) throws Exception
    {
        parse(new InputSource(file));
    }
    
    public void parse(Reader reader) throws Exception
    {
        parse(new InputSource(reader));
    }
    public void parse(InputStream byteStream) throws Exception
    {
        parse(new InputSource(byteStream));
    }
    
    private void parse(InputSource source) throws Exception
    {                 
        SAXParserFactory spfactory = SAXParserFactory.newInstance();
        spfactory.setValidating(false);        
        SAXParser saxParser = spfactory.newSAXParser();
        xmlReader = saxParser.getXMLReader(); 
        
        xmlReader.setFeature(NAMESPACES_FEATURE_ID, true);
        xmlReader.setFeature(NAMESPACE_PREFIXES_FEATURE_ID, true);
        xmlReader.setFeature(VALIDATION_FEATURE_ID, true);
        xmlReader.setFeature(SCHEMA_VALIDATION_FEATURE_ID, true);
        xmlReader.setFeature(DYNAMIC_VALIDATION_FEATURE_ID, true);
                 
        xmlReader.setContentHandler(this);
        xmlReader.setEntityResolver(this);
        xmlReader.setErrorHandler(this);
        xmlReader.parse(source);
    }
    
    public void addTask(ReportTask task)
    {
        taskList.add(task);
    }  
    
    
    //--------------------------------------------------------------------------
    //XML SAX methods
    //--------------------------------------------------------------------------
    boolean started = false;
    StringBuffer value = new StringBuffer(100);    
    String key = null;
    
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
    throws SAXException 
    {
       value.setLength(0);
       if ( qName.equals(taskTagName)){
           ReportTask task = new ReportTask();
           task.initProperties(globalProps);
           this.addTask(task);
           ReportTaskHandler thandler = new ReportTaskHandler(this, task);
           thandler.init(qName, atts); 
           this.setContentHandler(thandler);
           started = true;
       }else if (qName.equals(propertyTagName)){
           PropertyElementHandler peHandler = new PropertyElementHandler();
           peHandler.init(this, globalProps, atts);
           this.setContentHandler(peHandler);           
       }else if (!qName.equals(rootTagName)){
           key = qName;
           started = true;
       }else{
           started = false;
       }
        
    }
    
    public void characters(char[] ch, int start, int length)
    throws SAXException 
    {       
        String content = (new String(ch, start, length)); 
        if(content!=null && length>0){            
         content=StringUtil.decodeEntity(content, "&#60;|&lt;", "<");         
         content=StringUtil.decodeEntity(content, "&#62;|&gt;", ">");        
         content=StringUtil.decodeEntity(content, "&#47;", "/");
        }
        if( started)value.append(content);
    }
    
    public void endElement(String namespaceURI, String localName, String qName)
    throws SAXException 
    {
         
        if ( key!=null && value.length()>0 && started){
            String pvalue = value.toString();           
            pvalue = StringUtil.strip(pvalue);
            globalProps.setProperty(key, pvalue);            
        }
        started = false;
        key = null;
        value.setLength(0);
    }
    
    public String getProperty(String key)
    {
        return globalProps.getProperty(key);
    }
    
    public void setProperty(String key, String value)
    {
        globalProps.setProperty(key, value);
    }
    
    public void addAllProperties(Properties prop)
    {
        this.globalProps.putAll(prop);
    }
    
    public Properties getGlobalProperties()
    {
        return this.globalProps;
    }
    
    public ReportTask getTask(int index)
    {
        return (ReportTask)taskList.get(index);
    }
    
    
    public ReportTask getTask(String taskId)
    {
        ReportTask[] tasks = getAllTask();
        for (int i=0; i<tasks.length; i++){
            String id = tasks[i].getProperty("id");
            if (id !=null && id.equals(taskId)) return tasks[i];            
        }
        return null;
    }
    
    public ReportTask[] getAllTask()
    {
        ReportTask[] tasks = new ReportTask[taskList.size()];
        taskList.toArray(tasks);
        return tasks;
    }
	/**
	 * Returns the rootTagName.
	 * @return String
	 */
	public String getRootTagName() {
		return rootTagName;
	}

	/**
	 * Returns the taskTagName.
	 * @return String
	 */
	public String getTaskTagName() {
		return taskTagName;
	}

	/**
	 * Sets the rootTagName.
	 * @param rootTagName The rootTagName to set
	 */
	public void setRootTagName(String rootTagName) {
		this.rootTagName = rootTagName;
	}

	/**
	 * Sets the taskTagName.
	 * @param taskTagName The taskTagName to set
	 */
	public void setTaskTagName(String taskTagName) {
		this.taskTagName = taskTagName;
	}

	/**
	 * @see com.gonetix.xml.parser.AbstractChainableHandler#setContentHandler(org.xml.sax.ContentHandler)
	 */
	public void setContentHandler(ContentHandler handler) {
		xmlReader.setContentHandler(handler);
	}

}
