/*
 * TaskHandler.java
 *
 * Created on August 13, 2002, 4:53 PM
 */

package com.gonetix.xml.parser.report;


import java.io.*;
import java.util.*;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.FactoryConfigurationError;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

import com.gonetix.framework.dbengine.*;
import com.gonetix.framework.dbpublish.*;
import com.gonetix.framework.util.*;
import com.gonetix.framework.xsltEngine.*;
import com.gonetix.report.ReportTask;
import com.gonetix.xml.parser.*;

/**
 *
 * @author  SLiu
 */
public class ReportTaskHandler extends AbstractChainableHandler
{
    private String propertyTagName = "property";
    private String statementTagName = "statement";
    private String parameterTagName = "param";
    private String typeTagName = "type";
    
    protected ReportConfigReader parent;
    protected String tag;
    protected ReportTask task;
    /** Creates a new instance of TaskHandler */
    public ReportTaskHandler(ReportConfigReader parent, ReportTask task) {
        this.parent = parent;
        this.task = task;
    }
    
    public void init(String qname, Attributes atts)
    {
        this.tag = qname;
        int num = atts.getLength(); 
        for(int i=0; i<num; i++){
            task.setProperty(atts.getQName(i), atts.getValue(i));
         }
    }
    
    boolean started = false;    
    String key = null;
    StringBuffer value = new StringBuffer(100);
    String stmtType = null;
    String stmtId=null;
    
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
    throws SAXException 
    {
        value.setLength(0);
        if(qName.equals(statementTagName)){
            stmtType = atts.getValue(typeTagName);
            stmtId = atts.getValue("id"); 
            started = true;
        }else if (qName.equals(propertyTagName)){
           PropertyElementHandler peHandler = new PropertyElementHandler();
           peHandler.init(this, task.getProperties(), atts);
           this.setContentHandler(peHandler);           
       }else if (qName.equals(parameterTagName)){
            started = true;
            String type = atts.getValue(typeTagName);
            String foramt = atts.getValue("format");
            String paramValue = atts.getValue("value");
            String cmdIndex = atts.getValue("cmdIndex");            
            try{
                SQLParameter param;
                if (paramValue!=null && paramValue.equals("CMDLINE")){                    
                    param = new SQLParameter(type, paramValue, foramt);                     
                    param.setCmdIndex(cmdIndex);                    
                }else{
                    param = new SQLParameter(type, paramValue, foramt);
                }
                task.addParam(param);               
            }catch(Exception e){
                throw new SAXException("FATAL: " + e.toString());
            }  
        }else{            
            key = qName;
            started = true;
        }
    }
    
    public void characters(char[] ch, int start, int length)
    throws SAXException 
    {       
        String content = (new String(ch, start, length)); 
        if(content!=null && length>0){            
         content=StringUtil.decodeEntity(content, "&#60;|&lt;", "<");         
         content=StringUtil.decodeEntity(content, "&#62;|&lt;", "<");        
         content=StringUtil.decodeEntity(content, "&#47;", "/");
        }
        if( started) value.append(content);
    }
    
    public void endElement(String namespaceURI, String localName, String qName)
    throws SAXException 
    {
        String pvalue = value.toString();
        value.setLength(0);        
        if ( qName.equals(tag)){            
            parent.setContentHandler(parent);          
        }else if (qName.equals(statementTagName)){
            try{
               task.setSQLStatement(new SQLStatement(stmtId, stmtType, pvalue));
            }catch(Exception e){
               throw new SAXException("FATAL: " + e.toString());
            }
            stmtType = null;
            stmtId=null;
        }else if (qName.equals(parameterTagName)){
            ;
        }else{            
            if(pvalue.length()>0) {                
                task.setProperty(key, pvalue);   
            }
        }            
        key = null ;        
        started = false;
    }     
	/**
	 * @see com.gonetix.xml.parser.AbstractChainableHandler#setContentHandler(org.xml.sax.ContentHandler)
	 */
	public void setContentHandler(ContentHandler handler) {
		parent.setContentHandler(handler); 
	}

}
