package com.gonetix.xml.parser.sql;

import org.xml.sax.ContentHandler;
import com.gonetix.framework.dbengine.SQLStatement;
import com.gonetix.util.*;
import com.gonetix.xml.parser.AbstractChainableHandler;
import java.io.*;
import java.util.*;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.FactoryConfigurationError;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

/**
 * @author SLiu
 * 
 */
public class SQLStatementHandler extends AbstractChainableHandler {

	
	
	private final String HREF = "href";
    private final String SRC = "src";
    private final String TYPE = "type";   
    private final String ID = "id";  
    
    private AbstractChainableHandler parent;
    private SQLStatement sqlStmt;
    private String tagName;
    private StringBuffer value;
    private String xmlBaseDir = ".";
    
    /** Creates a new instance of SQLStatementHandler */
    public SQLStatementHandler(AbstractChainableHandler parent, SQLStatement sqlStmt) {
        super();
        this.parent = parent;
        this.sqlStmt = sqlStmt;
        value = new StringBuffer(100); 
    }
    
    public void init(String tagName, Attributes atts)
    {
        this.tagName = tagName;
        String value = atts.getValue(HREF);
        if ( value != null ) sqlStmt.setHref(value);
        value = atts.getValue(SRC);
        if ( value != null ) sqlStmt.setSrc(FileUtil.getAbsolutePath(this.xmlBaseDir, value));
        value = atts.getValue(TYPE);
        if ( value != null ) sqlStmt.setType(value); 
        value = atts.getValue(ID);
        if ( value != null ) sqlStmt.setId(value);        
    }

    
    public void characters(char[] ch, int start, int length)
    throws SAXException 
    {       
        String content = (new String(ch, start, length));
        value.append(content);
    }
    
    public void endElement(String namespaceURI, String localName, String qName)
    throws SAXException 
    {
        if (qName.equals(tagName)){
            String stmt = value.toString(); 
            stmt = stmt.trim();
            if(stmt.length()>0){ 
                sqlStmt.setSqlText(stmt);
            }
            setContentHandler(parent);
            value = null;
        }        
    }
	/**
	 * @see com.gonetix.xml.parser.AbstractAbstractChainableHandler#setContentHandler(org.xml.sax.ContentHandler)
	 */
	public void setContentHandler(ContentHandler handler) 
	{
		parent.setContentHandler(handler);
	}

	/**
	 * Sets the xmlBaseDir.
	 * @param xmlBaseDir The xmlBaseDir to set
	 */
	public void setXmlBaseDir(String xmlBaseDir) {
		this.xmlBaseDir = xmlBaseDir;
	}

}
