package com.gonetix.xml.parser.sql;

import org.xml.sax.ContentHandler;
import com.gonetix.framework.dbengine.SQLStatement;
import com.gonetix.xml.parser.*;
import com.gonetix.util.*;
import java.io.*;
import java.util.*;


import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.FactoryConfigurationError;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;


/**
 * @author SLiu
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class SQLXMLReader extends AbstractXMLHandler
{	
	public static final String ROOT_TAG = "sql";
	public static final String STATEMENT_TAG = "statement";
	public static final String PROPERTY_TAG = "property";
	public static final String VALUE_ATTR = "value";
	public static final String NAME_ATTR = "name";
	public static final String FILE_ATTR = "file";
	public static final String XMLFILE_ATTR = "xmlfile";
	
	private final String ID = "id";
	protected Hashtable stmtHolder;
	protected PropertyTreeRoot propRoot = null;
	private String rootTag = ROOT_TAG;
	
	public SQLXMLReader()
    {
        super();
        stmtHolder = new Hashtable(128);
        propRoot = new PropertyTreeRoot();
    }
    
    
    
    //--------------------------------------------------------------------------
    //XML SAX methods
    //--------------------------------------------------------------------------
        
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
    throws SAXException 
    {
       if (qName.equals(rootTag)){     	
       
       }else if (qName.equals(STATEMENT_TAG)){
       		SQLStatement stmt = new SQLStatement();
       		SQLStatementHandler stmtHandler = new SQLStatementHandler(this, stmt);
       		stmtHandler.setXmlBaseDir(this.xml_dir);
       		stmtHandler.init(qName, atts);
       		try{
       			stmtHolder.put(stmt.getId(), stmt);
       		}catch(Exception ex){
       			throw new SAXException("id attribute is required for statement element.\n" + ex.toString());
       		}      		
       		this.setContentHandler(stmtHandler);
       }else if (qName.equals(PROPERTY_TAG)){
       		String name = atts.getValue(NAME_ATTR);
       		String value = atts.getValue(VALUE_ATTR);
       		String file = atts.getValue(FILE_ATTR);
       		String xmlfile = atts.getValue(XMLFILE_ATTR);
       		if (name!=null && value!=null){
       			PropertyTreeNode node = new PropertyTreeNode(name, propRoot);
       			node.setValue(value);
       			propRoot.addChild(node);
       		}else if (file!=null){
       			TxtPropertyHandler txtHandler = new TxtPropertyHandler(propRoot);       			
       			try{
	       			txtHandler.load(name, FileUtil.getAbsolutePath(this.getXmlBaseDir(), file));
	       		}catch(IOException ex){
	       			throw new SAXException(ex.toString());
	       		} 
       		}else if (xmlfile!=null){
       			PropertyTreeNode pNode = propRoot;
       			if ( name!=null){
       				pNode = new PropertyTreeNode(name, propRoot); 
       				propRoot.addChild(pNode);      				
       			}
       			       			
       			XMLPropertyParser xmlParser = new XMLPropertyParser(pNode);       			
       			try{
	       			xmlParser.parse(this.getXmlBaseDir(), xmlfile);
	       		}catch(Exception ex){
	       			throw new SAXException(ex.toString());
	       		} 
       			
       		}       		
       }else{
       		XMLPropertyHandler childHandler = new XMLPropertyHandler(this, propRoot);
        	childHandler.init(qName, atts);
        	setContentHandler(childHandler);       	
       }
        
    }
    
    
    public PropertyTreeNode  getPropertyTree()
    {
    	return this.propRoot;	
    }
    
    public String getProperty(String key)
    {
    	return propRoot.getPropertyByPath(key);
    }
    
    public String getSQLStatementText(String id)
    {
    	SQLStatement stmt = getSQLStatement(id);
    	return (stmt==null)?null:stmt.getSqlText();
    }
    
    public SQLStatement getSQLStatement(String id)
    {
    	return (SQLStatement)this.stmtHolder.get(id);	
    }
    
    public Iterator getSQLStatements(String id)
    {
    	return this.stmtHolder.values().iterator();	
    }   

    
	/**
	 * Returns the rootTag.
	 * @return String
	 */
	public String getRootTag() {
		return rootTag;
	}

	/**
	 * Sets the rootTag.
	 * @param rootTag The rootTag to set
	 */
	public void setRootTag(String rootTag) {
		this.rootTag = rootTag;
	}

}
